#include <signal.h>
#include <unistd.h>
#include <time.h>
#include "keyboard.h"

static char flg_alarm_initted;
static volatile int flg_kill_initted;
static int flg_sleep_kill_initted;

static time_t t_timeout;

void
sleep_kill_start (void)
{       /*  ,     */
  flg_sleep_kill_initted++;
}

void
raise_old_pid (int pid)
{
  kill (pid, SIGUSR1);
}

static void
usr1_handl (int ns)
{
  time_t t_timeout_cur;
  if (t_timeout) {
	time (&t_timeout_cur);
	if ((t_timeout_cur - t_timeout) > ALARM_KILL)
	  stoped_new_pid ();
  }
}

void
set_sig_usr1_already_login (void)
{
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  struct sigaction sa;
#endif
  signal (SIGUSR1, usr1_handl);
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  sigaction (SIGUSR1, (struct sigaction *) 0, &sa);
#ifdef SA_RESTART
  sa.sa_flags &= ~SA_RESTART;
#endif
#ifdef SA_INTERRUPT
  sa.sa_flags |= SA_INTERRUPT;
#endif
  sigaction (SIGUSR1, &sa, (struct sigaction *) 0);
#endif
}

static void
raise_kill (void)
{       /*  */
  if (flg_kill_initted != 0 && flg_sleep_kill_initted == 0) {
	/*     */
	scanclose ();
	signal (SIGTERM, SIG_DFL);
	signal (SIGHUP, SIG_DFL);
	kill (getpid (), flg_kill_initted);
  }
}

void
sleep_kill_stop (void)
{
  flg_sleep_kill_initted--; /* ,   ,  */
  raise_kill ();            /*  () ? */
}

int
if_killed (void)
{
  return flg_kill_initted;
}

static void
fun_kill (int ns)
{
  flg_kill_initted = ns;                /*   */
  if (ns == SIGTERM)
	stop_term (flg_alarm_initted);
  else
	stop_hup ();
  raise_kill ();                        /*   */
}

void
handler_alarm (void)
{
  if (flg_alarm_initted < 0) {
	warn_timeout (0);
	kill (getpid (), SIGTERM);
  } else {
	flg_alarm_initted = -flg_alarm_initted;
	warn_timeout (1);
  }
}

static void
handler_alarm_out (int ns)
{
  stop_idle ();
  signal (SIGTERM, SIG_DFL);
  kill (getpid (), SIGTERM);
}

static void
set_interrupt_all (void)
{
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  struct sigaction sa;
#endif
  signal (SIGTERM, fun_kill);
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  sigaction (SIGTERM, (struct sigaction *) 0, &sa);
#ifdef SA_RESTART
  sa.sa_flags &= ~SA_RESTART;
#endif
#ifdef SA_INTERRUPT
  sa.sa_flags |= SA_INTERRUPT;
#endif
  sigaction (SIGTERM, &sa, (struct sigaction *) 0);
#endif
  signal (SIGHUP, fun_kill);
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  sigaction (SIGHUP, (struct sigaction *) 0, &sa);
#ifdef SA_RESTART
  sa.sa_flags &= ~SA_RESTART;
#endif
#ifdef SA_INTERRUPT
  sa.sa_flags |= SA_INTERRUPT;
#endif
  sigaction (SIGHUP, &sa, (struct sigaction *) 0);
#endif
  set_sig_usr1_already_login ();
}

void
set_alarm_out (void)
{
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  struct sigaction sa;
#endif
  signal (SIGALRM, handler_alarm_out);
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  sigaction (SIGALRM, (struct sigaction *) 0, &sa);
#ifdef SA_RESTART
  sa.sa_flags &= ~SA_RESTART;
#endif
#ifdef SA_INTERRUPT
  sa.sa_flags |= SA_INTERRUPT;
#endif
  sigaction (SIGALRM, &sa, (struct sigaction *) 0);
#endif
  alarm (90);                                   /* 90 sec - retrain */
  set_interrupt_all ();
}

void
unset_alarm_out (void)
{
  alarm (0);
  signal (SIGALRM, SIG_IGN);
  set_hup_term ();
  signal (SIGUSR1, SIG_IGN);
}

void
set_hup_term (void)
{
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  struct sigaction sa;
#endif
  signal (SIGTERM, fun_kill);
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  sigaction (SIGTERM, (struct sigaction *) 0, &sa);
#ifdef SA_RESTART
  sa.sa_flags |= SA_RESTART;
#endif
#ifdef SA_INTERRUPT
  sa.sa_flags |= SA_INTERRUPT;
#endif
  sigaction (SIGTERM, &sa, (struct sigaction *) 0);
#endif
  signal (SIGHUP, fun_kill);
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  sigaction (SIGHUP, (struct sigaction *) 0, &sa);
#ifdef SA_RESTART
  sa.sa_flags |= SA_RESTART;
#endif
#ifdef SA_INTERRUPT
  sa.sa_flags |= SA_INTERRUPT;
#endif
  sigaction (SIGHUP, &sa, (struct sigaction *) 0);
#endif
}

void
set_alarm (void)
{
  if (!flg_alarm_initted) {
	flg_alarm_initted++;
	time (&t_timeout);
	set_interrupt_all ();
  }
}

void
unset_alarm (void)
{
  if (flg_alarm_initted) {
	signal (SIGUSR1, SIG_IGN);
	flg_alarm_initted = 0;
	t_timeout = 0;
	set_hup_term ();
  }
}
