/*
 *  Base64 encoder. Stolen from busybox and optimize by
 *  (C) 2003 Vladimir Oleynik <dzo@simtreas.ru>
 *
 *  based on the function base64_encode from http.c in wget v1.6
 *  Copyright (C) 1995, 1996, 1997, 1998, 2000 Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>

/* Conversion table.  for base 64 */
static const char tbl_base64[] = {
  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
  'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
  'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
  'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
  'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
  'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
  'w', 'x', 'y', 'z', '0', '1', '2', '3',
  '4', '5', '6', '7', '8', '9', '+', '/',
};


static int
b64encode (const char *from, const char *store, const int length)
{
  int i;
  unsigned char *p = (unsigned char *) store;
  const unsigned char *s = (const unsigned char *) from;

  /* Transform the 3x8 bits to 4x6 bits, as required by base64.  */
  for (i = 0; i < length; i += 3) {
	*p++ = tbl_base64[s[0] >> 2];
	*p++ = tbl_base64[((s[0] & 3) << 4) + (s[1] >> 4)];
	*p++ = tbl_base64[((s[1] & 0xf) << 2) + (s[2] >> 6)];
	*p++ = tbl_base64[s[2] & 0x3f];
	s += 3;
  }
  /* Pad the result if necessary...  */
  if (i == length + 1) {
	*(p - 1) = '=';
  } else if (i == length + 2) {
	*(p - 1) = *(p - 2) = '=';
  }
  /* ...and CR-terminate it.  */
  *p = '\n';

  return i / 3 * 4 + 1;
}

#define SRC_BUF_SIZE 54                 /* 4/3 set output line 72 size */
#if (SRC_BUF_SIZE/3)*3 != SRC_BUF_SIZE
# error SRC_BUF_SIZE *MUST* be a multiple of 3
#endif
#define DST_BUF_SIZE  4 * (SRC_BUF_SIZE / 3)

int
main (int argc, char **argv)
{
  char src_buf[SRC_BUF_SIZE];
  char dst_buf[DST_BUF_SIZE + 1];       /* + '\n' */
  FILE *src_stream;
  size_t size;
  const char *pn, *s;

  pn = argv[0];
  s = strrchr (pn, '/');
  if (s)
	pn = s + 1;

  if (argc != 2) {
	fprintf (stderr, "Usage: %s file\n", pn);
	return 1;
  }

  src_stream = freopen (argv[1], "r", stdin);
  if (src_stream == NULL) {
	fprintf (stderr, "%s: can`t open source file %s: %m\n", pn, argv[1]);
	return 2;
  }
  while ((size = fread (src_buf, 1, SRC_BUF_SIZE, src_stream)) > 0) {
	if (size != SRC_BUF_SIZE) {
	  /* pad with 0s so we can just encode extra bits */
	  memset (&src_buf[size], 0, SRC_BUF_SIZE - size);
	}
	/* Encode the buffer we just read in */
	size = b64encode (src_buf, dst_buf, size);
	if (fwrite (dst_buf, 1, size, stdout) != size) {
	  fprintf (stderr, "%s: couldnt finish writing: %m\n", pn);
	  return 3;
	}
  }
  return 0;
}
