/* This program is derived from 4.3 BSD software and is
   subject to the copyright notice below.
*/
/*
 * Copyright (c) 1980, 1987, 1988 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
 * login [ name ]
 * login -h hostname    (for telnetd, etc.)
 * login -f name        (for pre-authenticated login: datakit, xterm, etc.)
 */

#ifndef BB_FEATURE_DZO
#define LOGIN_NAME "dionis"
#define TLG_FEATURE_SHADOWPASSWDS 1
#include <shadow.h>
static void errorMsg(const char *s, ...);
static void fatalError(const char *s, ...);
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
static void *xmalloc(size_t size);
static void usage(const char *usage);
#else
#include "internal.h"
#endif

#include <sys/param.h>
#include <stdio.h>
#include <ctype.h>
#include <getopt.h>
#include <memory.h>
#include <time.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/file.h>
#include <termios.h>
#define index strchr
#define rindex strrchr
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <signal.h>
#include <errno.h>
#include <grp.h>
#include <pwd.h>
#include <setjmp.h>
#include <sys/syslog.h>
#include <sys/sysmacros.h>
#include <netdb.h>
#include <stdarg.h>

#if defined (__GNU_LIBRARY__) && __GNU_LIBRARY__ > 1
#include <crypt.h>
#endif

#ifdef linux
#include <sys/sysmacros.h>
#include <linux/major.h>
#define DO_PS_FIDDLING
static void initproctitle (int argc, char **argv);
static void setproctitle (const char *prog, const char *txt);
#endif


#include <lastlog.h>

#define SLEEP_EXIT_TIMEOUT 5

#ifndef __STDC__
# error "we need an ANSI compiler"
#endif

/* The paths for some of these are wrong in /usr/include/paths.h, but we
   re-define them here. */

#undef _PATH_DEFPATH
#undef _PATH_DEFPATH_ROOT

#ifndef SBINDIR
#define SBINDIR                 "/sbin"
#endif

#ifndef USRSBINDIR
#define USRSBINDIR              "/usr/sbin"
#endif

#ifndef UT_NAMESIZE
#define UT_NAMESIZE             8
#endif

#define _PATH_BSHELL            "/bin/sh"
#define _PATH_TTY               "/dev/tty"
#define SECURETTY               "/etc/securetty"

/* no more . in DEFPATH */
#define _PATH_DEFPATH           "/usr/local/bin:/bin:/usr/bin"
#define _PATH_DEFPATH_ROOT      SBINDIR ":/bin:" USRSBINDIR ":/usr/bin"
#define _PATH_HUSHLOGIN         ".hushlogin"

#define P_(s) ()
static void opentty P_((const char *tty));
static void getloginname P_((void));
static void timedout P_((void));
static int rootterm P_((char *ttyn));
static void badlogin P_((const char *name));
static void sleepexit P_((int eval));
#undef P_

#ifdef USE_TTY_GROUP
#  define TTY_MODE 0620
#else
#  define TTY_MODE 0600
#endif

#define TTYGRPNAME      "tty"           /* name of group to own ttys */
/**#  define TTYGRPNAME      "other" **/

#ifndef MAXPATHLEN
#  define MAXPATHLEN 1024
#endif

/*
 * This bounds the time given to login.  Not a define so it can
 * be patched on machines where it's too small.
 */
static int     timeout = 60;

static struct  passwd *pwd;
static int     failures = 1;
static char    *hostname, *username, *tty;
static struct  hostent hostaddress;
static char    thishost[100];

/* provided by Linus Torvalds 16-Feb-93 */
static void
opentty(const char * tty)
{
    int i;
    int fd = open(tty, O_RDWR);

    for (i = 0 ; i < fd ; i++)
      close(i);
    for (i = 0 ; i < 3 ; i++)
      dup2(fd, i);
    if (fd >= 3)
      close(fd);
}

/* true if the filedescriptor fd is a console tty, very Linux specific */
/*static int
consoletty(int fd)
{
    struct stat stb;

    if ((fstat(fd, &stb) >= 0)
	&& (major(stb.st_rdev) == TTY_MAJOR)
	&& (minor(stb.st_rdev) < 64)) {
	return 1;
    }
    return 0;
} */


int
#ifndef BB_FEATURE_DZO
main
#else
login_main
#endif
	    (int argc, char **argv)
{
    extern int errno, optind;
    extern char *optarg, **environ;
    struct group *gr;
    register int ch;
    register char *p;
    int ask, fflag, hflag, pflag, cnt, errsv;
    int  passwd_req;
    char *domain, *ttyn;
    char tbuf[MAXPATHLEN + 2], tname[sizeof(_PATH_TTY) + 10];
    char *ttyname();
    time_t time();
    char *termenv;
    char * childArgv[10];
    char * buff;
    int childArgc = 0;

    char *salt, *pp;

    signal(SIGALRM, timedout);
    alarm((unsigned int)timeout);
    signal(SIGQUIT, SIG_IGN);
    signal(SIGINT, SIG_IGN);

    setpriority(PRIO_PROCESS, 0, 0);
#ifdef DO_PS_FIDDLING
    initproctitle(argc, argv);
#endif

    /*
     * -p is used by getty to tell login not to destroy the environment
     * -f is used to skip a second login authentication
     * -h is used by other servers to pass the name of the remote
     */
    gethostname(tbuf, sizeof(tbuf));
    strncpy(thishost, tbuf, sizeof(thishost)-1);
    thishost[sizeof(thishost)-1] = 0;
    domain = index(tbuf, '.');

    username = tty = hostname = NULL;
    ask = fflag = hflag = pflag = 0;
    passwd_req = 1;

    while ((ch = getopt(argc, argv, "fh:p")) != EOF)
      switch (ch) {
	case 'f':
	  fflag = 1;
	  break;

	case 'h':
	  if (getuid()) {
	      fatalError("login: -h for super-user only.");
	  }
	  hflag = 1;
	  if (domain && (p = index(optarg, '.')) &&
	      strcasecmp(p, domain) == 0)
	    *p = 0;
	  hostname = strdup(optarg);    /* strdup: Ambrose C. Li */
	  {
	      struct hostent *he = gethostbyname(hostname);
	      if (he) {
		  memcpy(&hostaddress, he, sizeof(hostaddress));
	      } else {
		  memset(&hostaddress, 0, sizeof(hostaddress));
	      }
	  }
	  break;

	case 'p':
	  pflag = 1;
	  break;

	case '?':
	default:
	  usage("login [-fp] [username]");
      }
    argc -= optind;
    argv += optind;
    if (*argv)
	username = strdup(*argv);
#ifdef BB_FEATURE_DZO
     else
	ask = 1;
#endif
    for (cnt = getdtablesize(); cnt > 2; cnt--)
      close(cnt);

    ttyn = ttyname(0);
    if (ttyn == NULL || *ttyn == '\0') {
	/* no snprintf required - see definition of tname */
	sprintf(tname, "%s??", _PATH_TTY);
	ttyn = tname;
    }
#ifndef BB_FEATURE_DZO
    cnt = strlen(_PATH_TTY); /* hack for console */
    if(strncmp(ttyn, _PATH_TTY, cnt) == 0 && ttyn[cnt]>='0' && ttyn[cnt]<='9') {
	if(username == NULL)
		ask = 1;
	}
     else {
	if(username == NULL)
	    username = strdup(LOGIN_NAME);
	}
#endif
    setpgrp();

    {
	struct termios tt, ttt;

	tcgetattr(0, &tt);
	ttt = tt;
	ttt.c_cflag &= ~HUPCL;

	if((chown(ttyn, 0, 0) == 0) && (chmod(ttyn, 0622) == 0)) {
	    tcsetattr(0,TCSAFLUSH,&ttt);
	    signal(SIGHUP, SIG_IGN); /* so vhangup() wont kill us */
	    vhangup();
	    signal(SIGHUP, SIG_DFL);
	}

	setsid();

	/* re-open stdin,stdout,stderr after vhangup() closed them */
	/* if it did, after 0.99.5 it doesn't! */
	opentty(ttyn);
	tcsetattr(0,TCSAFLUSH,&tt);
    }

    if ((tty = rindex(ttyn, '/')))
      ++tty;
    else
      tty = ttyn;

    openlog("login", LOG_ODELAY, LOG_AUTHPRIV);

    for (cnt = 0;; ask = 1) {

	if (ask) {
	    fflag = 0;
	    getloginname();
	}
	/* (void)strcpy(tbuf, username); why was this here? */
	if ((pwd = getpwnam(username))) {
#ifdef TLG_FEATURE_SHADOWPASSWDS
	    struct spwd *spwd;

	    if (strcmp(pwd->pw_passwd, "x") == 0
					|| strcmp(pwd->pw_passwd, "*") == 0) {
		spwd = getspnam(username);
		if (spwd)
		    pwd->pw_passwd = spwd->sp_pwdp;
		}
#endif
	    salt = pwd->pw_passwd;
	    }
	 else
	  salt = "xx";

	if (pwd)
	    initgroups(username, pwd->pw_gid);

	/*
	 * Disallow automatic login to root; if not invoked by
	 * root, disallow if the uid's differ.
	 */
	if (fflag && pwd) {
	    int uid = getuid();

	    passwd_req = pwd->pw_uid == 0 ||
	      (uid && uid != pwd->pw_uid);
	}

	/*
	 * If trying to log in as root, but with insecure terminal,
	 * refuse the login attempt.
	 */
	if (pwd && pwd->pw_uid == 0 && !rootterm(tty)) {
	    errorMsg("%s login refused on this terminal.", pwd->pw_name);

	    if (hostname)
	      syslog(LOG_NOTICE, "LOGIN %s REFUSED FROM %s ON TTY %s",
		     pwd->pw_name, hostname, tty);
	    else
	      syslog(LOG_NOTICE, "LOGIN %s REFUSED ON TTY %s",
			    pwd->pw_name, tty);
	    continue;
	}

	/*
	 * If no pre-authentication and a password exists
	 * for this user, prompt for one and verify it.
	 */
	if (!passwd_req || (pwd && !*pwd->pw_passwd))
	  break;

	setpriority(PRIO_PROCESS, 0, -4);
	pp = getpass("Password: ");

	p = crypt(pp, salt);
	setpriority(PRIO_PROCESS, 0, 0);

	memset(pp, 0, strlen(pp));
	if (pwd && !strcmp(p, pwd->pw_passwd))
	  break;

	printf("Login incorrect\n");
	badlogin(username); /* log ALL bad logins */
	failures++;

	/* we allow 10 tries, but after 3 we start backing off */
	if (++cnt > 3) {
	    if (cnt >= 10) {
		sleepexit(1);
	    }
	    sleep((unsigned int)((cnt - 3) * 5));
	}
    }

    /* committed to login -- turn off timeout */
    alarm((unsigned int)0);

    endpwent();
#ifdef TLG_FEATURE_SHADOWPASSWDS
	endspent();   /* stop access to shadow passwd file */
#endif

    /* This requires some explanation: As root we may not be able to
       read the directory of the user if it is on an NFS mounted
       filesystem. We temporarily set our effective uid to the user-uid
       making sure that we keep root privs. in the real uid.

       A portable solution would require a fork(), but we rely on Linux
       having the BSD setreuid() */

    {
	uid_t ruid = getuid();
	gid_t egid = getegid();

	setregid(-1, pwd->pw_gid);
	setreuid(0, pwd->pw_uid);
	setuid(0); /* setreuid doesn't do it alone! */
	setreuid(ruid, 0);
	setregid(-1, egid);
    }

    chown(ttyn, pwd->pw_uid,
	  (gr = getgrnam(TTYGRPNAME)) ? gr->gr_gid : pwd->pw_gid);
    chmod(ttyn, TTY_MODE);

    setgid(pwd->pw_gid);

    if (*pwd->pw_shell == '\0')
      pwd->pw_shell = _PATH_BSHELL;

    /* preserve TERM even without -p flag */
    {
	char *ep;

	if(!((ep = getenv("TERM")) && (termenv = strdup(ep))))
		termenv = "dumb";
    }

    /* destroy environment unless user has requested preservation */
    if (!pflag)
      {
	  environ = (char**)malloc(sizeof(char*));
	  memset(environ, 0, sizeof(char*));
      }

    setenv("HOME", pwd->pw_dir, 0);      /* legal to override */
    if(pwd->pw_uid)
      setenv("PATH", _PATH_DEFPATH, 1);
    else
      setenv("PATH", _PATH_DEFPATH_ROOT, 1);

    setenv("SHELL", pwd->pw_shell, 1);
    setenv("TERM", termenv, 1);

/* add from host by dzO */
#ifndef BB_FEATURE_DZO
    if(hostname)
	setenv("FROMHOST", hostname, 1);
     else if(strncmp(ttyn, _PATH_TTY, cnt)==0)
	setenv("FROMHOST", &ttyn[5], 1); /* skip /dev/ */
#endif

    /* LOGNAME is not documented in login(1) but
       HP-UX 6.5 does it. We'll not allow modifying it.
       */
    setenv("LOGNAME", pwd->pw_name, 1);

#ifdef DO_PS_FIDDLING
    setproctitle("login", username);
#endif

    if (tty[sizeof("tty")-1] == 'S')
      syslog(LOG_INFO, "DIALUP AT %s BY %s", tty, pwd->pw_name);

    /* allow tracking of good logins.
       -steve philp (sphilp@mail.alliance.net) */

    if (pwd->pw_uid == 0) {
	if (hostname)
	  syslog(LOG_NOTICE, "ROOT LOGIN ON %s FROM %s",
		 tty, hostname);
	else
	  syslog(LOG_NOTICE, "ROOT LOGIN ON %s", tty);
    } else {
	if (hostname)
	  syslog(LOG_INFO, "LOGIN ON %s BY %s FROM %s", tty,
		 pwd->pw_name, hostname);
	else
	  syslog(LOG_INFO, "LOGIN ON %s BY %s", tty,
		 pwd->pw_name);
    }

    signal(SIGALRM, SIG_DFL);
    signal(SIGQUIT, SIG_DFL);
    signal(SIGTSTP, SIG_IGN);
    signal(SIGHUP, SIG_DFL);
    signal(SIGINT, SIG_DFL);

    /* discard permissions last so can't get killed and drop core */
    if(setuid(pwd->pw_uid) < 0 && pwd->pw_uid) {
	syslog(LOG_ALERT, "setuid() failed");
	exit(1);
    }

    /* wait until here to change directory! */
    if (chdir(pwd->pw_dir) < 0) {
	printf("No directory %s!\n", pwd->pw_dir);
	if (chdir("/"))
	  exit(0);
	pwd->pw_dir = "/";
	printf("Logging in with home = \"/\".\n");
    }

    /* if the shell field has a space: treat it like a shell script */
    if (strchr(pwd->pw_shell, ' ')) {
	buff = xmalloc(strlen(pwd->pw_shell) + 6);

	if (!buff) {
	    exit(0);
	}

	strcpy(buff, "exec ");
	strcat(buff, pwd->pw_shell);
	childArgv[childArgc++] = "/bin/sh";
	childArgv[childArgc++] = "-sh";
	childArgv[childArgc++] = "-c";
	childArgv[childArgc++] = buff;
    } else {
	tbuf[0] = '-';
	strncpy(tbuf + 1, ((p = rindex(pwd->pw_shell, '/')) ?
			   p + 1 : pwd->pw_shell),
		sizeof(tbuf)-1);
	tbuf[sizeof(tbuf)-1] = 0;

	childArgv[childArgc++] = pwd->pw_shell;
	childArgv[childArgc++] = tbuf;
    }

    childArgv[childArgc++] = NULL;

    execvp(childArgv[0], childArgv + 1);

    errsv = errno;

    if (!strcmp(childArgv[0], "/bin/sh"))
	errorMsg("login: couldn't exec shell script: %s.",
		strerror(errsv));
    else
	errorMsg("login: no shell: %s.", strerror(errsv));

    exit(0);
}

static void
getloginname()
{
    register int ch;
    register char *p;
    static char nbuf[UT_NAMESIZE + 1];
    int cnt, cnt2;

    cnt2 = 0;
    for (;;) {
	cnt = 0;
	printf("\n%s login: ", thishost); fflush(stdout);
	for (p = nbuf; (ch = getchar()) != '\n'; ) {
	    if (ch == EOF) {
		badlogin("EOF");
		exit(0);
	    }
	    if (p < nbuf + UT_NAMESIZE)
	      *p++ = ch;

	    cnt++;
	    if (cnt > UT_NAMESIZE + 20) {
		errorMsg("login name much too long.");
		badlogin("NAME too long");
		exit(0);
	    }
	}
	if (p > nbuf) {
	  if (nbuf[0] == '-')
	    errorMsg("login names may not start with '-'.");
	  else {
	      *p = '\0';
	      username = nbuf;
	      break;
	  }
	}

	cnt2++;
	if (cnt2 > 50) {
	    errorMsg("too many bare linefeeds.");
	    badlogin("EXCESSIVE linefeeds");
	    exit(0);
	}
    }
}

static void
timedout()
{
    struct termio ti;

    errorMsg("Login timed out after %d seconds", timeout);

    /* reset echo */
    ioctl(0, TCGETA, &ti);
    ti.c_lflag |= ECHO;
    ioctl(0, TCSETA, &ti);
    exit(0);
}

static int
rootterm(ttyn)
     char *ttyn;
{
    int fd;
    char buf[100],*p;
    int cnt, more = 0;

    fd = open(SECURETTY, O_RDONLY);
    if(fd < 0) return 1;

    /* read each line in /etc/securetty, if a line matches our ttyline
       then root is allowed to login on this tty, and we should return
       true. */
    for(;;) {
	p = buf; cnt = 100;
	while(--cnt >= 0 && (more = read(fd, p, 1)) == 1 && *p != '\n') p++;
	if(more && *p == '\n') {
	    *p = '\0';
	    if(!strcmp(buf, ttyn)) {
		close(fd);
		return 1;
	    } else
	      continue;
	} else {
	    close(fd);
	    return 0;
	}
    }
}

static void
badlogin(const char *name)
{
    if (failures == 1) {
	if (hostname)
	  syslog(LOG_NOTICE, "LOGIN FAILURE FROM %s, %s",
		 hostname, name);
	else
	  syslog(LOG_NOTICE, "LOGIN FAILURE ON %s, %s",
		 tty, name);
    } else {
	if (hostname)
	  syslog(LOG_NOTICE, "%d LOGIN FAILURES FROM %s, %s",
		 failures, hostname, name);
	else
	  syslog(LOG_NOTICE, "%d LOGIN FAILURES ON %s, %s",
		 failures, tty, name);
    }
}

/* should not be called from PAM code... Why? */
static void
sleepexit(eval)
     int eval;
{
    sleep(SLEEP_EXIT_TIMEOUT);
    exit(eval);
}

#ifdef DO_PS_FIDDLING
/* proctitle code - we know this to work only on linux... */

/*
**  SETPROCTITLE -- set process title for ps (from sendmail)
**
**      Parameters:
**              fmt -- a printf style format string.
**
**      Returns:
**              none.
**
**      Side Effects:
**              Clobbers argv of our main procedure so ps(1) will
**              display the title.
*/

#ifndef SPT_BUFSIZE
#define SPT_BUFSIZE     2048
#endif

extern char** environ;

static char** argv0;
static int argv_lth;

static void
initproctitle (int argc, char **argv) {
	int i;
	char **envp = environ;

	/*
	 * Move the environment so we can reuse the memory.
	 * (Code borrowed from sendmail.)
	 * WARNING: ugly assumptions on memory layout here;
	 *          if this ever causes problems, #undef DO_PS_FIDDLING
	 */
	for (i = 0; envp[i] != NULL; i++)
		continue;
	environ = (char **) malloc(sizeof(char *) * (i + 1));
	if (environ == NULL)
		return;
	for (i = 0; envp[i] != NULL; i++)
		if ((environ[i] = strdup(envp[i])) == NULL)
			return;
	environ[i] = NULL;

	argv0 = argv;
	if (i > 0)
		argv_lth = envp[i-1] + strlen(envp[i-1]) - argv0[0];
	else
		argv_lth = argv0[argc-1] + strlen(argv0[argc-1]) - argv0[0];
}

static void
setproctitle (const char *prog, const char *txt) {
	int        i;
	char       buf[SPT_BUFSIZE];

	if (!argv0)
		return;

	if (strlen(prog) + strlen(txt) + 5 > SPT_BUFSIZE)
		return;

	(void) sprintf(buf, "%s -- %s", prog, txt);

	i = strlen (buf);
	if (i > argv_lth - 2) {
		i = argv_lth - 2;
		buf[i] = '\0';
	}
	memset(argv0[0], '\0', argv_lth);       /* clear the memory area */
	(void) strcpy (argv0[0], buf);

	argv0[1] = NULL;
}
#endif

#ifndef BB_FEATURE_DZO
static void errorMsg(const char *s, ...)
{
	va_list p;

	va_start(p, s);
	fflush(stdout);
	vfprintf(stderr, s, p);
	fprintf(stderr, "\n");
	va_end(p);
}
static void *xmalloc(size_t size)
{
	void *cp = malloc(size);

	if (cp == NULL) {
		errorMsg("out of memory");
	}
	return cp;
}
static void usage(const char *usage)
{
	fatalError("\nUsage: %s", usage);
}

static void fatalError(const char *s, ...)
{
	va_list p;

	va_start(p, s);
	vfprintf(stderr, s, p);
	fprintf(stderr, "\n");
	va_end(p);
	exit(1);
}
#endif
