#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <gdbm.h>
#include <errno.h>

#define MAXPU_LEN 12
#define MAXPU_REM_LEN 70
/*       */
#define UBD_REC_LEN (1+4+1+5+4+2+MAXPU_LEN+MAXPU_LEN+MAXPU_REM_LEN+3)

struct PU {     /*   */
	char user[MAXPU_LEN];
	char grp [MAXPU_LEN];
	char pass[MAXPU_LEN];
	int  l;
	int  priv;
	int  af;
	int  gate;
	int  flg_ar;
	int  flg_nocolor;
	int  flg_gmf;
	int  time_keeping_files;
	int  ex_direction;
	char rem [MAXPU_REM_LEN];
	int  naliases;
	char **aliases;
};

/* priv bits */
#define GRANTS_NO_GRANTS          32   /* ' ' */
#define GRANTS_ACCESSMASTER       1    /* change access */
#define GRANTS_POSTMASTER         2    /* change or disable email */
#define GRANTS_GATEMASTER         4    /* only change number gate for users */
#define GRANTS_ARCHIVER           8    /* ena/disable/destroy archive for users */
#define GRANTS_USERMASTER         16   /* remove user & change groups/pass/rem for users */
#define GRANTS_NEWSMASTER         64   /* only create & destroy conferences */
#define GRANTS_RESERVED           128  /* not use! flag ADM */
#define GRANTS_ADM                255  /* ADM user */

#define MAIL_FILE_GATE_COPY 2
#define MAIL_FILE_GATE_INFO 4
#define MAIL_FILE_GATE_REG  8

static void ErrorMsgAndDie (int exit_code, const char *s, va_list p)
  __attribute__ ((noreturn));

static void
ErrorMsgAndDie (int exit_code, const char *s, va_list p)
{

  vprintf (s, p);
  putchar ('\n');
  va_end (p);
  fflush(stdout);
  sleep(1);
  exit (exit_code);
}

void
error_d (int exit_code, const char *s, ...)
{
  va_list p;

  va_start (p, s);
  ErrorMsgAndDie (exit_code, s, p);
}

void
warn_d (const char *s, ...)
{
  va_list p;

  va_start (p, s);
  vprintf (s, p);
  va_end (p);
}

static void
err0alloc (void *ptr)
{
  if (ptr == 0)
	error_d (22, " ");
}

void *
mallocc (size_t size)
{
  void *ptr;
  if (size == 0)
	return 0;
  ptr = malloc (size);
  err0alloc (ptr);
  return memset (ptr, 0, size);
}

void *
reallocc (void *ptr, size_t size)
{
  if (ptr) {
	ptr = realloc (ptr, size);
	err0alloc (ptr);
  } else
	ptr = mallocc (size);
  return ptr;
}

static char *
strcpym (const char *s)
{
  char *ptr;
  ptr = mallocc (strlen (s) + 1);
  strcpy (ptr, s);
  return ptr;
}


static struct PU *all_pu; /*   */
static int n_all_pu;

static char name_user_db[] = "users/users.db";

static void
open_db_reader (GDBM_FILE * dbf, char* namedb)
{
  do {    /*     */
	*dbf = gdbm_open (namedb, 512, GDBM_READER, 0600, 0);
	if (gdbm_errno == GDBM_CANT_BE_READER)
	  sleep (1);
	else if (gdbm_errno != GDBM_NO_ERROR) {
	  error_d (2, "    %s", namedb);
	}
  } while (*dbf == NULL);
}


/*
  
key = USER%MAXPU_LENs
 L:
	- K koi8
	- A alt
	- W win1251
	- E translit
	- 7- -  ansi- 
 GATE  -   (telnet  ppp) %4d
 GRANTS:
	- GRANTS_ADM administrator
	- GRANTS_NO_GRANTS default
	- bits see fun.h
 KEEPING -     %5d
 DIRECT  -   %4d
 A(access flag):
       - A access
       - D deny
       - a access, no detect idle
       - d deny, no detect idle for change to access
       - e external host
 ARCH(     -)
	- '1'  '0'
       (  -)
	- ARCH|2 -       
	- ARCH|4 -         
	- ARCH|8 -        
 GROUP=%MAXPU_LENs
 PASS=%MAXPU_LENs
	- password=* - any
 REM=%MAXPU_REM_LENs
 naliases= %d (s) (  )
 aliases = N%s
*/

char Adm[] = "ADM";

void
form_r_pu (const char *recu, char *rec, int size_rec, struct PU *pu)
{
  int msk_flg;

  strcpy (pu->user, recu);
  pu->l = *rec;
  if (*rec < 0) {
	pu->l &= 0177;
	pu->flg_nocolor = 1;
  } else
	pu->flg_nocolor = 0;
  rec++;
  pu->gate = atoi (rec);
  rec += 4;
  pu->priv = *rec++;
  /* old database compatibility */
  if ((pu->priv & GRANTS_NO_GRANTS))
	pu->priv = GRANTS_NO_GRANTS;
  if (strcmp (pu->user, Adm) == 0)
	pu->priv = GRANTS_ADM;          /* end old database compatibility */
  else                                  /* to (unsigned char) */
	pu->priv &= GRANTS_ADM;
  pu->time_keeping_files = atoi (rec);
  rec += 5;
  pu->ex_direction = atoi (rec);
  rec += 4;
  pu->af = *rec++;
  msk_flg = *rec++;
  pu->flg_ar = msk_flg & 1;
  pu->flg_gmf = (msk_flg - '0') & ~1;
  strcpy (pu->grp, rec);
  rec += strlen (rec) + 1;
  strcpy (pu->pass, rec);
  rec += strlen (rec) + 1;
  strcpy (pu->rem, rec);
  rec += strlen (rec) + 1;
  if (size_rec > UBD_REC_LEN) {
	pu->naliases = atoi (rec);
	rec += strlen (rec) + 1;
	pu->aliases = (char **) mallocc (pu->naliases * sizeof (char *));
	for (msk_flg = 0; msk_flg < pu->naliases; msk_flg++) {
	  pu->aliases[msk_flg] = strcpym (rec);
	  rec += strlen (rec) + 1;
	}
  } else {
	pu->naliases = 0;
	pu->aliases  = 0;
  }
}


int
all_abonents (struct PU **ppu)
{
  int i, un;
  struct PU *pu;

  GDBM_FILE dbf;
  datum key, content;

  open_db_reader (&dbf, name_user_db);
  key.dptr = "*";
  key.dsize = 2;
  content = gdbm_fetch (dbf, key);
  if (content.dptr == NULL) {
rete:
	warn_d ("   !\n");
	return 0;
  } else
	un = atoi (content.dptr);
  pu = (struct PU *) mallocc (un * sizeof (struct PU));
  *ppu = pu;
  key = gdbm_firstkey (dbf);
  for (i = 0; i <= un; i++) {
	if (key.dptr == NULL)
	  goto rete;
	content = gdbm_fetch (dbf, key);
	if (content.dptr == NULL)
	  goto rete;
	if (strcmp (key.dptr, "*")) {
	  form_r_pu (key.dptr, content.dptr, content.dsize, pu);
	  pu++;
	}
	content = gdbm_nextkey (dbf, key);
	key = content;
  }
  gdbm_close (dbf);
  return un;
}


char *
form_w_pu (struct PU *pu, int *size_rec)
{
  char *rec = NULL;
  char *s = NULL;
  int i, sz;
  char ob[MAXPU_REM_LEN+58];
  char grants[8];
  int l;

  for(l = 0; l < 2; l++) {
	sz = 0;
	sprintf(ob, "LANG=%c", pu->l);
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	sprintf(ob, "VT200=%c", (pu->flg_nocolor ? '0' : '1'));
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	sprintf(ob, "GATE=%d", pu->gate);
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	grants[0] = grants[1] = 0;
	if((pu->priv & GRANTS_ADM) == GRANTS_ADM) {
	    grants[0] = 'A';
	} else {
	    if(pu->priv & GRANTS_ACCESSMASTER)
		grants[0] = 'C';
	    if(pu->priv & GRANTS_POSTMASTER)
		strcat(grants, "P");
	    if(pu->priv & GRANTS_GATEMASTER)
		strcat(grants, "G");
	    if(pu->priv & GRANTS_ARCHIVER)
		strcat(grants, "R");
	    if(pu->priv & GRANTS_USERMASTER)
		strcat(grants, "U");
	    if(pu->priv & GRANTS_NEWSMASTER)
		strcat(grants, "N");
	    if(grants[0] == 0)
		grants[0] = '-';
	}
	sprintf(ob, "GRANTS=%s", grants);
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	sprintf(ob, "KEEP=%d", pu->time_keeping_files);
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	sprintf(ob, "HOST=%d", pu->ex_direction);
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	i = pu->af;
	if(i == 'a')
	    i = 'A';
	if(i == 'd')
	    i = 'D';
	sprintf(ob, "ACCESS=%c", i);
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	i = pu->af;
	i = (pu->af == 'A' || pu->af == 'D') ? '1' : '0';
	sprintf(ob, "DWAIT=%c", i);
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	sprintf(ob, "ARC=%c", pu->flg_ar + '0');
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	sprintf(ob, "MGATE=%c", pu->flg_gmf + '0');
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	sprintf(ob, "GRP=%s", pu->grp);
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	for (i = 0; i < pu->naliases; i++) {
	  sprintf(ob, "ALIAS=%s", pu->aliases[i]);
	  if(l)
	    strcpy(s + sz, ob);
	  sz += strlen(ob) + 1;
	}
	sprintf(ob, "PASS=%s", pu->pass);
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	sprintf(ob, "REM=%s", pu->rem);
	if(l)
	    strcpy(s + sz, ob);
	sz += strlen(ob) + 1;
	if(l)
	    strcpy(s + sz, "END.");
	sz += 5;
	if(l == 0) {
	    *size_rec = sz;
	    s = rec = mallocc (sz);
	}
  }
  return rec;
}

static int
check_name (char *s)
{
  int i = 0;
  if (strlen (s) >= MAXPU_LEN) {
	warn_d ("`%s' -   \n", s);
	return 0;
  }
  if(*s == 0) {
	warn_d ("`%s' -  \n", s);
	return 0;
  }
  if (*s < 'A' || *s > 'Z');
  else
	for (i = 1; s[i]; i++)
	  if (!((s[i] >= 'A' && s[i] <= 'Z') ||
		 s[i] == '-' || s[i] == '_' || (s[i] >= '0' && s[i] <= '9')))
		break;
  if (s[i]) {
	warn_d ("      \n");
	return 0;
  }
  return 1;
}

static int
chk_pass (char *s)
{
  if (strlen (s) >= MAXPU_LEN) {
	warn_d ("  \n");
	return 0;
  }
  if (strlen (s) <= 2) {
	warn_d ("  \n");
	return 0;
  }
  return 1;
}

static char default_user_rem[] = " ";
static char default_ADM_rem[] = "";


int main(int argc, char **argv)
{
    int i, cr;
    int size_rec;
    char *rec, *m;
    FILE *in;

    n_all_pu = all_abonents (&all_pu);
    for (i = 0; i < n_all_pu; i++) {
	m = rec = form_w_pu(all_pu + i, &size_rec);
	printf("#\nUSER=%s:", all_pu[i].user);
	while(1) {
	    if(strcmp(rec, "END.") == 0)
		break;
	    cr = strncmp(rec, "PASS=", 5) == 0
		    || strncmp(rec, "REM=", 4) == 0
		    || strncmp(rec, "ARC=", 4) == 0
		    || strncmp(rec, "GRP=", 4) == 0
		    || strncmp(rec, "ALIAS=", 6) == 0;
	    printf("%s%c", rec, (cr ? '\n' : ':'));
	    rec += strlen(rec) + 1;
	}
	free(m);
    }
    free (all_pu);
    all_pu = 0;
    n_all_pu = 0;
    if(argc > 1) {
	char buf[512];
	char *user;
	struct PU *pu;

	in = fopen(argv[1], "r");
	if(in == NULL)
	    error_d(105, "%s: %m", argv[1]);
	user = 0;
	pu = 0;
	i = 0;
	while( (rec = fgets(buf, sizeof buf, in)) != NULL) {
	    i++;
	    while(*rec == ' ' || *rec == '\t')
		rec++;
	    if(*rec == '#' || *rec == '\n' || *rec == 0)
		continue;
	    cr = 0;
loop:
	    m = strchr(rec, ':');
	    if(m == NULL) {
		m = strchr(rec, '\n');
		if(m == NULL)
		    error_d(105, "%s: line %d too long", argv[1], i);
		cr = 1;
	    }
	    *m = 0;
	    if(user == 0 && strncmp(rec, "USER=", 5) != 0)
		error_d(105, "%s: line %d: user not defined", argv[1], i);
	    if(strncmp(rec, "USER=", 5) == 0) {
		int a;

		n_all_pu++;
		all_pu = (struct PU *) reallocc (all_pu, n_all_pu * sizeof (struct PU));
		pu = all_pu + n_all_pu - 1;
		memset(pu, 0, sizeof (struct PU));

		user = rec + 5;
		if(check_name(user) == 0) {
		    error_d(105, "%s: line %d, break processing", argv[1], i);
		}
		strcpy(pu->user, user);
		/* set default values */
		strcpy (pu->pass, "*");                /*   */
		pu->l = 'K';                           /* 8 */
		/*     */
		a = strcmp (user, Adm);
		pu->priv = a ? GRANTS_NO_GRANTS : GRANTS_ADM;
		pu->af = 'A';            /*  ,   */
		strcpy (pu->grp, "ALL");
		strcpy (pu->rem, a ? default_user_rem : default_ADM_rem);
		/*   , ,  ,  
		     ,  =0 */
next:
		rec += strlen(rec) + 1;
		if(cr)
		    continue;
		else
		    goto loop;
	    }
	    if(strncmp(rec, "LANG=", 5) == 0) {
		rec += 5;
		if(rec[1] != 0 || (*rec != 'K' && *rec != 'E' && *rec != 'A' && *rec != 'W'))
		    error_d(105, "%s: line %d, invalid %s", argv[1], i, rec-5);
		pu->l = *rec;
		goto next;
	    }
	    if(strncmp(rec, "VT200=", 6) == 0) {
		rec += 6;
		if(rec[1] != 0 || (*rec != '1' && *rec != '0'))
		    error_d(105, "%s: line %d, invalid %s", argv[1], i, rec-6);
		pu->flg_nocolor = (*rec - '0') ^ 1;
		goto next;
	    }
	    if(strncmp(rec, "GATE=", 5) == 0) {
		rec += 5;
		pu->gate = strtol(rec, &m, 10);
		if(m == rec || *m != 0)
		    error_d(105, "%s: line %d, invalid %s", argv[1], i, rec-5);
		goto next;
	    }
	    if(strncmp(rec, "GRANTS=", 7) == 0) {
		int g = 0;

		m = rec + 7;
		if(*m == 0) {
errg:
		    error_d(105, "%s: line %d, invalid %s", argv[1], i, rec);
		}
		while(*m) {
		    switch(*m) {
		    case 'A':
			if (g) {
			    goto errg;
			} else
			    g |= GRANTS_ADM;
			break;
		    case '-':
			if (g)
			    goto errg;
			g |= GRANTS_NO_GRANTS;
			break;
		    case 'C':
			if ((g & GRANTS_NO_GRANTS) != 0)        /* NO_GRANTS or ADM */
			    goto errg;
			g |= GRANTS_ACCESSMASTER;
			break;
		    case 'P':
			if ((g & GRANTS_NO_GRANTS) != 0)        /* NO_GRANTS or ADM */
			    goto errg;
			g |= GRANTS_POSTMASTER;
			break;
		    case 'G':
			if ((g & GRANTS_NO_GRANTS) != 0)        /* NO_GRANTS or ADM */
			    goto errg;
			g |= GRANTS_GATEMASTER;
			break;
		    case 'R':
			if ((g & GRANTS_NO_GRANTS) != 0)        /* NO_GRANTS or ADM */
			    goto errg;
			g |= GRANTS_ARCHIVER;
			break;
		    case 'U':
			if ((g & GRANTS_NO_GRANTS) != 0)        /* NO_GRANTS or ADM */
			    goto errg;
			g |= GRANTS_USERMASTER;
			break;
		    case 'N':
			if ((g & GRANTS_NO_GRANTS) != 0)        /* NO_GRANTS or ADM */
			    goto errg;
			g |= GRANTS_NEWSMASTER;
			break;
		    default:
			goto errg;
		    }
		    m++;
		}
		pu->priv = g;
		goto next;
	    }
	    if(strncmp(rec, "KEEP=", 5) == 0) {
		rec += 5;
		pu->time_keeping_files = strtol(rec, &m, 10);
		if(m == rec || *m != 0)
		    error_d(105, "%s: line %d, invalid %s", argv[1], i, rec-5);
		goto next;
	    }
	    if(strncmp(rec, "HOST=", 5) == 0) {
		rec += 5;
		pu->ex_direction = strtol(rec, &m, 10);
		if(m == rec || *m != 0)
		    error_d(105, "%s: line %d, invalid %s", argv[1], i, rec-5);
		goto next;
	    }
	    if(strncmp(rec, "ACCESS=", 7) == 0) {
		rec += 7;
		if(rec[1] != 0 || (*rec != 'a' && *rec != 'd' && *rec != 'A' && *rec != 'D'))
		    error_d(105, "%s: line %d, invalid %s", argv[1], i, rec-7);
		pu->af = *rec;
		goto next;
	    }
	    if(strncmp(rec, "DWAIT=", 6) == 0) {
		rec += 6;
		if(rec[1] != 0 || (*rec != '0' && *rec != '1'))
		    error_d(105, "%s: line %d, invalid %s", argv[1], i, rec-6);
		pu->af &= ~040;
		goto next;
	    }
	    if(strncmp(rec, "ARC=", 4) == 0) {
		rec += 4;
		if(rec[1] != 0 || (*rec != '0' && *rec != '1'))
		    error_d(105, "%s: line %d, invalid %s", argv[1], i, rec-4);
		pu->flg_ar = *rec - '0';
lcr:
		if(cr == 0)
		    error_d(105, "%s: line %d, <CR> inspected", argv[1], i);
		goto next;
	    }
	    if(strncmp(rec, "MGATE=", 6) == 0) {
		int g;

		rec += 6;
		g = *rec - '0';
		if(rec[1] != 0 || g < 0 || (g | (2|4|8)) > (2|4|8))
		    error_d(105, "%s: line %d, invalid %s", argv[1], i, rec-6);
		pu->flg_gmf = g;
		goto next;
	    }
	    if(strncmp(rec, "GRP=", 4) == 0) {
		m = rec + 4;
		if(check_name(m) == 0) {
		    error_d(105, "%s: line %d, break processing", argv[1], i);
		}
		strcpy(pu->grp, m);
		goto lcr;
	    }
	    if(strncmp(rec, "ALIAS=", 6) == 0) {
		int a;

		m = rec + 6;
		if(check_name(m) == 0) {
		    error_d(105, "%s: line %d, break processing", argv[1], i);
		}
		a = pu->naliases;
		pu->naliases++;
		pu->aliases = reallocc(pu->aliases, pu->naliases * sizeof (char *));
		pu->aliases[a] = strcpym(m);
		goto lcr;
	    }
	    if(strncmp(rec, "PASS=", 5) == 0) {
		m = rec + 5;
		if(chk_pass(m) == 0) {
		    error_d(105, "%s: line %d, break processing", argv[1], i);
		}
		strcpy(pu->pass, m);
		goto lcr;
	    }
	    if(strncmp(rec, "REM=", 4) == 0) {
		m = rec + 4;
		strncpy(pu->rem, m, sizeof(pu->rem)-1);
		if(strlen(m) >= sizeof(pu->rem)) {
		    warn_d("%s: line %d,    =%s=", argv[1], i, pu->rem);
		}
		goto lcr;
	    }
	    if(strcmp(rec, "END.") == 0) {
		user = 0;
		goto lcr;
	    }
	    warn_d("%s: line %d, invalid field", argv[1], i, rec);
	}
    }
    return 0;
}
