/*
    /  /   /
*/

#include "fun.h"
#include "db.h"
#include "ex.h"
#include "keyboard.h"
#include "crc32.h"

/*        */
struct PU pu_all_user = {
      "*", /* user */
      "*", /* grp  */
      "*", /* pass */
       0 , /* l */
       0 , /* priv */
      'e', /* af */
       0 , /* gate */
       0 , /* flg_ar */
       1 , /* flg_nocolor */
       0 , /* flg_gmf */
      -1 , /* time_keeping_files */
      -1 , /* ex_direction -   */
      "" , /* rem */
       0 , /* naliases */
       0 , /* **aliases */
};

#ifndef NO_EXT_DIRECTIONS

static char epath[16+INC_KEY_SIZE]="../../ext000/";
#define EPATH_NUM_P 9

void delete_direction(int direction)
{
    sprintf(epath+EPATH_NUM_P, "%03d", direction);
    unlink(epath);
}

void create_new_direction(int direction)
{
    sprintf(epath+EPATH_NUM_P, "%03d", direction);
    mkdir(epath, 0770);
}

/*
         
*/

static void
ex_cmd_dir(int cmd, const char *fi, const char *cmda, const char *cur_user,
			    const char *user, int direction)
{

    FILE *out;
    mode_t old_mask;
    const char *na_inc;

    na_inc = name_auto_increment();
    sprintf(epath+EPATH_NUM_P, "%03d/%s", direction, na_inc);
    old_mask = umask(0444); /*   */
    out = fopen(epath, "w");
    if(out==NULL)
	error_d(71, "        ");
    fprintf(out, "%c %s %s %s", cmd, na_inc, cur_user, fi);
    if(cmd>0) {
	if(cmda)
	    fprintf(out, " %s", cmda);
    }
    fprintf(out, " %s/%d", user, direction);
    putc('\n', out);
    if(cmd<0) {
	/* cmda    -     */
	while(*cmda)
	    putc(*cmda++, out);
	putc('\n', out);
    }
    fclose(out);
    umask(old_mask);
    chmod(epath, 0660); /*   */
}


void
ex_cmd(int cmd, const char *fi, const char *cmda, const struct PU *for_user)
{
    int i, d;

    if(for_user != &pu_all_user)
	ex_cmd_dir(cmd, fi, cmda, pu_cur.user, for_user->user, for_user->ex_direction);
    else {
	for(d=0; d<=MAX_DIRECTIONS; d++) {
	    if(d==host_direction)
	      continue;
	    for (i = 0; i < n_all_pu; i++)
	      if (all_pu[i].ex_direction==d) {
		/*    */
		ex_cmd_dir(cmd, fi, cmda, pu_cur.user, pu_all_user.user, d);
		/*        */
		break;
	      }
	}
    }
}

/*  ' '  '\t'  '/'  '\001' */
char *convert_din(char *sdin)
{
    char *ret = sdin;

    while(*sdin) {
	if(*sdin == ' ')
	    *sdin = '\t';
	else if(*sdin == '/')
	    *sdin = '\001';
	sdin++;
    }
    return ret;
}

/*    convert_din() */
char *decode_converted_din(char *sdin)
{
    char *ret = sdin;

    while(*sdin) {
	if(*sdin == '\t')
	    *sdin = ' ';
	else if(*sdin == '\001')
	    *sdin = '/';
	sdin++;
    }
    return ret;
}

static int split_args(char *inbuf, char **cmd, char **na_inc,
		      char **a1, char **a2, char **a3, char **a4)
{
    char *as[6];
    int i;
    int flg_found_cr = 0;

    for(i=0; i<6; i++) {
	char c;

	while(*inbuf == ' ')
	    inbuf++;
	as[i]=inbuf;
	c = *inbuf;
	if(c == '\n' || c == 0) {
	    if(c=='\n') {
		*inbuf = 0;
		flg_found_cr++;
	    }
	    continue;
	}
	while(*inbuf != ' ' && *inbuf != '\n' && *inbuf != 0)
	    inbuf++;
	c = *inbuf;
	*inbuf = 0;
	if(c) {
	    inbuf++;
	    if(c=='\n')
		flg_found_cr++;
	}
    }
    if(!flg_found_cr)
	return -1;
    *cmd    = as[0];
    *na_inc = as[1];
    *a1     = as[2];
    *a2     = as[3];
    *a3     = as[4];
    *a4     = as[5];
    if(**a2 == 0)
	return 0;
    if(**a3 == 0)
	return 3;
    return **a4 ? 5 : 4;
}

static int split_file_args(char *a, char **a1, char **a2, char **a3)
{
    char *p;

    p = strchr(a, '/');
    if(p==0)
	return -1;
    *a1 = a;
    *p++ = 0;
    *a2 = p;
    p = strchr(p, '/');
    if(p==0)
	return -2;
    *p++ = 0;
    *a3 = p;
    return 3;
}

/*       */
static char * read_coded_buffer(void)
{
    char *b = 0;
    int sz = 0;
    int readed=0;
    int c;

    while((c=getchar())!=EOF && c!='\n') {
	if(readed==sz)
	    b = reallocc(b, sz += 128);
	b[readed++] = c;
    }
    if(c==EOF)
	return 0;
    if(readed==sz)
	b = reallocc(b, sz + 1);
    b[readed] = 0;
    return b;
}


/*        */

static void begin_c_s(int sg)
{
    disable_out = 1;            /*    stdout */
    pu_cur.flg_nocolor = 1;     /*  ansi colors */
    pu_cur.priv = GRANTS_ADM;   /*     */
    log_login (sg);
    chdir("..");                /*   users   Adm */
}

/*  */

static int chdir_u_out(const char *user, int flg_chdir_out, int line)
{
    if(chdir(user)) {
	put_log ("L=%d       %s\n", line, user);
	return 0;
    }
    if(flg_chdir_out!=0)
	    if(chdir("out")) {
		put_log ("L=%d      %s/out\n", line, user);
		return 0;
	    }
    return 1;
}

static int
forward_direction(int cmd, const char *cur_user, const char *fi,
			     const char *cmda, char *user)
{
    char *sdir;
    int direction;

    sdir = strchr(user, '/');
    if(sdir == 0) {
	put_log (" ,    %s\n", user);
	disable_out = 0;
	error_d (80, "-");
    }
    *sdir++ = 0;
    direction = g_int(&sdir);
    if(direction<0 || direction>MAX_DIRECTIONS) {
	put_log (" ,  %d\n", direction);
	disable_out = 0;
	error_d (80, "-");
    }
    if(direction!=host_direction) {
	/*   */
	if(strcmp(user, pu_all_user.user)) { /*   */
	    chdir(Adm);
	    put_log ("=%d %c %s\n", direction, cmd, fi);
	    ex_cmd_dir(cmd, fi, cmda, cur_user, user, direction);
	    chdir("..");
	}
	return 1;
    }
    return 0;
}

int d_cle(void)
{
    char buf[1024];
    char *cmd;
    int na;
    struct stat statb;
    int req_send;       /*   -  uucode */
    int tmp;

    struct INF_S    inf;
    struct REG_S    pregs;
    struct SYS_NEWS sys_news;
    struct NUM_NEWS num_news;
    struct PU       pu;
    struct PU       pu_old;

    char *file, *din, *rem;
    FILE *uuout;
    int   fd;
    char *size;
    char *na_inc;
    char *user;     /* ,     */
    char *c_user;   /* ,   */
    char *fi;       /*  , ,  */
    char *cmda;     /*   */

    begin_c_s('C');
    if((fd=open(".", O_RDONLY, 0)) < 0) {
	put_log("    users. Exiting\n");
	disable_out = 0;
	error_d (77, "-");
    }
    req_send = 0;

    while( fgets(buf, sizeof(buf), stdin) != NULL) {
	   if(buf[0]=='-')
		continue;
	   if(req_send)
		na = split_args(buf, &cmd, &din, &file, &cmda, &cmda, &cmda);
	    else
		na = split_args(buf, &cmd, &na_inc, &c_user, &fi, &cmda, &user);
	   if(na < 0) {
err_proto:
		put_log (" , =%d\n", na);
		disable_out = 0;
		error_d (72, "-");
	   }
	   if(na > 0) {
		/*  uuencode */
	     if(req_send!=0) {
		put_log ("  na=%d,  uuencode\n", na);
		disable_out = 0;
		error_d (72, "-");
	     }
	     if(na==4) {
		user = cmda;
		cmda = 0;
	     }
	     if(cmd[0]<0) {
		/*       */
		cmda = read_coded_buffer();
		if(cmda==0) {
lrec:
		    put_log("    . Exiting\n");
		    disable_out = 0;
		    error_d (95, "-");
		}
	     }
	     if(forward_direction(cmd[0], c_user, fi, cmda, user)) {
		if(cmd[0]<0)
		    free(cmda);
		goto Ok;
	     }
	     if(cmd[0]<0) {
		size = form_decode_ex(cmda, &tmp);
		free(cmda);
		cmda = size;
		if(cmda==0) goto lrec;
	     }
	     na_inc = strcpym(f_prompt("%s/%s/%s", Adm, dir_ext, na_inc));
	     if(access(na_inc, R_OK)==0) {
		/* ,      */
		free(na_inc);
		/*       */
		goto Ok;
	     }
	     if(access(na_inc, W_OK)==0) {
		/*  */
		;
	     } else {
		int nf;
		mode_t old_mask;

		/*    */
		file = strrchr(na_inc, '/');
		*file = 0;
		if(chdir(na_inc)!=0) {
		    put_log("    %s. Exiting\n", na_inc);
		    disable_out = 0;
		    error_d(70, "-");
		}
		nf = my_scandir(".");
		while(--nf >= 0) {
		  if(access(name_dlist[nf], R_OK)!=0)
		    put_log("   %s!\n", name_dlist[nf]);
		  /*    */
		  unlink(name_dlist[nf]);
		}
		/*    */
		old_mask = umask(0444); /*    -   */
		uuout = fopen(file+1, "w");
		if(uuout==NULL) {
		    put_log("    . Exiting\n");
		    disable_out = 0;
		    error_d(70, "-");
		}
		fclose(uuout);
		umask(old_mask);
		*file = '/';    /*  / */
		fchdir(fd);
	     }
	   }
	   switch(cmd[0]) {
	    case EXCMD_SEND_FILE:
		if(na!=5)
		    goto err_proto;
		if(!chdir_u_out(c_user, 0, __LINE__))
		    break;
		if (get_pu (&pu, user) == 0) {
		    put_log ("     %s\n", user);
		    break;
		}
		fchdir(fd);
		if(!chdir_u_out(c_user, 1, __LINE__))
		    break;
		fi = strcpy (inf.file_inc, fi); /*   */
		if(split_file_args(cmda, &file, &rem, &size)!=3)
		    goto err_proto;
		strcpy (inf.user, c_user);
		inf.file = strcpym (file);
		strcpy (inf.rem, decode_converted_din(rem));
		rem = name_auto_increment ();
		decode_inc (rem, inf.d);
		inf.flg_r = '0';
		inf.flg_new = '1';
		inf.flg_d = '0';
		inf.sz = strtol(size, 0, 16);   /*   */
		/*  -    ? */
		if(stat(fi, &statb) != 0)
		    statb.st_size = 0;
		if(statb.st_size == inf.sz && inf.sz > 0) {
		    fchdir(fd);
		    if(chdir_u_out(user, 0, __LINE__)) {
			put_db_inf (&inf);
			send_email_ex(pu.flg_gmf, inf.file, inf.user,
				    statb.st_size, user, fi);
			put_log (" %s     %s\n", fi, user);
			free(inf.file);
		    }
		    break;
		}
		if(statb.st_size <= 256) {
		    /*   -   */
		    statb.st_size = 0;
		    /*    */
		    printf("Ok %s/out %s 0 0\n", c_user, fi);
		    put_log ("Ok %s/out %s 0 0\n", c_user, fi);
		} else {
		    /*   */
		    unsigned long oldcrc32;
		    long sz;
		    int c;

		    statb.st_size -= 256;
		    if(truncate(fi, statb.st_size)) {
			put_log ("   %s  %ld\n", fi, statb.st_size);
			break;
		    }
		    uuout=fopen(fi, "r");
		    oldcrc32 = INITC32();
		    sz = 0;
		    while( (c=fgetc(uuout)) != EOF) {
			oldcrc32 = UPDC32 (c, oldcrc32);
			sz++;
		    }
		    oldcrc32 = ENDC32(oldcrc32);
		    fclose(uuout);
		    if(sz != statb.st_size) {
			put_log ("Hmm, truncate problem?\n");
			break;
		    }
		    /*    */
		    printf("Ok %s/out %s %lx %lx\n", c_user, fi, sz, oldcrc32);
		    put_log ("Ok %s/out %s %lx %lx\n", c_user, fi, sz, oldcrc32);
		}
		/*       */
		user = strcpy(pu.user, user);
		req_send++;
		continue;
	    case EXCMD_REVOKE_FILE:
		if(na!=5)
		    goto err_proto;
		if(!chdir_u_out(cmda, 0, __LINE__))
		    break;
		strcpy (inf.file_inc, fi);
		inf.flg_d = '1';
		unlink_db_in (&inf, 1, 1);
		break;
	    case EXCMD_DELETE_FILE:
		if(na!=4)
		    goto err_proto;
		if(!chdir_u_out(c_user, 1, __LINE__))
		    break;
		unlink (fi);
		put_log (" %s   %s\n", fi, c_user);
		break;
	    case EXCMD_NOTIFY_FILE:
	    case EXCMD_NOTIFYEFILE:
		if(na!=5)
		    goto err_proto;
		strcpy (pregs.user, c_user);
		strcpy (pregs.file_inc, fi);
		if(split_file_args(cmda, &file, &din, &rem)!=3)
		    goto err_proto;
		if(!chdir_u_out(user, 0, __LINE__)) {
		    break;
		}
		pregs.file = decode_converted_din(strcpym (file));
		strcpy (pregs.rem,  decode_converted_din(rem));
		strcpy (pregs.din,  decode_converted_din(din));
		put_db_reg (&pregs);
		put_log ("  %s  %s   %s  %s\n",
				    file, c_user, pregs.din, user);
		if(*cmd == EXCMD_NOTIFYEFILE)
		    send_email (f_prompt ("  %s     %s\n",
			c_user, pregs.file), 0, 0, 0, 0);
		free(pregs.file);
		break;
	    case 'b':
		if(na!=0 || req_send==0)
		    goto err_proto;
		req_send=0;
		if(cmd[1]=='-') /*     */
		    break;
		if(cmd[1]==' ')
		    goto err_proto;
	    {
		long lxskip, ldskip;
		char *e;
		int c;

		lxskip = strtol(din, &e, 16);
		if(e == din || *e != '\0') {
		    put_log("   lxskip. Exiting\n");
		    disable_out = 0;
		    error_d (76, "-");
		}
		ldskip = strtol(file, &e, 10);
		if(e == file || *e != '\0') {
		    put_log("   ldskip. Exiting\n");
		    disable_out = 0;
		    error_d (76, "-");
		}
		if(lxskip != ldskip || lxskip < 0) {
		    put_log("lxskip=%ld ldskip=%ld. Exiting\n", lxskip, ldskip);
		    disable_out = 0;
		    error_d (76, "-");
		}
		uuout=fopen(fi, lxskip > 0 ? "a" : "w");
		if(uuout==NULL) {
		    put_log("   %s  lxskip=%ld. Exiting\n", fi, lxskip);
		    disable_out = 0;
		    error_d (84, "-");
		}
		if(lxskip > 0) {
		    ldskip = ftell(uuout);
		    if(ldskip != lxskip) {
			put_log("END=%ld != skip=%ld. Truncating. Exiting\n", ldskip, lxskip);
			fclose(uuout);
			truncate(fi, 0);
			disable_out = 0;
			error_d (84, "-");
		    }
		}
		lxskip = 0;
		while( (c=getchar()) != 'x') {
		    if(c == EOF) {
			put_log("Unexpected EOF. Exiting 85\n");
			disable_out = 0;
			error_d (85, "-");
		    }
		    if(c == 1) {
			c = getchar();
			if(c == EOF) {
			    put_log("Unexpected EOF. Exiting 86\n");
			    disable_out = 0;
			    error_d (86, "-");
			}
			if(c == 2)
			    c = 'x';
			else if(c == '0')
			    c = 0;
			else if(c != 1) {
			    put_log("Unexpected c=%02x after 1. Exiting\n", c);
			    disable_out = 0;
			    error_d (87, "-");
			}
		    }
		    putc(c, uuout);
		    lxskip++;
		    if(lxskip >= 255) {
			fflush(uuout);
			lxskip = 0;
		    }
		}
		fclose(uuout);
	    }
		if(stat(fi, &statb)!=0 || statb.st_size != inf.sz)  {
		    /*    */
		    put_log (" %s    %s\n", fi, user);
		    break;
		}
		if(user[0]!='/') {
		    /*  ,    */
		    fchdir(fd);
		    if(chdir_u_out(user, 0, __LINE__)) {
			put_db_inf (&inf);
			send_email_ex(pu.flg_gmf, inf.file, inf.user,
				    statb.st_size, user, fi);
			put_log (" %s   %s\n", fi, user);
			free(inf.file);
		    }
		}
		break;  /*    */
	    case EXCMD_DELETE_NOTF:
		if(na!=4)
		    goto err_proto;
		if(!chdir_u_out(c_user, 0, __LINE__))
		    break;
		strcpy(pregs.file_inc, fi);
		strcpy(pregs.user, user);
		pregs.flg_d = '1';
		unlink_db_reg (&pregs, 1);
		break;
	    case EXCMD_DELETE_KONF:
		if(na!=4)
		    goto err_proto;
		if(!chdir_u_out(c_user, 0, __LINE__))
		    break;
		sys_news.num = atoi(fi);
		unlink_db_news(&sys_news);
		break;
	    case EXCMD_CHG_SYS_KNF:
	    case EXCMD_NEW_SYS_KNF:
		if(na!=4)
		    goto err_proto;
		if(!chdir_u_out(c_user, 0, __LINE__))
		    break;
		sys_news.num = strtol(fi, &user, 10);
		put_db_news (&sys_news, cmd[0]==EXCMD_CHG_SYS_KNF, &cmda, &tmp);
		break;
	    case EXCMD_DEL_REC_KNF:
		if(na!=4)
		    goto err_proto;
		if(!chdir_u_out(c_user, 0, __LINE__))
		    break;
		if(split_file_args(fi, &file, &din, &rem)!=3)
		    goto err_proto;
		num_news.tema = atoi(din);
		num_news.otvet = atoi(rem);
		unlink_db_news_num (&num_news, atoi(file));
		break;
	    case EXCMD_DEL_ATF_KNF:
		if(na!=5)
		    goto err_proto;
		if(!chdir_u_out(c_user, 0, __LINE__))
		    break;
		if(split_file_args(fi, &file, &din, &rem)!=3)
		    goto err_proto;
		tmp = atoi(file);
		num_news.tema = atoi(din);
		num_news.otvet = atoi(rem);
		if(split_file_args(cmda, &file, &din, &rem)!=3)
		    goto err_proto;
		strcpy(num_news.bb, din);
		put_db_news_num_del (&num_news, tmp, file);
		break;
	    case EXCMD_AA_NUM_KNF:
		if(na!=4)
		    goto err_proto;
		if(!chdir_u_out(c_user, 0, __LINE__))
		    break;
		if(split_file_args(fi, &file, &din, &rem)!=3)
		    goto err_proto;
		/* K.T=x@Author/F/NF$REM */
		na = strtol(file, &size, 10);
		num_news.tema = strtol(size+1, &size, 10);
		num_news.ls = strtol(size+1, &size, 16);
		strcpy(num_news.author, size+1);
		strcpy(num_news.bb, din);
		num_news.nf = strtol(rem, &size, 10);
		decode_converted_din(strcpy(num_news.rem, size+1));
		put_db_news_num (&num_news, na, num_news.tema, &cmda, &tmp);
		free(cmda);
		break;
	    case EXCMD_CHG_NUM_KNF:
		if(na!=4)
		    goto err_proto;
		if(!chdir_u_out(c_user, 0, __LINE__))
		    break;
		if(split_file_args(fi, &file, &din, &rem)!=3)
		    goto err_proto;
		/*   */
		na = strtol(file, &size, 10);
		num_news.tema = strtol(size+1, &size, 10);
		num_news.otvet = strtol(size+1, &size, 10);
		strcpy(num_news.bb, din);
		num_news.nf = strtol(rem, &size, 10);
		put_db_news_num_add (&num_news, na, &cmda, &tmp);
		free(cmda);
		break;
	    case EXCMD_CHGFNUM_KNF:
		if(na!=5)
		    goto err_proto;
		inf.sz = strtol(cmda, 0, 16);
		/*  -    ? */
			/* +1 - skip '/' */
		if(stat(fi+1, &statb) != 0)
		    statb.st_size = 0;
		if(statb.st_size == inf.sz && inf.sz > 0)
		    break;
		file = strrchr(fi, '/');
		*file++ = 0;
		if(chdir(fi+1)) {   /* +1 - skip '/' */
		    put_log("     . Exiting\n");
		    disable_out = 0;
		    error_d (96, "-");
		}
		if(statb.st_size <= 256) {
		    /*   -   */
		    statb.st_size = 0;
		    /*    */
		    printf("Ok %s %s 0 0\n", fi, file);
		    put_log ("Ok %s %s 0 0\n", fi, file);
		} else {
		    /*   */
		    unsigned long oldcrc32;
		    int c;

		    statb.st_size -= 256;
		    if(truncate(file, statb.st_size)) {
			put_log ("   %s  %ld\n", file, statb.st_size);
			break;
		    }
		    uuout=fopen(file, "r");
		    oldcrc32 = INITC32();
		    while( (c=fgetc(uuout)) != EOF) {
			oldcrc32 = UPDC32 (c, oldcrc32);
		    }
		    oldcrc32 = ENDC32(oldcrc32);
		    fclose(uuout);
		    /*    */
		    printf("Ok %s %s %lx %lx\n", fi, file, statb.st_size, oldcrc32);
		    put_log ("Ok %s %s %lx %lx\n", fi, file, statb.st_size, oldcrc32);
		}
		/*   */
		fi = strcpy (inf.file_inc, file);
		user = "/";  /*  -  ,   */
		req_send++;
		continue;

	    case EXCMD_ADD_NEW_USR:
		if(na!=4)
		    goto err_proto;
		if(!chdir_u_out(c_user, 0, __LINE__))
		    break;
		/*      */
		if (get_pu (&pu, Adm) == 0) {
		    put_log ("   get_pu (Adm)\n");
		    disable_out = 0;
		    error_d (78, "-");
		}
		add_user_fun(fi, &pu);
		break;
	    case EXCMD_ERASE_USR:
		if(na!=4)
		    goto err_proto;
		if(!chdir_u_out(c_user, 0, __LINE__))
		    break;
		if (get_pu (&pu, fi) == 0) {
		    put_log ("   get_pu (%s)\n", fi);
		    disable_out = 0;
		    error_d (78, "-");
		}
		tmp = pu.ex_direction;
		del_user(fi);
		chdir("..");
		system(f_prompt("rm -rf %s", fi));
		chdir(Adm);
		/*   */
		if (get_pu (&pu, Adm) == 0) {
		    put_log ("   get_pu (Adm)\n");
		    disable_out = 0;
		    error_d (78, "-");
		}
		if(have_ex_direction(tmp)==0)
		    delete_direction(tmp);
		break;
	    case EXCMD_CHG_USER:
		if(na!=4)
		    goto err_proto;
		form_r_pu (fi, cmda, tmp, &pu);
		if(!chdir_u_out(c_user, 0, __LINE__))
		    break;
		if (get_pu (&pu_old, pu.user) == 0) {
		    put_log ("   get_pu (%s)\n", pu.user);
		    disable_out = 0;
		    error_d (78, "-");
		}
		if (db_add_user (&pu, 1) == 0)
		    put_log ("      %s\n", cmda);
		create_new_direction(pu.ex_direction);
		if(pu_old.flg_ar != pu.flg_ar)
		    create_delete_arch_ex(pu.user, pu.flg_ar);
		break;
	    default:
		goto err_proto;
	   }
	   fchdir(fd);
	   if(req_send==0) {
	    chmod(na_inc, 0660); /*   - :  */
	    free(na_inc);
	   }
Ok:
	   printf("Ok\n");
    }
    return 0;
}


/*  */

int d_sle(void)
{
    int n, na, i;
    int fd;

    begin_c_s('S');
    sprintf(epath+EPATH_NUM_P, "%03d", host_direction);
    if(chdir(epath+3) || (fd=open(".", O_RDONLY, 0)) < 0) {
	put_log("     %s. Exiting\n", epath+3);
	disable_out = 0;
	error_d (75, "-");
	}

    for(;;) {
	n = my_scandir (".");
	if(n<0) {
	    put_log("    \n");
	    return 1;
	}
	na = 0;
	for(i=0; i<n; i++) {
	    if(access(name_dlist[i], R_OK)!=0)
		na++;
	}
	if((n-na)>0) {

	    for(i=0; i<n; i++) {
		if(access(name_dlist[i], R_OK)==0) {
		    FILE *in;
		    char buf[1024];
		    char *si;

		    in = fopen(name_dlist[i], "r");
		    if(in==NULL || fgets(buf, sizeof(buf), in)==NULL) {
			put_log("   %s\n", name_dlist[i]);
			if(in!=NULL)
			    fclose(in);
			continue;
		    }
		    printf("%s", buf);
		    buf[70] = 0;    /*       */
		    if(strlen(buf) == 70 && buf[69] != '\n') {
			buf[70] = '\n';
			buf[71] = 0;
		    }
		    put_log("%s", buf);
	    /*    -   */
		    while(fgets(buf, sizeof(buf), in)!=NULL)
			printf("%s", buf);
		    fclose(in);
		    for(;;) {
			char *s_Ok, *user, *file, *skip, *crc_first_part;

			si = fgets(buf, sizeof(buf), stdin);
			if(si==NULL)
			    return 1;
			if(buf[0]=='-') {
			    put_log("  \n");
			    return 1;
			}
			if(strncmp(buf, "Ok", 2)!=0) {
err_proto:
			    put_log("  buf=%s. Exiting\n", buf);
			    disable_out = 0;
			    error_d (74, "b- 0 0");
			}
			if(buf[2]=='\n') {  /* "Ok\n" */
			    unlink(name_dlist[i]);
			    put_log("%s Ok\n", name_dlist[i]);
			    break;
			}
			if(buf[2]!=' ')
				goto err_proto;
			na = split_args(buf, &s_Ok, &user, &file, &skip, &crc_first_part, &si);
			if(na!=4) {
			    put_log("  na=%d. Exiting\n", na);
			    disable_out = 0;
			    error_d (81, "b- 0 0");
			}
			si = f_prompt("../users/%s", user);
			if(chdir(si)==0) {
			    if((in=fopen(file, "r")) != NULL) {
				/*       */
				long lskip;
				char *e;
				int c;

				lskip = strtol(skip, &e, 16);
				if(lskip < 0 || e == skip || *e != '\0') {
				    put_log(" ,  skip=%s. Exiting\n", skip);
				    disable_out = 0;
				    error_d (82, "b- 0 0");
				}
				if(lskip > 0) {
				    unsigned long oldcrc32;
				    long skiped = lskip;
				    unsigned long getedcrc32;

				    getedcrc32 = strtoul(crc_first_part, &e, 16);
				    if(e == crc_first_part || *e != '\0') {
					put_log(" ,  crc_first_part=%s. Exiting\n", crc_first_part);
					disable_out = 0;
					error_d (83, "b- 0 0");
				    }
				    c = EOF;    /* fuck gcc warn */
				    oldcrc32 = INITC32();
				    while(skiped > 0 && (c=fgetc(in)) != EOF) {
					oldcrc32 = UPDC32 (c, oldcrc32);
					skiped--;
				    }
				    oldcrc32 = ENDC32(oldcrc32);
				    if(c == EOF) {
					put_log(" , skip > filesize.\n");
					lskip = 0;
				    } else if(oldcrc32 != getedcrc32) {
					put_log(" crc_first=%lx getedcrc32=%s skip=%lx,   \n",
						oldcrc32, crc_first_part, lskip);
					lskip = 0;
				    }
				}
				fseek(in, lskip, SEEK_SET);
				printf("b %lx %ld\n", lskip, lskip);
				put_log("Send %s skip=%ld\n", file, lskip);
				lskip = 0;
				while( (c=fgetc(in)) != EOF) {
				    if(c == 1) {
					putchar(c);
					lskip++;
				    }
				    else if(c == 'x') {
					putchar(1);
					c = 2;
					lskip++;
				    } else if(c == 0) {
					putchar(1);
					c = '0';
					lskip++;
				    }
				    putchar(c);
				    if(++lskip >= 254) {
					lskip = 0;
					fflush(stdout);
				    }
				}
				putchar('x');   /* end indicator */
				fflush(stdout);
				fclose(in);
			    } else {
				/*        */
				put_log("  %s     \n", file);
				printf("b- 0 0\n");
			    }
			    fchdir(fd);
			} else {
			    put_log("    %s\n", si);
			    printf("b- 0 0\n");
			}
		    }
		}
	    }
	} else {
	    /*   ,  */
	    sleep(1);
	}
    }
}

#else
void delete_direction(int direction) { }
void create_new_direction(int direction) { }
char *convert_din(char *sdin) { return sdin; }
char *decode_converted_din(char *sdin) { return sdin; }
void ex_cmd(int cmd, const char *fi, const char *cmda, const struct PU *for_user) { }
int d_cle(void)
{
    error_d (77, "     ");
}
int d_sle(void)
{
    error_d (77, "     ");
}
#endif /* NO_EXT_DIRECTIONS */
