/*    */
#include "fun.h"
#include "keyboard.h"
#include "db.h"
#include "hlps.h"

static char file_gates[] = "../gates.lst";

#define GT_PPP    0
#define GT_TELNET 1
#define GT_END    1

static char *names_type_gates[] = {
  "PPP   ",
  "Telnet"
};

struct S_GATES
{
  int num;
  int type;
  char *adr;
  struct S_GATES *next;
};

static struct S_GATES *s_gates;

static char title[] = "\n\
 # +-  -+ \n";

static void
list_gates (int flg_all)
{
  struct S_GATES *cur;
  int i;

  for (i = 0, cur = s_gates; cur != 0; cur = cur->next, i++) {
	if (i == 0)
	  putcs (title);
	if (flg_all == -1 || flg_all == cur->type)
	  warn_d ("%2d %s    %s\n", cur->num, names_type_gates[cur->type],
			  cur->adr);
  }
}


static void
chg_gate (int type)
{

  static char *prompt[] = {
	"\n /* PPP */ (, , , , -, ?)",
	"\n (, , , , -, ?)",
  };
  static const char *const alhlp[] = {
	" -  PPP    \n\
 -  PPP    \n\
   -     PPP \n\
  -  PPP    .\n\n",

	" -    telnet-\n\
 -    telnet-\n\
   -     telnet \n\
  -    telnet-.\n\n",
  };
  static char *dlg_com_gates_all[] = {
	"",
	"",
	"",
	"",
	"-",
	NULL
  };

  struct IN_DLG in_dlg;
  int n, i, flg_chg;
  struct S_GATES *cur;
  struct S_GATES *n_cur;
  FILE *out;

  in_dlg.msgs = &dlg_com_gates_all;
  in_dlg.prompt = prompt[type];
  in_dlg.alhlp = alhlp[type];
  switch (type) {
  case GT_PPP:
	in_dlg.hlps = &gate_dlg_hlps_ppp;
	break;
  case GT_TELNET:
	in_dlg.hlps = &gate_dlg_hlps_telnet;
	break;
  }
  in_dlg.zvezda = 0;
  flg_chg = 0;
  for (;;) {
	n = scan_input (&in_dlg);
	switch (n) {
	case 1:
	  i = 0;
	  for (cur = s_gates; cur != 0; cur = cur->next)
		if (cur->type == type && cur->num > i)
		  i = cur->num;
	  i++;
	  n_cur = mallocc (sizeof (struct S_GATES));
	  n_cur->next = 0;
	  n_cur->type = type;
	  n_cur->num = i;
	  n_cur->adr = strcpym (editstring (""));
	  if (s_gates == 0)
		s_gates = n_cur;
	  else {
		for (cur = s_gates; cur->next != 0; cur = cur->next);
		cur->next = n_cur;
	  }
	  flg_chg++;
	  break;
	case 2:
	  i = atoi (editstring (" "));
	  for (cur = s_gates; cur != 0; cur = cur->next)
		if (cur->type == type && cur->num == i)
		  break;
	  if (cur == 0)
		putcs ("  \n");
	  else {
		free (cur->adr);
		cur->adr = strcpym (editstring (""));
		flg_chg++;
	  }
	  break;
	case 3:
	  list_gates (type);
	  break;
	case 4:
	  i = atoi (editstring (" "));
	  for (n_cur = cur = s_gates; cur != 0; cur = cur->next) {
		if (cur->type == type && cur->num == i)
		  break;
		if (cur != s_gates)
		  n_cur = n_cur->next;
	  }
	  if (cur == 0)
		putcs ("  \n");
	  else {
		if (cur == s_gates)
		  s_gates = s_gates->next;
		else
		  n_cur->next = cur->next;
		free (cur->adr);
		free (cur);
		flg_chg++;
	  }
	  break;
	case 5:
	  if (flg_chg) {
		out = fopen (file_gates, "w");
		if (out == NULL)
		  error_d (24, "    ");
		for (cur = s_gates; cur != 0; cur = cur->next)
		  fprintf (out, "%d %d %s\n", cur->type, cur->num, cur->adr);
	  }
	  return;
	}
  }
}


static void
get_gates (void)
{
  FILE *fg;
  char buff[80];
  char buff2[80];
  struct S_GATES *cur;
  struct S_GATES *prev = NULL;

  if (s_gates)
	return;
  fg = fopen (file_gates, "r");
  if (fg != NULL) {
	while (fgets (buff, 80, fg) != NULL) {
	  cur = mallocc (sizeof (struct S_GATES));
	  cur->next = 0;
	  sscanf (buff, "%d%d%s", &cur->type, &cur->num, buff2);
	  cur->adr = strcpym (buff2);
	  if (s_gates == 0)
		s_gates = cur;
	  else
		prev->next = cur;
	  prev = cur;
	}
	fclose (fg);
  }
}

static char *dlg_com_gates[] = {
  "",
  "",
  "",
  "-",
  "",
  "\n",
  "",
  "",
  "",
  "",
  "",
  NULL
};

static char prompt[] = "\n (, , , -, ?)";
static const char alhlp[] = "\
 -   PPP\n\
   -  -\n\
   -   .\n\n";

int
dlg_gates (void)
{
  struct IN_DLG in_dlg;
  int n;

  in_dlg.msgs = &dlg_com_gates;
  in_dlg.prompt = prompt;
  in_dlg.alhlp = alhlp;
  in_dlg.hlps = &gate_dlg_hlps;
  in_dlg.zvezda = 0;
  if (chk_adm () == 0)
	return 0;
  get_gates ();
  for (;;) {
	n = scan_input (&in_dlg);
	switch (n) {
	case 1:
	  chg_gate (GT_PPP);
	  break;
	case 2:
	  chg_gate (GT_TELNET);
	  break;
	case 3:
	  list_gates (-1);
	  break;
	case 4:
	  return 0;
	case 5:
	  return 100;
	case 6:                                 /* \n */
	  break;
	case 7:
	  return 1;
	case 8:
	  return 2;
	case 9:
	  return 3;
	case 10:
	  return 4;
	case 11:
	  return 5;
	}
  }
}

/*      flg_all,
 flg_all=(-1),   -  */
void *
gate_exist (int g, int flg_all)
{
  struct S_GATES *cur;

  get_gates ();
  for (cur = s_gates; cur != 0; cur = cur->next)
	if (cur->num == g) {
	  if (flg_all == -1 || flg_all == cur->type)
		return cur;
	}
  return 0;
}


void
start_gate (int s)
{
  int t = 0;
  char com_buf[80];
  struct S_GATES *cur;

  if (pu_cur.gate == 0) {
	putcs ("\r2      .\r7\n");
	if (pu_cur.priv != GRANTS_ADM)
	  putcs ("  .\n");
	putcc ('\n');
	return;
  }
  switch (s) {
  case 'P':
	t = GT_PPP;
	break;
  case 'T':
	t = GT_TELNET;
	break;
  }
  cur = gate_exist (pu_cur.gate, t);
  if (cur == 0) {
	putcs ("   .\n");
	if (pu_cur.priv != GRANTS_ADM)
	  putcs ("  .\n");
	putcc ('\n');
	return;
  }
  switch (t) {
  case GT_PPP:
	strcpy (com_buf, "../ppp.sh ");
	strcat (com_buf, cur->adr);
	put_log ("%s\n", com_buf);
	run_and_check_idle (com_buf);
	break;
  case GT_TELNET:
	strcpy (com_buf, "telnet -8E ");
	strcat (com_buf, cur->adr);
	put_log ("%s\n", com_buf);
	run_and_check_idle (com_buf);
	break;
  }
  exit (0);
}
