/*   */

/* #define DEBUG */

#include "fun.h"
#include "keyboard.h"
#include "db.h"
#include "hlps.h"
#include "code.h"
#include "ex.h"

#define PRIV_CH 1
#define PRIV_Z  2
#define PRIV_U  4
#define PRIV_P  8
#define PRIV_O  16
#define PRIV_R  32
#define PRIV_4  64
#define PRIV_5  128
#define PRIV_6  256
#define PRIV_7  512

#define PRIV_ALL (PRIV_CH|PRIV_Z|PRIV_U|PRIV_P|PRIV_O|PRIV_R|PRIV_4|PRIV_5|PRIV_6|PRIV_7)



static struct SYS_NEWS *sys_news;
static int n_sys_news;

static struct U_NEWS *u_news;
static int n_u_news;

static void
get_udb_news (void)
{
  int n, i, k;
  if (stat_db_chg (N_DB_NU) != 0) {
	struct U_NEWS *pinf;
	pinf = u_news;
	for (n = 0; n < n_u_news; n++) {
	  for (i = 0; i < pinf->sz; i++)
		free0 (&pinf->tof[i]);
	  free0 (&pinf->tof);
	  free0 (&pinf->hash);
	  pinf++;
	}
	free0 (&u_news);
	n_u_news = 0;
  }
  if (n_u_news == 0) {
	struct U_NEWS u_news_chg;

	i = n_u_news = get_db_un (&u_news);
	for (n = 0; n < (i - 1); n++)
	  for (k = n + 1; k < i; k++)
		if (u_news[k].num < u_news[n].num) {
		  u_news_chg = u_news[n];
		  u_news[n] = u_news[k];
		  u_news[k] = u_news_chg;
		}
  }
}

static void
free_sys_news_if_chg (void)
{
  int n, i;

  for (n = 0; n < n_sys_news; n++) {
	for (i = 0; i < sys_news[n].nusers; i++)
	  free (sys_news[n].users[i]);
	free (sys_news[n].users);
	free (sys_news[n].priv);
  }
  free0 (&sys_news);
  n_sys_news = 0;
}

void
scan_new_konf (void)
{
  int n, k, i, j, ni;
  struct SYS_NEWS sys_news_chg;

  if (stat_db_chg (N_DB_NEWS) != 0)
	free_sys_news_if_chg ();
  get_udb_news ();
  if (n_sys_news == 0) {
	char *new_k;

	i = n_sys_news = get_db_news (&sys_news);
	new_k = mallocc(i);
	for (n = 0; n < (i - 1); n++)
	  for (k = n + 1; k < i; k++)
		if (sys_news[k].num < sys_news[n].num) {
		  sys_news_chg = sys_news[n];
		  sys_news[n] = sys_news[k];
		  sys_news[k] = sys_news_chg;
		}
	ni = j = 0;
	for (n = 0; n < n_u_news; n++) {
	  for (k = 0; k < n_sys_news; k++)
		if (u_news[n].num == sys_news[k].num) {
		  new_k[k] |= 1;
		  i--;
		  if (sys_news[k].time_new > u_news[n].time_lr) {
			ni++;
			new_k[k] |= 2;
		  }
		  break;
		}
	  if (k == n_sys_news) {
		j++;
	  }
	}
	if(ni > 0) {
	    if(ni > 10) {
		warn_d ("    : \r2%d\r7\n", ni);
	    } else {
		ni = 0;
		warn_d (" ");
		for (k = 0; k < n_sys_news; k++)
		    if (new_k[k] & 2) {
			warn_d ("%s\r2[%d]\r7", (ni == 0 ? "" : ", "), sys_news[k].num);
			ni++;
		    }
		warn_d (" %s   \n",
			  (ni == 1 ? "" : ""));
	    }
	}
	if(j) {
	  warn_d ("   :");
	  for (n = 0; n < n_u_news; n++) {
	    for (k = 0; k < n_sys_news; k++)
		if (u_news[n].num == sys_news[k].num) {
		  break;
		}
	    if (k == n_sys_news) {
		warn_d (" %d", u_news[n].num);
		unlink_db_un (&u_news[n]);
	    }
	  }
	  warn_d ("\n");
	  get_udb_news ();
	}
	if (i) {
	  char srus1;

	  i = 0;
	  for (k = 0; k < n_sys_news; k++) {
		if((new_k[k] & 1) == 0) {
		    for (j = 0; j < sys_news[k].nusers; j++)
			if (strcmp (sys_news[k].users[j], pu_cur.user) == 0)
			    i++;
		}
	  }
	  if (i) {
		if (i == 1 || ((i % 100) >= 21 && (i % 10) == 1))
		  srus1 = '';
		else if ((i % 10) > 1 && (i % 10) < 5 && (i < 5 || (i % 100) > 21))
		  srus1 = '';
		else
		  srus1 = '';
		warn_d ("%c \r2%d\r7 %c:\n",
				(i == 1 ? '' : ''), i, srus1);
		i = 0;
		for (k = 0; k < n_sys_news; k++) {
		    if((new_k[k] & 1) == 0) {
			for (j = 0; j < sys_news[k].nusers; j++)
			    if (strcmp (sys_news[k].users[j], pu_cur.user) == 0) {
				if(++i == 10) {
				    warn_d ("...\n");
				    break;
				}
				warn_d ("[%d] %s\n", sys_news[k].num, sys_news[k].name);
			    }
		    }
		}
	  }
	}
	free0(&new_k);
  }
}

static void
put_access (int k)
{
  static char mode_code[] = "4567";
  char vs[2];
  int i, m;

  if ((k & PRIV_CH) != 0)               /*  ,  4,5 */
	k &= ~(PRIV_4 | PRIV_5);
  if ((k & PRIV_Z) != 0)                /*  ,  6,7 */
	k &= ~(PRIV_6 | PRIV_7);
  vs[1] = 0;
  m = 1;
  for (i = 0; i < 10; i++) {
	vs[0] = (k & m) != 0 ? mode_code[i] : '_';
	putcs (vs);
	m = m << 1;
  }
}

static void
list_members (int ns)
{
  int i;

  putcc ('\n');
  for (i = 1; i <= sys_news[ns].nusers; i++) {
	warn_d ("%3d %-20s", i, sys_news[ns].users[i - 1]);
	put_access (sys_news[ns].priv[i - 1]);
	if ((i % 2) == 0)
	  putcc ('\n');
	else
	  putcc (' ');
	if (((i / 2) % 20) == 19)
	  if (wait_cr ())
		break;
  }
  putcc ('\n');
}

static void
remove_user_from_news (int ns, int num)
{
  int i, k;
  char **usrs;
  int *priv;

  usrs = (char **) mallocc ((sys_news[ns].nusers - num) * sizeof (char *));
  priv = (int *) mallocc ((sys_news[ns].nusers - num) * sizeof (int));
  for (i = k = 0; k < sys_news[ns].nusers; k++)
	if (sys_news[ns].users[k] != 0) {
	  usrs[i] = sys_news[ns].users[k];
	  priv[i] = sys_news[ns].priv[k];
	  i++;
	}
  sys_news[ns].nusers = i;
  free (sys_news[ns].users);
  sys_news[ns].users = usrs;
  free (sys_news[ns].priv);
  sys_news[ns].priv = priv;
}

/*       num
   buf - ,  ,      
    num<0 -   
   */
static void update_ex_sys_news(int num, char *buf, int size)
{
    char snum[64];
    char *coded;

    if(have_ext_users()) {
	/*       */
	coded = form_code_ex((unsigned char *)buf, size);
	sprintf(snum, "%d", num < 0 ? -num : num);
	ex_cmd((num > 0 ? EXCMD_CHG_SYS_KNF : EXCMD_NEW_SYS_KNF),
			    snum, coded, &pu_all_user);
	free(coded);
    }
}

/*
       
*/
static void f_ex_num_news(int num, const char *file)
{
    const char *s;
    struct stat b;
    char *ssz;

    if(have_ext_users()) {
	/*      */
	/*     users/ */
	s = f_prompt("/../%s/%d/%s", dir_news, num, file);
	chdir("..");
	if(stat(s+1, &b)!=0)
	    error_d(93, "   %s", s+1);
	chdir(pu_cur.user);
	s = strcpym(s);
	ssz = f_prompt("%lx", b.st_size);
	ex_cmd(EXCMD_CHGFNUM_KNF, s, ssz, &pu_all_user);
	ssz = (char *)s;
	free(ssz);
    }
}

/*
           num
*/
static void
update_ex_num_news(struct NUM_NEWS *news, int num, char *buf, int size)
{
    char *sa;
    char *coded;

    if(have_ext_users()) {
	coded = form_code_ex((unsigned char *)buf, size);
	sa = f_prompt("%d:%d.%d/%s/%d", num, news->tema, news->otvet,
							news->bb, news->nf);
	ex_cmd(EXCMD_CHG_NUM_KNF, sa, coded, &pu_all_user);
	free(coded);
    }
}

/*
       /    num
*/
static void
aa_ex_num_news(struct NUM_NEWS *news, int num, char *buf, int size)
{
    char *sa;
    char *coded;
    int t;

    if(have_ext_users()) {
	coded = form_code_ex((unsigned char *)buf, size);
	t = news->otvet == 0 ? 0 : news->tema;
	sa = f_prompt("%d.%d=%lx@%s/%s/%d$%s",
		      num, t,                           /* K.T */
			     news->ls,                  /* =x */
				news->author,           /* @Author */
				news->bb, news->nf,     /* /F/NF */
				convert_din(news->rem));/* $REM */
	ex_cmd(EXCMD_AA_NUM_KNF, sa, coded, &pu_all_user);
	free(coded);
	decode_converted_din(news->rem);
    }
}

/*
       //   k
    file_rec=0 -  
*/
static void
del_ex_num_news(int k, struct NUM_NEWS *pnews, int file_rec, const char *file)
{
    if(have_ext_users()) {
	/*   -     */
	if(file_rec==0) {
	    ex_cmd(EXCMD_DEL_REC_KNF,
		f_prompt("%d/%d/%d", k, pnews->tema, pnews->otvet), 0, &pu_all_user);
	} else {
	    char *f_bb = strcpym(f_prompt("%s/%s/", file, pnews->bb));

	    ex_cmd(EXCMD_DEL_ATF_KNF,
		f_prompt("%d/%d/%d", k, pnews->tema, pnews->otvet), f_bb, &pu_all_user);
	    free(f_bb);
	}
    }
}

void
remove_user_from_all_news (char *user)
{
  int n, i, j, *numa = NULL, nnuma;
  char *genbuf;
  int size;

  if (stat_db_chg (N_DB_NEWS) != 0)
	free_sys_news_if_chg ();
  n = n_sys_news;
  if (n == 0)
	n = n_sys_news = get_db_news (&sys_news);
  if (n)
	numa = alloca (n * sizeof (int));
  nnuma = 0;
  for (j = 0; j < n; j++) {
	for (i = 0; i < sys_news[j].nusers; i++)
	  if (strcmp (sys_news[j].users[i], user) == 0) {
		free0 (&sys_news[j].users[i]);
		remove_user_from_news (j, 1);
		size = 0; /*  -     */
		put_db_news (&sys_news[j], 1, &genbuf, &size);
		update_ex_sys_news(sys_news[j].num, genbuf, size);
		free(genbuf);
		numa[nnuma++] = j + 1;
		break;
	  }
  }
  if (nnuma) {
	warn_d ("\r2    \r1");
	for (i = 0;;) {
	  warn_d ("%d", numa[i++]);
	  if (i < nnuma)
		putcc (',');
	  else
		break;
	}
	warn_d ("\r2 %c.\r7\n", nnuma == 1 ? '' : '');
  }
}

static void
chg_status (int ns)
{
  char *s;
  int i, j, p, flg_chg, k;
  struct List_To_Ret list_to;

  list_members (ns);
  s = 0;
  i = 0;
  for (;;) {
	if (s == 0) {
	  s = editstring (" (4567, -, ?)");
	  i = 0;
	}
	if (*s == 0) {
	  if (i)
		break;
	  s = 0;
	} else if (*s == '-') {
	  if (i) {
		s = 0;
		putcs ("  \n");
	  } else
		break;
	} else if (*s == '?') {
	  putcs (" -    ,\n4 -  ,\n\
5 -  ,\n -    ,\n6 -  ,\n\
7 -  ,\n - ,\n -  ,\n - ,\n\
 -  \n\
`-' -     \n");
	  s = 0;
	} else {
	  if (*s == '^') {
		i |= PRIV_CH | PRIV_4 | PRIV_5;
		s++;
	  } else if (*s == '4') {
		i |= PRIV_4;
		s++;
	  } else if (*s == '5') {
		i |= PRIV_5;
		s++;
	  } else if (*s == 'Z') {
		i |= PRIV_Z | PRIV_6 | PRIV_7;
		s++;
	  } else if (*s == '6') {
		i |= PRIV_6;
		s++;
	  } else if (*s == '7') {
		i |= PRIV_7;
		s++;
	  } else if (*s == 'U') {
		i |= PRIV_U;
		s++;
	  } else if (*s == 'P') {
		i |= PRIV_P;
		s++;
	  } else if (*s == 'O') {
		i |= PRIV_O;
		s++;
	  } else if (*s == 'R') {
		i |= PRIV_R;
		putcs ("\r1,   \r7\n");
		s++;
	  } else {
		s = 0;
		putcs ("  \n");
	  }
	}
  }
  if (i == 0)
	return;
  p = i;

  list_to.n_list_to = 0;
  list_to.list_to = 0;

  for (i = 0; i < sys_news[ns].nusers; i++)
	if (sys_news[ns].priv[i] == p)
	  list_to.n_list_to++;
  list_to.list_to = (char **) mallocc (list_to.n_list_to * sizeof (char *));
  j = 0;
  flg_chg = 0;
  for (i = 0; i < sys_news[ns].nusers; i++)
	if (sys_news[ns].priv[i] == p)
	  list_to.list_to[j++] = strcpym (sys_news[ns].users[i]);
  get_list_users (0, &list_to, "", 0, 0);
  if (list_to.n_list_to < 0) {
	putcs ("\r1  !\r7\n");
	return;
  }
  /*       */
  for (i = 0; i < list_to.n_list_to; i++) {
	s = list_to.list_to[i];
	for (k = 0; k < sys_news[ns].nusers; k++) {
	  if (strcmp (sys_news[ns].users[k], s) == 0) {
		if (sys_news[ns].priv[k] == p) {
		  k = -1;                               /*   */
		  break;
		}
		warn_d (" %s       ", s);
		put_access (sys_news[ns].priv[k]);
		putcc ('\n');
		if (strcmp (s, Adm) == 0)
		  putcs ("    !\n");
		else {
		  if (strcmp (s, sys_news[ns].moder) == 0)
			putcs(
	  ":      \n");
		  if (no_yes (" ")) {
			sys_news[ns].priv[k] = p;
			flg_chg++;
			if (strcmp (s, sys_news[ns].moder) == 0 && (p & PRIV_R) == 0) {
			  putcs ("\r2  ,\n\
   \r7\n");
			  strcpy (sys_news[ns].moder, Adm);
			}
		  }
		}
		k = -1;                   /*      */
		break;
	  }
	}
	if (k == sys_news[ns].nusers) {   /*   */
	  sys_news[ns].nusers++;
	  sys_news[ns].users = (char **) reallocc (sys_news[ns].users,
				      sys_news[ns].nusers * sizeof (char *));
	  sys_news[ns].priv = (int *) reallocc (sys_news[ns].priv,
				      sys_news[ns].nusers * sizeof (int));
	  sys_news[ns].users[k] = strcpym (s);
	  sys_news[ns].priv[k] = p;
	  flg_chg++;
	}
  }
  for (k = 0; k < sys_news[ns].nusers; k++) {   /*   */
	if (sys_news[ns].priv[k] != p)
	  continue;
	for (i = 0; i < list_to.n_list_to; i++)
	  if (strcmp (sys_news[ns].users[k], list_to.list_to[i]) == 0)
		break;
	if (i == list_to.n_list_to) {   /*   */
	  if (strcmp (sys_news[ns].users[k], Adm) == 0)
		putcs ("      \n");
	  else {
		if (strcmp (sys_news[ns].users[k], sys_news[ns].moder) == 0) {
		  putcs ("\r2  ,\n\
   \r7\n");
		  strcpy (sys_news[ns].moder, Adm);
		}
		free0 (&sys_news[ns].users[k]);
	  }
	}
  }
  destroy_nlist (&list_to);
  for (j = k = 0; k < sys_news[ns].nusers; k++)
	if (sys_news[ns].users[k] == 0)
	  j++;
  if (j) {
	remove_user_from_news (ns, j);
	flg_chg++;
  }
  if (flg_chg) {
	j = 0; /*  -     */
	put_db_news (&sys_news[ns], 1, &s, &j);
	update_ex_sys_news(sys_news[ns].num, s, j);
	free(s);
	putcs ("\n  \n");
	list_members (ns);
  }
}

/*     k */
static void
paste_to (struct NUM_NEWS *news_s, struct U_NEWS *pu_news, int k)
{
  int i, j, n;
  char *p1;
  struct stat statb;
  char *s;

  if (yes_no ("  ")) {
	clear_tmp (1);
	if (flg_inetd == 0)
	  putcs ("    Zmodem\n");
	debug1 (link ("../../../test.long.name", "test.very-very-long.name"));
	debug1 (link ("../../../Makefile", "Makefile"));
	if (flg_inetd == 0)
	  debug2 (run_and_check_idle ("rz"));
	else {
	  for (;;) {
		i = recv_ip (&s, 1);
		if (i < 0)
		  break;
		free (s);
	  }
	}
	chdir ("..");
	j = my_scandir ("tmp");
	if (j <= 0)
	  return;
	chdir ("tmp");
	for (i = 0; i < j; i++) {
	  if (test_long_name (name_dlist[i], 1) == 0) {
		unlink (name_dlist[i]);
		putcs ("\n");
	  }
	}
	chdir ("..");
	j = my_scandir ("tmp");
	if (j == 0)
	  return;
	for (n = 0; n < j; n++)
	  for (i = 0; i < news_s->nf; i++)
		if (strcmp (news_s->namef[i], name_dlist[n]) == 0) {
		  warn_d ("\n:  %s     \n",
				  name_dlist[n]);
		  break;
		}
	for (i = 0; i < news_s->nf; i++) {  /*    */
	  free (news_s->namef[i]);
	  free (news_s->nsf[i]);
	  free (news_s->nameu[i]);
	}
	free0 (&news_s->namef);
	free0 (&news_s->nsf);
	free0 (&news_s->nameu);
	free0 (&news_s->sizef);
	news_s->nf = j;
	/*       */
	news_s->namef = (char **) mallocc (j * sizeof (char *));
	news_s->nsf = (char **) mallocc (j * sizeof (char *));
	news_s->nameu = (char **) mallocc (j * sizeof (char *));
	news_s->sizef = (long *) mallocc (j * sizeof (long));
	for (i = 0; i < j; i++) {
	  news_s->namef[i] = strcpym (name_dlist[i]);
	  news_s->nsf[i] = strcpym (name_auto_increment ());
	  news_s->nameu[i] = strcpym (pu_cur.user);
	  p1 = bb_asprintf("tmp/%s", name_dlist[i]);
	  stat (p1, &statb);
	  news_s->sizef[i] = statb.st_size;
	  sprintf (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY],
			   "%d/%s", k, news_s->nsf[i]);
	  if (rename (p1, buf_news_empty)) {
		putcs ("      \n");
		free(p1);
		return;
	  }
	  free(p1);
	}
  } else
	return;
  for (i = 0; i < j; i++) {
    /*        */
    f_ex_num_news(k, news_s->nsf[i]);
  }
  j = 0; /*  -     */
  put_db_news_num_add (news_s, k, &s, &j);
  /*        */
  update_ex_num_news(news_s, k, s, j);
  free(s);
  for (i = 0; i < n_sys_news; i++)
	if (sys_news[i].num == k) {
	  sys_news[i].time_new = time (0);
	  j = 0; /*  -     */
	  put_db_news (&sys_news[i], 1, &s, &j);
	  update_ex_sys_news(sys_news[i].num, s, j);
	  free(s);
	  break;
	}
  /*        */
  put_db_un (pu_news, 1);
}


/*    (num_tema=0)     k */
static void
put_to (int priv, int k, int num_tema, char *rem)
{
  int i, j;
  struct NUM_NEWS new_nn;
  char rem_b[80];
  char *p1;
  struct stat statb;
  char *s;

  if (num_tema == 0) {
	strncpy (new_nn.rem, editrem ("  "), 48);
	new_nn.rem[36] = 0;
	if(new_nn.rem[0] == '-' || new_nn.rem[0] == 0)
	    return;
  } else {
	strncpy (rem_b, rem, 44);
	rem_b[32] = 0;
	strcpy (new_nn.rem, "Re: ");
	strcat (new_nn.rem, rem_b);
  }
  strcpy (new_nn.author, pu_cur.user);
  new_nn.ls = 0;
  new_nn.nf = 0;
  new_nn.namef = 0;
  new_nn.nsf = 0;
  strcpy (new_nn.bb, name_auto_increment ());
  if (yes_no ("  ")) {
	switch (pu_cur.l) {
	case 'A':
	  putcs("    ,\n\
    .\n");
	  break;
	case 'W':
	  putcs("    ,\n\
    CP1251.\n");
	  break;
	default:
	  putcs ("    ,\n\
    8.\n");
	  break;
	}
	clear_tmp (1);
	if (flg_inetd == 0)
	  putcs ("\n    Zmodem\n");
	debug1 (link ("../../../Makefile", "Makefile"));
	if (flg_inetd == 0)
	  debug2 (run_and_check_idle ("rz"));
	else {
	  i = recv_ip (&s, 1);
	  if (i > 0)
	    free (s);
	}
	chdir ("..");
	j = my_scandir ("tmp");
	if (j <= 0)
	  return;
	if (j > 1) {
	  putcs ("    \n");
	  return;
	}
	p1 = bb_asprintf("tmp/%s", name_dlist[0]);
	stat (p1, &statb);
	if (statb.st_size == 0)
	  putcs ("  - \n");
	else {
	  FILE *in, *out;
	  int c;

	  sprintf (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY],
			   "%d/%s", k, new_nn.bb);
	  out = fopen (buf_news_empty, "w");
	  if (out == 0) {
		putcs ("      \n");
		return;
	  }
	  in = fopen (p1, "r");
	  if (in == 0) {                        /* ??? */
		putcs ("   .\n");
		fclose (out);
		return;
	  }
	  new_nn.ls = statb.st_size;
	  j = 0;
	  while ((c = getc (in)) != EOF) {
		if (flg_inetd == 0) {
		  if (pu_cur.l == 'A')
			c = code_k_alt_k8[c];
		  else if (pu_cur.l == 'W')
			c = code_k_win_k8[c];
		}
		if (!code_k[c]) {
		  c = ' ';
		  j++;
		}
		putc (c, out);
	  }
	  fclose (in);
	  fclose (out);
	  if (j)
	    putcs ("       \n");
	  f_ex_num_news(k, new_nn.bb);
	}
	free(p1);
  }
  if ((priv & PRIV_P) == 0) {
	putcs ("\
:     ()    .\n");
	if (new_nn.ls == 0)
	  return;
  } else {
	if (yes_no ("  ")) {
	  clear_tmp (1);
	  if (flg_inetd == 0)
		putcs ("    Zmodem\n");
	  debug1 (link ("../../../test.long.name", "test.very-very-long.name"));
	  debug1 (link ("../../../Makefile", "Makefile"));
	  if (flg_inetd == 0)
		debug2 (run_and_check_idle ("rz"));
	  else {
		for (;;) {
		  i = recv_ip (&s, 1);
		  if (i < 0)
		    break;
		  free (s);
		}
	  }
	  chdir ("..");
	  j = my_scandir ("tmp");
	  if (j <= 0)
		return;
	  chdir ("tmp");
	  for (i = 0; i < j; i++) {
		if (test_long_name (name_dlist[i], 1) == 0) {
		  unlink (name_dlist[i]);
		  putcs ("\n");
		}
	  }
	  chdir ("..");
	  j = my_scandir ("tmp");
	  if (j == 0)
		return;
	  new_nn.nf = j;
	  new_nn.namef = (char **) alloca (j * sizeof (char *));
	  new_nn.nsf = (char **) alloca (j * sizeof (char *));
	  new_nn.nameu = (char **) alloca (j * sizeof (char *));
	  new_nn.sizef = (long *) alloca (j * sizeof (long));
	  for (i = 0; i < j; i++) {
		new_nn.namef[i] = strcpym (name_dlist[i]);
		new_nn.nsf[i] = strcpym (name_auto_increment ());
		new_nn.nameu[i] = strcpym (pu_cur.user);
		p1 = bb_asprintf("tmp/%s", name_dlist[i]);
		stat (p1, &statb);
		new_nn.sizef[i] = statb.st_size;
		sprintf (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY], "%d/%s",
				 k, new_nn.nsf[i]);
		if (rename (p1, buf_news_empty)) {
		  free(p1);
		  putcs ("      \n");
		  return;
		}
		free(p1);
	  }
	} else if (new_nn.ls == 0)
	  return;
  }
  if (new_nn.namef)
	for (i = 0; i < j; i++) {
	  /*        */
	  f_ex_num_news(k, new_nn.nsf[i]);
	}
  /*      */
  j = 0; /*  -     */
  put_db_news_num (&new_nn, k, num_tema, &s, &j);
  /*        */
  if(j) { /*      */
    aa_ex_num_news(&new_nn, k, s, j);
    free(s);
  }
  if (new_nn.namef)
	for (i = 0; i < new_nn.nf; i++) {
	  free (new_nn.namef[i]);
	  free (new_nn.nsf[i]);
	  free (new_nn.nameu[i]);
	}
  for (i = 0; i < n_sys_news; i++)
	if (sys_news[i].num == k) {
	  sys_news[i].time_new = time (0);
	  j = 0; /*  -     */
	  put_db_news (&sys_news[i], 1, &s, &j);
	  update_ex_sys_news(sys_news[i].num, s, j);
	  free(s);
	  for (j = 0; j < n_u_news; j++)
		if (u_news[j].num == k) {
		  put_db_un (&u_news[j], 1);
		  break;
		}
	  break;
	}
}

/* ,   ,    */
static void
sort_to (struct NUM_NEWS **pn, int un)
{
  int i, u;
  int chg_num;
  int *t;
  int *o;
  struct NUM_NEWS *pinf;
  struct NUM_NEWS *pinf2;
  int *p0;

  if (un == 0)
	return;
  t = (int *) alloca (un * sizeof (int));
  o = (int *) alloca (un * sizeof (int));
  p0 = (int *) alloca (un * sizeof (int));
  pinf = *pn;
  for (i = 0; i < un; i++) {
	p0[i] = i;
	t[i] = pinf[i].tema;
	o[i] = pinf[i].otvet;
  }
  for (i = 0; i < un; i++)
	for (u = i + 1; u < un; u++)
	  if (t[u] < t[i] || (t[u] == t[i] && o[u] < o[i])) {
		chg_num = p0[i];
		p0[i] = p0[u];
		p0[u] = chg_num;
		chg_num = t[i];
		t[i] = t[u];
		t[u] = chg_num;
		chg_num = o[i];
		o[i] = o[u];
		o[u] = chg_num;
	  }
  *pn = (struct NUM_NEWS *) mallocc (i * sizeof (struct NUM_NEWS));
  pinf2 = *pn;
  for (i = 0; i < un; i++)
	pinf2[i] = pinf[p0[i]];
  free (pinf);
}


static void
view_lst_konf (struct ALL_KONF *ac, int n, int flg_title)
{
  int j;
  int cn = 0;

  for (j = 0; j < n; j++) {
	if (cn >= 22) {
	  if (wait_cr ())
		break;
	  cn = 0;
	}
	if (cn == 0 && flg_title) {
	  putcs ("\n\
++. +------+   ++ / +-  -...\n");
	  cn += 2;
	}
	putcc (ac[j].flg_r ? '+' : '-');
	warn_d ("%4d %4d ", ac[j].num_news->tema, ac[j].num_news->otvet);
	warn_d ("%-12s ", ac[j].nf == 0 ?
	      ac[j].num_news->author : ac[j].num_news->nameu[ac[j].nf - 1]);
	warn_d ("%-9ld ", ac[j].sizef);
	if (ac[j].nf == 0) {
	  putcc (ac[j].num_news->otvet ? '' : '');
	  putcc (ac[j].num_news->nf ? '' : ' ');
	} else
	  putcs (" ");
	warn_d (" %s ", decode_inc_may_be_old(ac[j].nsf));
	if (ac[j].nf == 0)
	  warn_d ("%s\n", ac[j].num_news->rem);
	else {
	  char ns[16];
	  sprintf(ns, "#%-4d", ac[j].nf);
	  warn_d (" %s %s\n", ns, toshort(ac[j].namef, ns+2));
	}
	cn++;
  }
}

static void
put_regs_from_konf (struct ALL_KONF *all_conf, int k)
{
  struct tm *ptm;
  time_t tt;
  struct REG_S pregs;
  char *author;

  /*      */
  if (all_conf->nf != 0) {
	if (strcmp (all_conf->num_news->nameu[all_conf->nf - 1], pu_cur.user) == 0)
	    return;
  } else if (strcmp (all_conf->num_news->author, pu_cur.user) == 0)
	return;
  time (&tt);
  ptm = localtime (&tt);
  pregs.file = bb_asprintf ("%d/%d.%d", k, all_conf->num_news->tema,
						all_conf->num_news->otvet);
  strcpy (pregs.file_inc, all_conf->nsf);
  strcpy (pregs.user, pu_cur.user);
  sprintf (pregs.din, "+%02d.%02d %02d/%02d",
		   ptm->tm_hour, ptm->tm_min, ptm->tm_mday, ptm->tm_mon + 1);
  strcpy (pregs.rem, all_conf->nf == 0 ?
		    all_conf->num_news->rem :
		    f_prompt (" #%-4d %s", all_conf->nf,
		    toshort(all_conf->namef, "")));
  author = f_prompt("../%s", all_conf->nf == 0 ?
				all_conf->num_news->author :
				all_conf->num_news->nameu[all_conf->nf - 1]);
  if (chdir(author) == 0) {
	struct PU pu;

	put_db_reg (&pregs);
	author = strrchr(author, '/')+1;
	if(get_pu (&pu, author) > 0 && pu.ex_direction != host_direction)  {
	    /*     */
	    ex_cmd(EXCMD_NOTIFY_FILE, pregs.file_inc,
		    f_prompt ("%s/%s/%s", convert_din(pregs.file),
					    convert_din(pregs.din),
						convert_din(pregs.rem)),
		    &pu);
	}
	chdir (f_prompt ("../%s", pu_cur.user));
  }
  free(pregs.file);
}

static void
add_pu_news (struct U_NEWS *pu_news, struct ALL_KONF *all_conf, int in, int k)
{
  int j, i, n;

  j = 0;
  for (i = 0; i < in; i++) {
	if (all_conf[i].flg_r)          /*   */
	  continue;
	j++;
  }
  if (j) {                              /*   */
	n = pu_news->sz;
	pu_news->sz += j;
	pu_news->tof =
	  (char **) reallocc (pu_news->tof, (n + j) * sizeof (char *));
	pu_news->hash =
	  (long *) reallocc (pu_news->hash, (n + j) * sizeof (long));
	for (i = 0; i < in; i++)
	  if (all_conf[i].flg_r == 0) {
		pu_news->tof[n] = strcpym (all_conf[i].nsf);
		pu_news->hash[n] = hash_decode_inc (all_conf[i].nsf);
		put_regs_from_konf (&all_conf[i], k);
		n++;
	  }
	put_db_un (pu_news, 1);
  }
}


struct COPY_NAME_DLIST_FOR_CANSEL_GF
{
  char *name_dlist;
  struct ALL_KONF *pall_conf;
};

/* /      */
static void
gf_k (struct NUM_NEWS *news_s, struct U_NEWS *pu_news, int nn, int k,
	  int flg_re)
{
  int j, i, in;
  char *fname;
  char *tmpfname;
  struct ALL_KONF *all_conf;
  struct COPY_NAME_DLIST_FOR_CANSEL_GF *copy_name_dlist_for_cansel_gf;
  int ncp = 0;

  in = parser_konf (news_s, nn, pu_news, &all_conf);
  if (in == 0)
	return;
  copy_name_dlist_for_cansel_gf = (struct COPY_NAME_DLIST_FOR_CANSEL_GF *)
	mallocc (in * sizeof (struct COPY_NAME_DLIST_FOR_CANSEL_GF));
  clear_tmp (0);
  put_names (0, in);
  j = 0;
  for(i = 0; i < in; i++) {
    int tlen;

    if (all_conf[i].nf == 0) {
	char tmp_article_name[32];

	sprintf (tmp_article_name, "c%c%03d%03d.%d",
			   all_conf[i].num_news->otvet == 0 ? 't' : 'r',
			   all_conf[i].num_news->tema, all_conf[i].num_news->otvet, k);
	if((tlen=strlen(tmp_article_name)) > j)
	    j = tlen;
    } else {
	if((tlen=strlen(all_conf[i].namef)) > j)
	    j = tlen;
    }
  }
  fname = alloca(j + 1);
  tmpfname = alloca(j + 1 + 4);   /* + "/tmp" */
  for (i = 0; i < in; i++) {
	if (all_conf[i].nf == 0) {
	  sprintf (fname, "c%c%03d%03d.%d",
			   all_conf[i].num_news->otvet == 0 ? 't' : 'r',
			   all_conf[i].num_news->tema, all_conf[i].num_news->otvet, k);
	  if (all_conf[i].sizef == 0) {
		warn_d (" %d.%d \n", all_conf[i].num_news->tema,
						 all_conf[i].num_news->otvet);
		copy_name_dlist_for_cansel_gf[ncp].name_dlist = strcpym (fname);
		copy_name_dlist_for_cansel_gf[ncp].pall_conf = &all_conf[i];
		ncp++;
		continue;
	  }
	} else
	  strcpy (fname, all_conf[i].namef);
	for (j = 0; j < ncp; j++)
	  if (!strcmp (fname, copy_name_dlist_for_cansel_gf[j].name_dlist)) {
		putcs ("\
 //   ,\n\
      \n");
		clear_tmp (0);
		for (j = 0; j < ncp; j++)
		  copy_name_dlist_for_cansel_gf[ncp].pall_conf = 0;
		goto gf_all;
	  }
	put_names (fname, 0);
	sprintf (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY], "%d/%s", k,
			 all_conf[i].nsf);
	strcpy (tmpfname, "tmp/");
	strcat (tmpfname, fname);
	if (link (buf_news_empty, tmpfname))
	  putcs ("   .\n\
        .\n");
	else {
	  copy_name_dlist_for_cansel_gf[ncp].name_dlist = strcpym (fname);
	  copy_name_dlist_for_cansel_gf[ncp].pall_conf = &all_conf[i];
	  ncp++;
	}
  }
  j = my_scandir ("tmp");
  if (j <= 0) {
gf_all:
	j = 0;
	for (i = 0; i < ncp; i++) {
	  free (copy_name_dlist_for_cansel_gf[i].name_dlist);
	  if (copy_name_dlist_for_cansel_gf[i].pall_conf)
		all_conf[j++] = *copy_name_dlist_for_cansel_gf[i].pall_conf;
	}
	free (copy_name_dlist_for_cansel_gf);
	if (j)
	  add_pu_news (pu_news, all_conf, j, k);
	free (all_conf);
	return;
  }
  in = gf (j, flg_re);
  for (j = 0; j < in; j++)
	for (i = 0; i < ncp; i++)
	  if (!strcmp(name_dlist[j], copy_name_dlist_for_cansel_gf[i].name_dlist))
		copy_name_dlist_for_cansel_gf[i].pall_conf = 0;
  goto gf_all;
}


static void
compress_u_news (struct NUM_NEWS *to, int n, struct U_NEWS *uns)
{
  int i, j, nu;
  long hash;
  char *dim_cmp;
  char **pnf;
  long *phash;

  if (uns->sz == 0)
	return;
  dim_cmp = alloca (uns->sz);
  for (nu = 0; nu < uns->sz; nu++)
	dim_cmp[nu] = 0;
  for (i = 0; i < n; i++) {
	hash = hash_decode_inc (to[i].bb);
	for (nu = 0; nu < uns->sz; nu++)
	  if (dim_cmp[nu] == 0 && uns->hash[nu] == hash &&
				strcmp (to[i].bb, uns->tof[nu]) == 0) {
		dim_cmp[nu]++;
		break;
	  }
	for (j = 0; j < to[i].nf; j++) {
	  hash = hash_decode_inc (to[i].nsf[j]);
	  for (nu = 0; nu < uns->sz; nu++)
		if (dim_cmp[nu] == 0 && uns->hash[nu] == hash &&
			strcmp (to[i].nsf[j], uns->tof[nu]) == 0) {
		  dim_cmp[nu]++;
		  break;
		}
	}
  }
  j = 0;
  for (nu = 0; nu < uns->sz; nu++)
	if (dim_cmp[nu])
	  j++;
  if (j == uns->sz)
	return;
  warn_d ("\n   %d  \n", uns->sz - j);
  pnf = (char **) mallocc (j * sizeof (char *));
  phash = (long *) mallocc (j * sizeof (long));
  for (j = nu = 0; nu < uns->sz; nu++)
	if (dim_cmp[nu]) {
	  pnf[j] = uns->tof[nu];
	  phash[j++] = uns->hash[nu];
	} else
	  free (uns->tof[nu]);
  free (uns->tof);
  free (uns->hash);
  uns->tof = pnf;
  uns->hash = phash;
  uns->sz = j;
  put_db_un (uns, 2);   /* 2 -   */
}


static char *dlg_com[] = {
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "-",
  "",
  "\n",
  "",
  "",
  "",
  "",
  "",
  NULL
};

static char prompt[] =
  "\n (, , , , ..., -, ?)";

static const char alhlp[] = "\
        -    \n\
   [par] -    ,  \n\
		        \n\
       [par] -    \n\
      [par] -  ,     \n\
		     \n\
 [.] -      \n\
               -     \n\
           -    \n\
          -    \n\
      [par] -    \n\
  [par] -      \n\
      [par] -      \n\
		     \n\
             -   \n\
\n\
      [par]  ,    -\n\
 .      \n\
 : ? *.\n\n";

static int
konf (int k, int ns)
{
  struct IN_DLG k_dlg;
  int i, priv, j;
  char *s;
  struct NUM_NEWS *news_s;
  int nn;
  struct U_NEWS *pu_news;
  int in, ok;
  struct ALL_KONF *all_conf = 0;
  int ret;

  k_dlg.msgs = &dlg_com;
  k_dlg.hlps = &konf_dlg_hlps;
  k_dlg.prompt = prompt;
  k_dlg.alhlp = alhlp;
  k_dlg.zvezda = konf_zvezda;

  for (i = 0; i < sys_news[ns].nusers - 1; i++)
	for (j = i; j < sys_news[ns].nusers; j++)
	  if (strcmp (sys_news[ns].users[i], sys_news[ns].users[j]) > 0) {
		s = sys_news[ns].users[i];
		sys_news[ns].users[i] = sys_news[ns].users[j];
		sys_news[ns].users[j] = s;
		priv = sys_news[ns].priv[i];
		sys_news[ns].priv[i] = sys_news[ns].priv[j];
		sys_news[ns].priv[j] = priv;
	  }
  for (i = 0; i < sys_news[ns].nusers; i++)
	if (strcmp (sys_news[ns].users[i], pu_cur.user) == 0)
	  break;
  if (i == sys_news[ns].nusers || sys_news[ns].priv[i] == 0) {
  errp:
	putcs ("        \n");
	return 0;
  }
  priv = sys_news[ns].priv[i];
  for (i = 0; i < n_u_news; i++)
	if (u_news[i].num == sys_news[ns].num)
	  break;
  if (i == n_u_news)
	goto errp;
  pu_news = &u_news[i];
  warn_d ("\n : %s[%d]\n", sys_news[ns].name, k);
  warn_d (": %s\n", sys_news[ns].moder);
  warn_d ("   : ");
  put_access (priv);
  putcc ('\n');
  news_s = 0;
  nn = 0;
  ret = -1;
  for (;;) {
	if (news_s != 0 && (ret >= 0 || stat_nn (k, 1) != 0)) {
	  for (i = 0; i < nn; i++) {
		for (j = 0; j < news_s[i].nf; j++) {
		  free (news_s[i].namef[j]);
		  free (news_s[i].nsf[j]);
		  free (news_s[i].nameu[j]);
		}
		free0 (&news_s[i].namef);
		free0 (&news_s[i].nsf);
		free0 (&news_s[i].nameu);
		free0 (&news_s[i].sizef);
	  }
	  free0 (&news_s);
	  nn = 0;
	}
	free0 (&all_conf);
	if (ret >= 0)
	  return ret;
	if (nn == 0) {
	  nn = get_db_news_num (&news_s, k);
	  sort_to (&news_s, nn);
	  compress_u_news (news_s, nn, pu_news);
	}
	flg_auto_cr = 1;
	i = scan_input (&k_dlg);
	flg_auto_cr = 0;
	switch (i) {
	case 1:                                 /*  */
	  if (nn == 0) {
		putcs (" \n");
		break;
	  }
	  if ((priv & PRIV_CH) == 0)
		putcs (
"    ()       .\n");
	  else if ((j = parser_konf (news_s, nn, pu_news, &all_conf))) {
		ok = 0;
		for (i = 0; i < j; i++)
		  if (all_conf[i].flg_r) {
			ok++;
			free0 (&pu_news->tof[all_conf[i].u_news]);
		  }
		if (ok) {
		  char **pnf;
		  pnf = (char **) mallocc ((pu_news->sz - ok) * sizeof (char *));
		  for (j = i = 0; i < pu_news->sz; i++)
			if (pu_news->tof[i])
			  pnf[j++] = pu_news->tof[i];
		  free0 (&pu_news->tof);
		  pu_news->tof = pnf;
		  pu_news->sz = j;
		  warn_d (" : %d\n", ok);
		  put_db_un (pu_news, 1);
		}
	  }
	  break;
	case 2:                                 /*  */
	  if (nn == 0) {
		putcs (" \n");
		break;
	  }
	  if ((priv & PRIV_CH) == 0 && (priv & PRIV_4) == 0 &&
						(priv & PRIV_5) == 0)
		  putcs (
"    (), `4' ( )  `5' ( )\n\
      .\n");
	  else
		gf_k (news_s, pu_news, nn, k, 1);
	  break;
	case 3:                                 /*  */
	  s = two_e_word ();
	  if (s) {
		j = atoi (s);
		if (j == k) {
		  putcs ("     \n");
		  break;
		}
	  }
	  ret = 2;
	  break;
	case 4:                                 /*  */
	  if ((priv & PRIV_Z) == 0 && (priv & PRIV_7) == 0)
		putcs(
"    ()  `7'( )   \n\
   .\n");
	  else {
		s = two_e_word ();
		if (s == NULL)
		  s = editstring (" ");
		j = atoi (s);
		for (i = 0; i < nn; i++)
		  if (news_s[i].tema == j)
			break;
		if (i == nn)
		  putcs ("  \n");
		else
		  put_to (priv, k, j, news_s[i].rem);
	  }
	  break;
	case 5:                                 /*  */
	  if (nn == 0) {
		putcs (" \n");
		break;
	  }
	  if ((priv & PRIV_CH) == 0 && (priv & PRIV_4) == 0 &&
					      (priv & PRIV_5) == 0)
		  putcs(
"    (), `4' ( )  `5' ( )\n\
      .\n");
	  else
		gf_k (news_s, pu_news, nn, k, 0);
	  break;
	case 6:                                 /*  */
	  if ((priv & PRIV_Z) == 0 && (priv & PRIV_6) == 0)
		putcs(
"    ()  `6'( )   \n\
   .\n");
	  else {
		s = two_e_word ();
		if (s == NULL)
		  s = editstring (" : [.]");
		i = g_int (&s);
		if (i <= 0 || (*s != 0 && *s != '.')) {
		  putcs ("  \n");
		  break;
		}
		for (j = 0; j < nn; j++)
		  if (news_s[j].tema == i && news_s[j].otvet == 0)
			break;
		if (j == nn) {
		  putcs ("  \n");
		  break;
		}
		in = i;
		if (*s == '.' && *++s != 0) {
		  if ((priv & PRIV_Z) == 0) {
			putcs ("    () \n");
			break;
		  }
		  i = g_int (&s);
		  if (i <= 0 || *s != 0) {
			putcs ("  \n");
			break;
		  }
		  for (j = 0; j < nn; j++)
			if (news_s[j].tema == in && news_s[j].otvet == i)
			  break;
		  if (j == nn) {
			putcs ("  \n");
			break;
		  }
		}
		/*      */
		fixeditstr (f_prompt ("P %d.%d", news_s[j].tema, news_s[j].otvet));
		i = parser_konf (news_s, nn, pu_news, &all_conf);
		if (i == 0) {
		  putcs (" \n");
		  break;
		}
		view_lst_konf (all_conf, 1, 1);
		if (all_conf[0].num_news->nf) {
		  fixeditstr (f_prompt ("P %d.%d+", news_s[j].tema, news_s[j].otvet));
		  i = parser_konf (news_s, nn, pu_news, &all_conf);
		  if (i == 0) {
			putcs (" \n");
			break;
		  }
		  view_lst_konf (all_conf, i, 0);
		}
		paste_to (&news_s[j], pu_news, k);
	  }
	  break;
	case 7:                                 /*  */
	  if (nn == 0) {
		putcs (" \n");
		break;
	  }
	  if ((j = parser_konf (news_s, nn, pu_news, &all_conf)))
		view_lst_konf (all_conf, j, 1);
	  break;
	case 8:                                 /*  */
	  if ((priv & PRIV_R) == 0) {
		putcs(
"    ()   \n\
   .\n");
		break;
	  }
	  chg_status (ns);
	  break;
	case 9:                                 /*  */
	  if ((priv & PRIV_Z) == 0 && (priv & PRIV_6) == 0)
		putcs(
"    ()  `6'( )   \n\
   .\n");
	  else {
		if(two_e_word ()) {
		    putcs("     .\n");
		    break;
		}
		put_to (priv, k, 0, 0);
	  }
	  break;
	case 10:                                        /*  */
	  list_members (ns);
	  break;
	case 11:                                        /*  */
	  if (nn == 0) {
		putcs (" \n");
		break;
	  }
	  if ((priv & PRIV_U) == 0 && (priv & PRIV_O) == 0) {
		putcs(
"    ()  ()   \n\
   .\n");
		break;
	  }
	  if ((j = parser_konf (news_s, nn, pu_news, &all_conf))) {
		int prav_o = 0;
		for (i = 0; i < j; i++) {
		  view_lst_konf (&all_conf[i], 1, 1);
		  if ((priv & PRIV_O) == 0 && prav_o == 0 &&
			  strcmp (all_conf[i].nameu, pu_cur.user) != 0) {
			putcs(
"    ()   \n\
    .\n");
			prav_o++;
			continue;
		  }
		  if (all_conf[i].nf == 0)      /*  */
			if (all_conf[i].num_news->nf) {
			  putcs(
"      \n");
			  continue;
			}
		  if (yes_no ("")) {
			if (all_conf[i].nf == 0) {
			    /*  */
			  unlink_db_news_num (all_conf[i].num_news, k);
			  del_ex_num_news(k, all_conf[i].num_news, 0,
					    all_conf[i].num_news->bb);
			} else {
			  put_db_news_num_del (all_conf[i].num_news, k,
							    all_conf[i].nsf);
			  del_ex_num_news(k, all_conf[i].num_news, 1,
							    all_conf[i].nsf);
			}
			putcs ("\n");
		  }
		}
	  }
	  break;
	case 12:                                        /*  */
	  if (nn == 0) {
		putcs (" \n");
		break;
	  }
	  if ((priv & PRIV_CH) == 0 && (priv & PRIV_4) == 0
				      && (priv & PRIV_5) == 0) {
		putcs(
"    (), `4' ( )  `5' ( )\n\
      .\n");
		break;
	  }
	  if ((j = parser_konf (news_s, nn, pu_news, &all_conf))) {
		int ok = 0;
		for (i = 0; i < j; i++)
		  if (all_conf[i].nf)
			break;
		  else if (all_conf[i].sizef == 0)
			ok++;
		if (i != j) {
		  putcs ("       \n");
		  break;
		}
		if (ok != 0) {
		  if (ok == j) {
			putcs ("     \n");
			add_pu_news (pu_news, all_conf, j, k);
			break;
		  }
		  putcs ("      \n");
		  ok = 0;
		  for (i = 0; i < j; i++)
			if (all_conf[i].sizef != 0)
			  all_conf[ok++] = all_conf[i];
		  j = ok;
		}
		for (i = 0; i < j;) {
		  sprintf (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY],
				   "%d/%s", k, all_conf[i].nsf);
		  warn_d ("\n %d  %d\n\n",
					all_conf[i].num_news->tema,
					    all_conf[i].num_news->otvet);
		  i++;
		  ok = putcs_mnogo_in_file (buf_news_empty);
		  if (ok < 0)
			putcs (" \n");
		  else if (ok > 0 && i != j)
			if (wait_cr ())
			  break;
		}
		add_pu_news (pu_news, all_conf, i, k);
	  }
	  break;
	case 13:                                        /* - */
	  ret = 0;
	  break;
	case 14:                                        /*  */
	  ret = 100;
	  break;
	case 15:                                        /* \n */
	  break;
	case 16:                                        /*  */
	  ret = 1;
	  break;
	case 17:                                        /*  */
	  ret = 3;
	  break;
	case 18:                                        /*  */
	  ret = 4;
	  break;
	case 19:                                        /*  */
	  ret = 5;
	  break;
	case 20:                                        /*  */
	  ret = 6;
	  break;
	}
  }
}

static void
view_list_konf (int i, int flg_new)
{
  int k;

  if (flg_new == 0)
	putcc (' ');
  else
	putcc (flg_new > 0 ? '!' : '-');
  warn_d ("%-3d%s", sys_news[i].num, sys_news[i].name);
  k = strlen (sys_news[i].name);
  while (k < LEN_NAME_KONFY) {
	putcc (' ');
	k++;
  }
  putcs (sys_news[i].moder);
  k = strlen (sys_news[i].moder);
  while (k < (MAXPU_LEN + 1)) {
	putcc (' ');
	k++;
  }
  for (k = 0; k < sys_news[i].nusers; k++)
	if (!strcmp (sys_news[i].users[k], pu_cur.user))
	  break;
  if (k != sys_news[i].nusers)
	k = sys_news[i].priv[k];
  else
	k = 0;
  put_access (k);
  putcc ('\n');
}

static FILE *
news_lock (int k, int flg_exclusive)
{
  FILE *flk;
  int type_lock;

  stat_nn (k, 0);                     /*  patch    */
  strcat (buf_news_empty, "l");       /* 1.dbl */
  flk = fopen (buf_news_empty, "a");
  if (flk == NULL) {
	error_d (91, "     : %s: %m", buf_news_empty);
  }
  type_lock = LOCK_NB;
  type_lock |= flg_exclusive ? LOCK_EX : LOCK_SH;
  if (flock (fileno (flk), type_lock) < 0) {
	warn_d (flg_exclusive ? "     \n" :
			"  \n");
	fclose (flk);
	return 0;
  }
  return flk;
}

int
dlg_konf (void)
{
  static char head_news[] = "\n\
 # +----  ----------------------------------+ +--+ ++\n";
  char *s;
  int k, i;
  FILE *flk;

  scan_new_konf ();
  s = two_e_word ();
  for (;;) {
	if (s == NULL) {
	  flg_auto_cr = 1;
	  s = editstring (
		    "  (<>, <ENTER>, *, +, -, !, ?)");
	  flg_auto_cr = 0;
	}
	if (*s == '-')
	  return 0;
	else if (*s == 0) {
	  if (n_u_news == 0)
		putcs ("   \n");
	  else {
		if (n_sys_news) {
		  putcs (head_news);
		  for (i = 0; i < n_u_news; i++)
			for (k = 0; k < n_sys_news; k++)
			  if (sys_news[k].num == u_news[i].num)
				view_list_konf (k, sys_news[k].time_new > u_news[i].time_lr);
		} else
		  putcs (" \n");
	  }
	} else if (*s == '+') {
	  if ((pu_cur.priv & GRANTS_NEWSMASTER) == 0)
		putcs ("      .\n");
	  else {
		struct PU pu;
		struct SYS_NEWS nns;

		k = 0;
		nns.time_new = time (0);
		for (i = 0; i < n_sys_news; i++)
		  if (sys_news[i].num > k)
			k = sys_news[i].num;
		k++;                                    /*   */
		do {
		  putcs(
		"    `-'   \n");
		  s = editrem ("(max 47 )");
		}
		while (*s == 0);
		if (*s != '-') {
		  s[LEN_NAME_KONFY - 1] = 0;
		  strcpy (nns.name, s);
		  do {
			flg_auto_cr = 1;
			s = editstring(
"  -  `-'   ");
			flg_auto_cr = 0;
		  } while (*s == 0);
		  if (*s != '-') {
			if (get_pu (&pu, s) == 0)
			  warn_d (" %s    \n", s);
			else {
			  s = pu.user;
			  strcpy (nns.moder, pu.user);
			  nns.num = k;
			  k = 1 + (strcmp (s, Adm) != 0);
			  nns.nusers = k;
			  nns.users = (char **) alloca (k * sizeof (char *));
			  nns.priv = (int *) alloca (k * sizeof (int));
			  for (i = 0; i < k; i++) {
				if (i)
				  s = Adm;
				nns.users[i] = alloca (strlen (s) + 1);
				strcpy (nns.users[i], s);
				nns.priv[i] = PRIV_ALL; /*   */
			  }
			  i = 0; /*     */
			  put_db_news (&nns, 0, &s, &i);
			  update_ex_sys_news(-nns.num, s, i); /*  */
			  free(s);
			  scan_new_konf ();
			}
		  }
		}
	  }
	} else if (*s == '*' || *s == 'S') {
	  if (n_sys_news) {
		int flg_new;
		putcs (head_news);
		for (i = 0; i < n_sys_news; i++) {
		  for (k = 0; k < n_u_news; k++)
			if (sys_news[i].num == u_news[k].num)
			  break;
		  if (k == n_u_news)
			flg_new = -1;
		  else if (sys_news[i].time_new > u_news[k].time_lr)
			flg_new = 1;
		  else
			flg_new = 0;
		  view_list_konf (i, flg_new);
		}
	  } else
		putcs (" \n");
	} else if (*s == 'U') {
	  if ((pu_cur.priv & GRANTS_NEWSMASTER) == 0)
		putcs ("     .\n");
	  else {
		s = two_e_word();
		if (s == 0) {
		  flg_auto_cr = 1;
		  s = editstring ("   ");
		  flg_auto_cr = 0;
		  if (*s == 0 || *s == '-') {
			s = NULL;
			continue;
		  }
		}
		k = g_int (&s);
		if(*s != 0 || k<=0) {
		    putcs ("  \n");
		    s = NULL;
		    continue;
		}
		for (i = 0; i < n_sys_news; i++)
		  if (sys_news[i].num == k)
			break;
		if (i != n_sys_news) {
		  flk = news_lock (k, 1);
		  if (flk != NULL) {
			if (yes_no (f_prompt (" %s ", s))) {
			  unlink_db_news (&sys_news[i]);
			  if(have_ext_users()) {
			    /*    */
			    ex_cmd(EXCMD_DELETE_KONF,
				    f_prompt ("%d", sys_news[i].num),
				    0, &pu_all_user);
			  }
			  scan_new_konf ();
			}
			fclose (flk);
		  }
		} else
		  putcs ("  \n");
	  }
	} else if (*s == '?')
	  putcs ("\
<> -   \n\
<ENTER> -    \n\
!<>-     \n\
`*'     -    \n\
`+'     -   \n\
''     -  \n\
`-'     -     \n\n");
	else if (*s >= '0' && *s <= '9') {
	  k = atoi(s);
	  for (i = 0; i < n_sys_news; i++)
		if (sys_news[i].num == k)
		  break;
	  if (i != n_sys_news) {
		int ns = i;

		for (i = 0; i < n_u_news; i++)
		  if (u_news[i].num == k)
			break;
		if (i == n_u_news) {    /*      */
		  struct U_NEWS u_news_new;

		  u_news_new.num = k;
		  u_news_new.sz = 0;
		  put_db_un (&u_news_new, 0);
		  /*     */
		  get_udb_news ();
		} else {
		  /*     */
		  put_db_un (&u_news[i], 1);
		}
		flk = news_lock (k, 0);
		if (flk != NULL) {
		  i = konf (k, ns);
		  fclose (flk);
		  return i;
		}
	  } else
		putcs ("  \n");
	} else if (*s == '!') {
	  while (*++s == ' ');
	  if (*s >= '0' && *s <= '9') {
		k = atoi (s);
		for (i = 0; i < n_u_news; i++)
		  if (u_news[i].num == k)
			break;
		if (i != n_u_news) {
		  unlink_db_un (&u_news[i]);
		  get_udb_news ();
		  putcs ("    \n");
		} else
		  putcs ("     \n");
	  } else
		putcs (" !<>\n");
	} else if (*s == 'P' && *(s+1) == 'O' && *(s+2) == 'K') {
	    return 100;
	} else
	  putcs ("  \n");
	s = NULL;
  }
  return 0;
}
