#include "fun.h"
#include "db.h"
#include "keyboard.h"
#include <gdbm.h>
#include <errno.h>

static void
open_db_writer (GDBM_FILE * dbf, int namedb)
{
  do {    /*     */
	*dbf = gdbm_open (db_names[namedb], 512, GDBM_WRITER, 0600, 0);
	if (gdbm_errno == GDBM_CANT_BE_WRITER)
	  sleep (1);
	else if (gdbm_errno != GDBM_NO_ERROR) {
	  put_log ("Error open for read %s\n", db_names[namedb]);
	  error_d (2, "   %s", db_names[namedb]);
	}
  } while (*dbf == NULL);
}

static void
open_db_reader (GDBM_FILE * dbf, int namedb)
{
  do {    /*     */
	*dbf = gdbm_open (db_names[namedb], 512, GDBM_READER, 0600, 0);
	if (gdbm_errno == GDBM_CANT_BE_READER)
	  sleep (1);
	else if (gdbm_errno != GDBM_NO_ERROR) {
	  put_log ("Error open for write %s\n", db_names[namedb]);
	  error_d (2, "    %s", db_names[namedb]);
	}
  } while (*dbf == NULL);
}

int
stat_db_chg (int namedb) /* 1 -    */
{
  static time_t mtime[N_DB_NUM];
  struct stat b;
  if (stat (db_names[namedb], &b) != 0) {
	warn_d ("   stat   %s\n", db_names[namedb]);
	return 1;
  }
  if (mtime[namedb] < b.st_mtime) {
	mtime[namedb] = b.st_mtime;
	return 1;
  }
  return 0;
}


int
all_abonents (struct PU **ppu)
{
  int i, un;
  struct PU *pu;

  GDBM_FILE dbf;
  datum key, content;

  open_db_reader (&dbf, N_DB_USR);
  key.dptr = "*";
  key.dsize = 2;
  content = gdbm_fetch (dbf, key);
  if (content.dptr == NULL) {
rete:
	warn_d ("   !\n");
	return 0;
  } else
	un = atoi (content.dptr);
  pu = (struct PU *) mallocc (un * sizeof (struct PU));
  *ppu = pu;
  key = gdbm_firstkey (dbf);
  for (i = 0; i <= un; i++) {
	if (key.dptr == NULL)
	  goto rete;
	content = gdbm_fetch (dbf, key);
	if (content.dptr == NULL)
	  goto rete;
	if (strcmp (key.dptr, "*")) {
	  form_r_pu (key.dptr, content.dptr, content.dsize, pu);
	  pu++;
	}
	content = gdbm_nextkey (dbf, key);
	key = content;
  }
  gdbm_close (dbf);
  return un;
}

int
db_add_user (struct PU *pu, int flg_chg)
{
  int ret;

  GDBM_FILE dbf;
  datum key, content;
  datum key_num, content_num;

  open_db_writer (&dbf, N_DB_USR);
  if (flg_chg == 0) {
	key_num.dptr = "*";
	key_num.dsize = 2;
	content_num = gdbm_fetch (dbf, key_num);
  }
  key.dptr = form_w_u (pu->user);
  key.dsize = LEN_UDB_KEY;
  content.dptr = form_w_pu (pu, &content.dsize);
  sleep_kill_start ();
  ret = gdbm_store (dbf, key, content, flg_chg ? GDBM_REPLACE : GDBM_INSERT);
  free (content.dptr);
  if (ret != 0)
	ret = 0;
  else {
	if (flg_chg == 0) {
	  char strb[16];
	  ret = atoi (content_num.dptr);
	  ret++;
	  sprintf (strb, "%d", ret);
	  content_num.dptr = strb;
	  content_num.dsize = strlen (strb) + 1;
	  gdbm_store (dbf, key_num, content_num, GDBM_REPLACE);
	} else
	  ret = 1;
  }
  sleep_kill_stop ();
  gdbm_close (dbf);
  return ret;
}

int
del_user (char *u)
{
  GDBM_FILE dbf;
  int ret;
  int err = 0;
  datum key;
  datum content_num;

  open_db_writer (&dbf, N_DB_USR);
  key.dptr = form_w_u (u);
  key.dsize = LEN_UDB_KEY;
  sleep_kill_start ();
  if (gdbm_exists (dbf, key) <= 0 || gdbm_delete (dbf, key) != 0) {
	warn_d ("   \n");
	err++;
  } else {
	char strb[16];
	gdbm_reorganize (dbf);
	key.dptr = "*";
	key.dsize = 2;
	content_num = gdbm_fetch (dbf, key);
	ret = atoi (content_num.dptr);
	ret--;
	sprintf (strb, "%d", ret);
	content_num.dptr = strb;
	content_num.dsize = strlen (strb) + 1;
	gdbm_store (dbf, key, content_num, GDBM_REPLACE);
  }
  sleep_kill_stop ();
  gdbm_close (dbf);
  return err;
}


/*
  
key=file_inc/user    user - -  file
:
r           %c     ('0'|'1')
d           %c       
new         %c    
mm.ss dd.mm %s     (11+'\0')
file        %s    
rem         %s  
*/

#define LEN_REG_KEY (INC_KEY_SIZE+MAXPU_LEN+2)

int
get_db_reg (struct REG_S **reg_s)
{
  GDBM_FILE dbf;
  datum key, content;
  struct REG_S *preg;
  int un, i;
  char *s;

  open_db_reader (&dbf, N_DB_REG);
  key = gdbm_firstkey (dbf);
  for (i = 0;; i++) {
	if (key.dptr == NULL)
	  break;
	content = gdbm_fetch (dbf, key);
	if (content.dptr == NULL) {
  errr:
	  error_d (1, "   ");
	}
	content = gdbm_nextkey (dbf, key);
	key = content;
  }
  un = i;
  if (un) {
	preg = (struct REG_S *) mallocc (i * sizeof (struct REG_S));
	*reg_s = preg;
	key = gdbm_firstkey (dbf);
	for (i = 0; i < un; i++) {
	  if (key.dptr == NULL)
		goto errr;
	  content = gdbm_fetch (dbf, key);
	  if (content.dptr == NULL)
		goto errr;
	  s = key.dptr;
	  while (*s)
		if (*s == '/')
		  break;
		else
		  s++;
	  if (*s == 0)
		goto errr;
	  strncpy (preg->file_inc, key.dptr, (s - key.dptr));
	  preg->file_inc[(s - key.dptr)] = 0;
	  s++;
	  strcpy (preg->user, s);
	  s = content.dptr;
	  if (*s != '0' && *s != '1')
		goto errr;
	  preg->flg_r = *s++;
	  if (*s != '0' && *s != '1')
		goto errr;
	  preg->flg_d = *s++;
	  if (*s != '0' && *s != '1')
		goto errr;
	  preg->flg_new = *s++;
	  strcpy (preg->din, s);
	  s += strlen (s) + 1;
	  preg->file = strcpym(s);
	  s += strlen (s) + 1;
	  strcpy (preg->rem, s);
	  decode_inc (preg->file_inc, preg->dfile);
	  preg++;
	  content = gdbm_nextkey (dbf, key);
	  key = content;
	}
  }
  gdbm_close (dbf);
  return un;
}

void
set_r_reg (struct REG_S *preg, int n)
{
  char f_key[LEN_REG_KEY];
  GDBM_FILE dbf;
  datum key, content;
  int i , ok = 0;
  int flg_open = 0;

  for (i = 0; i < n; i++)
	if (preg[i].flg_r == 'Y') {
	  preg[i].flg_r = '1';
	  memset (f_key, 0, LEN_REG_KEY);
	  strcpy (f_key, preg[i].file_inc);
	  strcat (f_key, "/");
	  strcat (f_key, preg[i].user);
	  key.dptr = f_key;
	  key.dsize = LEN_REG_KEY;
	  if(flg_open == 0) {
	    flg_open++;
	    open_db_writer (&dbf, N_DB_REG);
	  }
	  content = gdbm_fetch (dbf, key);
	  if (content.dptr != NULL) {
		content.dptr[0] = '1';
		if(ok == 0) {
		    ok++;
		    sleep_kill_start ();
		}
		gdbm_store (dbf, key, content, GDBM_REPLACE);
	  }
	}
  if(flg_open) {
    gdbm_close (dbf);
    if(ok)
	sleep_kill_stop ();
  }
}

void
set_d_reg (struct REG_S *preg)
{
  char f_key[LEN_REG_KEY];
  GDBM_FILE dbf;
  datum key, content;

  open_db_writer (&dbf, N_DB_REG);
  memset (f_key, 0, LEN_REG_KEY);
  strcpy (f_key, preg->file_inc);
  strcat (f_key, "/");
  strcat (f_key, preg->user);
  key.dptr = f_key;
  key.dsize = LEN_REG_KEY;
  content = gdbm_fetch (dbf, key);
  sleep_kill_start ();
  if (content.dptr != NULL) {
	content.dptr[1] = preg->flg_d;
	gdbm_store (dbf, key, content, GDBM_REPLACE);
  }
  gdbm_close (dbf);
  sleep_kill_stop ();
}

void
unset_new_reg (struct REG_S *preg, int n)
{
  char f_key[LEN_REG_KEY];
  GDBM_FILE dbf;
  datum key, content;
  int i , ok = 0;
  int flg_open = 0;

  for (i = 0; i < n; i++, preg++) {
	if (preg->flg_new != '2')
	  continue;
	preg->flg_new = '0';
	memset (f_key, 0, LEN_REG_KEY);
	strcpy (f_key, preg->file_inc);
	strcat (f_key, "/");
	strcat (f_key, preg->user);
	key.dptr = f_key;
	key.dsize = LEN_REG_KEY;
	if(flg_open == 0) {
	    flg_open++;
	    open_db_writer (&dbf, N_DB_REG);
	}
	content = gdbm_fetch (dbf, key);
	if (content.dptr != NULL) {
	  content.dptr[2] = '0';
	  if(ok == 0) {
	    ok++;
	    sleep_kill_start ();
	  }
	  gdbm_store (dbf, key, content, GDBM_REPLACE);
	}
  }
  if(flg_open) {
    gdbm_close (dbf);
    if(ok)
	sleep_kill_stop ();
  }
}

void
put_db_reg (struct REG_S *preg)
{
  char f_key[LEN_REG_KEY];
  char *dc;
  char *s;
  GDBM_FILE dbf;
  datum key, content;

  memset (f_key, 0, LEN_REG_KEY);
  strcpy (f_key, preg->file_inc);
  strcat (f_key, "/");
  strcat (f_key, preg->user);
  key.dptr = f_key;
  key.dsize = LEN_REG_KEY;
  dc = alloca(3+strlen(preg->din)+strlen(preg->file)+strlen(preg->rem)+3);
  s = dc;
		/*   : */
  *s++ = '0';   /* flg_r   -   */
  *s++ = '0';   /* flg_d   -   */
  *s++ = '1';   /* flg_new -  */
  strcpy (s, preg->din);
  s += strlen (s) + 1;
  strcpy (s, preg->file);
  s += strlen (s) + 1;
  strcpy (s, preg->rem);
  s += strlen (s) + 1;
  content.dptr = dc;
  content.dsize = (s-dc);
  open_db_writer (&dbf, N_DB_REG);
  sleep_kill_start ();
  gdbm_store (dbf, key, content, GDBM_INSERT);
  gdbm_close (dbf);
  sleep_kill_stop ();
}

void
unlink_db_reg (struct REG_S *preg, int n)
{
  GDBM_FILE dbf;
  int i, ok;
  char f_key[LEN_REG_KEY];
  datum key;

  open_db_writer (&dbf, N_DB_REG);
  ok = 0;
  sleep_kill_start ();
  for (i = 0; i < n; i++) {
	if (preg->flg_d != '0') {
	  memset (f_key, 0, LEN_REG_KEY);
	  strcpy (f_key, preg->file_inc);
	  strcat (f_key, "/");
	  strcat (f_key, preg->user);
	  key.dptr = f_key;
	  key.dsize = LEN_REG_KEY;
	  if (gdbm_exists (dbf, key) <= 0 || gdbm_delete (dbf, key) != 0)
		warn_d ("      \n");
	  else
		ok++;
	}
	preg++;
  }
  if (ok)
	gdbm_reorganize (dbf);
  sleep_kill_stop ();
  gdbm_close (dbf);
}

/*
   
key=file_inc
:
r              %c    ('0'|'1')
new            %c    
d              %c       
user   - -  file
file           %s   
rem   %s
d   -   %s
size %d
*/

#define LEN_INF_OUF_KEY INC_KEY_SIZE

int
get_db_inf (struct INF_S **pinf_s)
{
  GDBM_FILE dbf;
  datum key, content;
  struct INF_S *pinf;
  int un, i;
  char *s;

  open_db_reader (&dbf, N_DB_INF);
  key = gdbm_firstkey (dbf);
  for (i = 0;; i++) {
	if (key.dptr == NULL)
	  break;
	content = gdbm_fetch (dbf, key);
	if (content.dptr == NULL) {
errr:
	  error_d (1, "    ");
	}
	content = gdbm_nextkey (dbf, key);
	key = content;
  }
  un = i;
  if (un) {
	pinf = (struct INF_S *) mallocc (i * sizeof (struct INF_S));
	*pinf_s = pinf;
	key = gdbm_firstkey (dbf);
	for (i = 0; i < un; i++) {
	  if (key.dptr == NULL)
		goto errr;
	  content = gdbm_fetch (dbf, key);
	  if (content.dptr == NULL)
		goto errr;
	  strcpy (pinf->file_inc, key.dptr);
	  s = content.dptr;
	  if (*s != '0' && *s != '1')
		goto errr;
	  pinf->flg_r = *s++;
	  if (*s != '0' && *s != '1')
		goto errr;
	  pinf->flg_new = *s++;
	  if (*s != '0' && *s != '1')
		goto errr;
	  pinf->flg_d = *s++;
	  strcpy (pinf->user, s);
	  s += strlen (s) + 1;
	  pinf->file = strcpym(s);
	  s += strlen (s) + 1;
	  strcpy (pinf->rem, s);
	  s += strlen (s) + 1;
	  strcpy (pinf->d, s);
	  s += strlen (s) + 1;
	  pinf->sz = atol (s);
	  /* pinf->chg_info=0; pinf->flg_uo=0; */
	  pinf++;
	  content = gdbm_nextkey (dbf, key);
	  key = content;
	}
  }
  gdbm_close (dbf);
  return un;
}

int
test_present_file (char *user, struct INF_S *pinf, struct OUTF_S *p_of,
				   int npof)
{
  GDBM_FILE dbf;
  datum key, content;
  int flg_found = 0;
  char *s;
  char user_patch[MAXPU_LEN + 4];       /* ../ */
  char *file_inc;
  char *file;
  char *pinf_user;

  strcpy (user_patch, "../");
  strcat (user_patch, user);
  if (chdir (user_patch)) {
	warn_d ("   \n");
	return -1;
  }
  open_db_reader (&dbf, N_DB_INF);
  chdir ("..");
  chdir (pu_cur.user);
  key = gdbm_firstkey (dbf);
  for (;;) {
	int flg_d;
	long sz;

	if (key.dptr == NULL)
	  break;
	content = gdbm_fetch (dbf, key);
	if (content.dptr == NULL) {
	errr:
	  warn_d ("       %s\n", user);
	  return -1;
	}
	file_inc = key.dptr;
	s = content.dptr;
	if (*s != '0' && *s != '1')
	  goto errr;
	s++;                                            /* pinf->flg_r */
	if (*s != '0' && *s != '1')
	  goto errr;
	s++;                                            /* pinf->flg_new */
	if (*s != '0' && *s != '1')
	  goto errr;
	flg_d = *s++;
	pinf_user = s;
	s += strlen (s) + 1;
	file = s;
	s += strlen (s) + 1;    /* skip file */
	s += strlen (s) + 1;    /* skip rem */
	s += strlen (s) + 1;    /* skip data */
	sz = atol (s);
	if (strcmp (file, pinf->file) == 0 && flg_d == '0') {
	  /* :    */
	  char path_tst_uo[10 + MAXPU_LEN + INC_KEY_SIZE]; /* ../user/out/file */
	  struct stat statb;
	  FILE *cmp1, *cmp2;
	  int c1, c2;

	  sprintf (path_tst_uo, "../%s/out/%s", pinf_user, file_inc);
	  if (stat (path_tst_uo, &statb))
		goto lnext;

	  flg_found = 1;

	  /* :     */
	  if (strcmp (pinf_user, pu_cur.user) == 0) {
		/* :      */
		while (--npof >= 0) {
		  if (strcmp (p_of[npof].file_inc, file_inc) == 0 &&
			  p_of[npof].flg_d != 0) flg_found = 0;
		  goto lnext;
		}
		flg_found |= 2;
	  }
	  if (statb.st_size != sz)
		break;
	  /*   */
	  cmp1 = fopen (path_tst_uo, "r");
	  if (cmp1 == NULL) {
		flg_found = 0;                  /*    */
		goto lnext;
	  }
	  sprintf (path_tst_uo, "out/%s", pinf->file_inc);
	  if ((cmp2 = fopen (path_tst_uo, "r")) == NULL)
		error_d (11, "Hmmm,    ?");
	  for (;;) {
		c1 = getc (cmp1);
		c2 = getc (cmp2);
		if (c1 != c2)
		  break;
		if (c1 == EOF) {
		  flg_found |= 4;
		  break;
		}
	  }
	  fclose (cmp1);
	  fclose (cmp2);
	  break;
	}
  lnext:
	content = gdbm_nextkey (dbf, key);
	key = content;
  }
  gdbm_close (dbf);
  return flg_found;
}

int
get_r_inf (char *file_inc, char *user)
{
  GDBM_FILE dbf;
  datum key, content;
  int i;
  char buf_key[LEN_INF_OUF_KEY];
  char user_patch[MAXPU_LEN + 4];       /* ../ */

  strcpy (user_patch, "../");
  strcat (user_patch, user);
  if (chdir (user_patch)) {
	warn_d ("   \n");
	return 1;
  }
  open_db_reader (&dbf, N_DB_INF);
  key.dptr = memset (buf_key, 0, LEN_INF_OUF_KEY);
  strcpy (buf_key, file_inc);
  key.dsize = LEN_INF_OUF_KEY;
  content = gdbm_fetch (dbf, key);
  i = content.dptr == NULL;
  gdbm_close (dbf);
  chdir ("..");
  chdir (pu_cur.user);
  return i;
}

void
set_rd_newu_inf (struct INF_S *pinf, int n)
{
  char buf_key[LEN_INF_OUF_KEY];
  GDBM_FILE dbf;
  datum key, content;
  int i, ok = 0;

  open_db_writer (&dbf, N_DB_INF);
  for (i = 0; i < n; i++)
	if (pinf[i].chg_info) {
	  key.dptr = memset (buf_key, 0, LEN_INF_OUF_KEY);
	  strcpy (buf_key, pinf[i].file_inc);
	  key.dsize = LEN_INF_OUF_KEY;
	  content = gdbm_fetch (dbf, key);
	  if (content.dptr != NULL) {
		content.dptr[0] = pinf[i].flg_r;
		content.dptr[1] = pinf[i].flg_new;
		content.dptr[2] = pinf[i].flg_d;
		if(ok == 0) {
		    ok = 1;
		    sleep_kill_start ();
		}
		gdbm_store (dbf, key, content, GDBM_REPLACE);
	  }
	  pinf[i].chg_info = 0;
	}
  gdbm_close (dbf);
  if(ok)
    sleep_kill_stop ();
}

void
put_db_inf (struct INF_S *pinf)
{
  char *dc;
  char buf_key[LEN_INF_OUF_KEY];
  char ss[32];
  char *s;
  GDBM_FILE dbf;
  datum key, content;

  open_db_writer (&dbf, N_DB_INF);
  key.dptr = memset (buf_key, 0, LEN_INF_OUF_KEY);
  strcpy (buf_key, pinf->file_inc);
  key.dsize = LEN_INF_OUF_KEY;

  sprintf (ss, "%ld", pinf->sz);
  dc = alloca(3+strlen(pinf->user)+strlen(pinf->file)+
		strlen(pinf->rem)+strlen(pinf->d)+strlen(ss)+5);
  s = dc;
  *s++ = pinf->flg_r;
  *s++ = pinf->flg_new;
  *s++ = pinf->flg_d;
  strcpy (s, pinf->user);
  s += strlen (s) + 1;
  strcpy (s, pinf->file);
  s += strlen (s) + 1;
  strcpy (s, pinf->rem);
  s += strlen (s) + 1;
  strcpy (s, pinf->d);
  s += strlen (s) + 1;
  strcpy (s, ss);
  s += strlen (s) + 1;
  content.dptr = dc;
  content.dsize = (s - dc);
  sleep_kill_start ();
  gdbm_store (dbf, key, content, GDBM_INSERT);
  gdbm_close (dbf);
  sleep_kill_stop ();
}

void
unlink_db_in (struct INF_S *pinf, int n, int flg_ex_no_error)
{
  char buf_key[LEN_INF_OUF_KEY];
  GDBM_FILE dbf;
  int i, ok;
  datum key;

  open_db_writer (&dbf, N_DB_INF);
  ok = 0;
  sleep_kill_start ();
  for (i = 0; i < n; i++)
	if (pinf[i].flg_d == '1') {
	  key.dptr = memset (buf_key, 0, LEN_INF_OUF_KEY);
	  strcpy (buf_key, pinf[i].file_inc);
	  key.dsize = LEN_INF_OUF_KEY;
	  if (gdbm_exists (dbf, key) <= 0 || gdbm_delete (dbf, key) != 0) {
	    if(flg_ex_no_error==0)
		warn_d ("      \n");
	    pinf[i].flg_d = '0';
	  } else
		ok++;
	}
  if (ok)
	gdbm_reorganize (dbf);
  gdbm_close (dbf);
  sleep_kill_stop ();
}


/*
   
key=file_inc
:
size  %d
nuser %d -   ;
file  %s -   
rem   %s
  :
user\0din\0r
user  %s - -  file
din   %s -   ( )
r     %c -    ('0'|'1')
*/

int
get_db_ouf (struct OUTF_S **ouf_s)
{
  GDBM_FILE dbf;
  datum key, content;
  struct OUTF_S *pinf;
  int un, i;
  char *s;

  open_db_reader (&dbf, N_DB_OUT);
  key = gdbm_firstkey (dbf);
  for (i = 0;; i++) {
	if (key.dptr == NULL)
	  break;
	content = gdbm_fetch (dbf, key);
	if (content.dptr == NULL) {
	errr:
	  error_d (1, "    ");
	}
	content = gdbm_nextkey (dbf, key);
	key = content;
  }
  un = i;
  if (un) {
	pinf = (struct OUTF_S *) mallocc (i * sizeof (struct OUTF_S));
	*ouf_s = pinf;
	key = gdbm_firstkey (dbf);
	for (i = 0; i < un; i++) {
	  int u;
	  if (key.dptr == NULL)
		goto errr;
	  content = gdbm_fetch (dbf, key);
	  if (content.dptr == NULL)
		goto errr;
	  strcpy (pinf->file_inc, key.dptr);
	  decode_inc (pinf->file_inc, pinf->d);
	  s = content.dptr;
	  pinf->sz = strtol (s, &s, 10);
	  s++;
	  pinf->nuser = strtol (s, &s, 10);
	  s++;
	  if (pinf->nuser < 0) {
		pinf->nuser = 0;
		pinf->flg_noeof = 1;
	  }
	  pinf->file = strcpym(s);
	  s += strlen (s) + 1;
	  strcpy (pinf->rem, s);
	  s += strlen (s) + 1;
	  pinf->user = (char **) mallocc (pinf->nuser * sizeof (char *));
	  pinf->din = (char **) mallocc (pinf->nuser * sizeof (char *));
	  pinf->flg_r = mallocc (pinf->nuser);
	  for (u = 0; u < pinf->nuser; u++) {
		pinf->user[u] = strcpym (s);
		s += strlen (s) + 1;
		pinf->din[u] = strcpym (s);
		s += strlen (s) + 1;
		if (*s != '0' && *s != '1')
		  goto errr;
		pinf->flg_r[u] = *s++;
	  }
	  /* pinf->flg_d = 0; */
	  pinf++;
	  content = gdbm_nextkey (dbf, key);
	  key = content;
	}
  }
  gdbm_close (dbf);
  return un;
}

void
put_db_ouf (struct OUTF_S *pinf, int flg_replace)
{
  char buf_key[LEN_INF_OUF_KEY];
  char sz_nuser[64];
  char *dc;
  GDBM_FILE dbf;
  datum key, content;
  int l, i;

  open_db_writer (&dbf, N_DB_OUT);
  key.dptr = memset (buf_key, 0, LEN_INF_OUF_KEY);
  strcpy (buf_key, pinf->file_inc);
  key.dsize = LEN_INF_OUF_KEY;

  sprintf(sz_nuser, "%ld %d", pinf->sz, pinf->flg_noeof ? -1 : pinf->nuser);
				      /* 64 - max sizeof din */
  l = strlen(sz_nuser) + 1 + strlen(pinf->file) + strlen(pinf->rem) + 2
			+ pinf->nuser * (MAXPU_LEN + 2 + 64);
  content.dsize = l;
  dc = alloca (l);
  content.dptr = dc;
  strcpy(dc, sz_nuser);
  dc += strlen (dc) + 1;
  strcpy (dc, pinf->file);
  dc += strlen (dc) + 1;
  strcpy (dc, pinf->rem);
  dc += strlen (dc) + 1;
  for (i = 0; i < pinf->nuser; i++) {
	strcpy (dc, pinf->user[i]);
	dc += strlen (dc) + 1;
	strcpy (dc, pinf->din[i]);
	dc += strlen (dc) + 1;
	*dc++ = pinf->flg_r[i];
  }
  sleep_kill_start ();
  gdbm_store (dbf, key, content,
			  flg_replace == 0 ? GDBM_INSERT : GDBM_REPLACE);
  gdbm_close (dbf);
  sleep_kill_stop ();
}

void
unlink_db_out (struct OUTF_S *po, int n)
{
  char buf_key[LEN_INF_OUF_KEY];
  GDBM_FILE dbf;
  int i, ok;
  datum key;

  open_db_writer (&dbf, N_DB_OUT);
  ok = 0;
  sleep_kill_start ();
  for (i = 0; i < n; i++)
	if (po[i].flg_d) {
	  key.dptr = memset (buf_key, 0, LEN_INF_OUF_KEY);
	  strcpy (buf_key, po[i].file_inc);
	  key.dsize = LEN_INF_OUF_KEY;
	  if (gdbm_exists (dbf, key) <= 0 || gdbm_delete (dbf, key) != 0)
		warn_d ("      \n");
	  else
		ok++;
	}
  if (ok)
	gdbm_reorganize (dbf);
  gdbm_close (dbf);
  sleep_kill_stop ();
}

/*
   
key      = # 
name     = %LEN_NAME_KONFYs  
moder    = %MAXPU_LENs  
time_new = %11d   
nusers   = %4d   
  :
user+priv  (%5d)
*/


int
get_db_news (struct SYS_NEWS **news_s)
{
  GDBM_FILE dbf;
  datum key, content;
  struct SYS_NEWS *pinf;
  int un, i;
  char *s;

  open_db_reader (&dbf, N_DB_NEWS);
  key = gdbm_firstkey (dbf);
  for (i = 0;; i++) {
	if (key.dptr == NULL)
	  break;
	content = gdbm_fetch (dbf, key);
	if (content.dptr == NULL) {
	errr:
	  error_d (1, "    ");
	}
	content = gdbm_nextkey (dbf, key);
	key = content;
  }
  un = i;
  if (un) {
	pinf = (struct SYS_NEWS *) mallocc (i * sizeof (struct SYS_NEWS));
	*news_s = pinf;
	key = gdbm_firstkey (dbf);
	for (i = 0; i < un; i++) {
	  int u;
	  if (key.dptr == NULL)
		goto errr;
	  content = gdbm_fetch (dbf, key);
	  if (content.dptr == NULL)
		goto errr;
	  pinf->num = atoi (key.dptr);
	  s = content.dptr;
	  strcpy (pinf->name, s);
	  s += strlen (s) + 1;
	  strcpy (pinf->moder, s);
	  s += strlen (s) + 1;
	  pinf->time_new = atol (s);
	  s += 11;
	  pinf->nusers = atoi (s);
	  s += 4;
	  pinf->users = (char **) mallocc (pinf->nusers * sizeof (char *));
	  pinf->priv = (int *) mallocc (pinf->nusers * sizeof (int));
	  for (u = 0; u < pinf->nusers; u++) {
		pinf->users[u] = strcpym (s);
		s += strlen (s) + 1;
		pinf->priv[u] = atoi (s);
		s += 5;
	  }
	  pinf++;
	  content = gdbm_nextkey (dbf, key);
	  key = content;
	}
  }
  gdbm_close (dbf);
  return un;
}


void create_new_news_ex(int num, int flg_replace)
{
    GDBM_FILE dbf;

    sprintf (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY], "%d", num);
    if (flg_replace == 0) {
	if (mkdir (buf_news_empty, 0770) != 0 && errno != EEXIST) {
erra:
	    error_d (27, "      ");
	}
	flg_replace = strlen(buf_news_empty);
	sprintf (&buf_news_empty[flg_replace], "/%s", db_names[N_DB_NU]);
	dbf = gdbm_open (buf_news_empty, 512, GDBM_WRCREAT, 0660, 0);
	if (dbf == NULL)
	    goto erra;
	else
	    gdbm_close (dbf);
	buf_news_empty[flg_replace] = 0;        /* ../news/# only */
    }
}

void
put_db_news (struct SYS_NEWS *pinf, int flg_replace, char **b, int *sb)
{
  char *dc;
  int i;
  GDBM_FILE dbf;
  datum key, content;

  create_new_news_ex(pinf->num, flg_replace);
  open_db_writer (&dbf, N_DB_NEWS);
  key.dptr = &buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY];
  key.dsize = strlen (key.dptr) + 1;
  if(*sb == 0) {
    /*   */
    dc = mallocc (LEN_NAME_KONFY + MAXPU_LEN + 2 + 11 + 4 +
			pinf->nusers * (MAXPU_LEN + 1 + 5));
    *b = dc;
    content.dptr = strcpy (dc, pinf->name);
    dc += strlen (dc) + 1;
    strcpy (dc, pinf->moder);
    dc += strlen (dc) + 1;
    sprintf (dc, "%-11ld", pinf->time_new);
    dc += 11;
    sprintf (dc, "%-4d", pinf->nusers);
    dc += 4;
    for (i = 0; i < pinf->nusers; i++) {
	strcpy (dc, pinf->users[i]);
	dc += strlen (dc) + 1;
	sprintf (dc, "%-5d", pinf->priv[i]);
	dc += 5;
    }
    content.dsize = dc - content.dptr;
    *sb = content.dsize;
  } else {
    /*      */
    content.dptr  = *b;
    content.dsize = *sb;
  }
  sleep_kill_start ();
  gdbm_store (dbf, key, content,
			  flg_replace == 0 ? GDBM_INSERT : GDBM_REPLACE);
  gdbm_close (dbf);
  sleep_kill_stop ();
}

void
unlink_db_news (struct SYS_NEWS *po)
{
  GDBM_FILE dbf;
  datum key;
  int n, i;

  open_db_writer (&dbf, N_DB_NEWS);
  sprintf (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY], "%d", po->num);
  key.dptr = &buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY];
  key.dsize = strlen (key.dptr) + 1;
  sleep_kill_start ();
  if (gdbm_exists (dbf, key) <= 0)
	warn_d ("         %d\n",
			po->num);
  else if (gdbm_delete (dbf, key) != 0)
	warn_d ("       \n");
  else
	gdbm_reorganize (dbf);
  gdbm_close (dbf);
  sleep_kill_stop ();
  if (chdir (buf_news_empty)) {
	warn_d ("    \n");
	return;
  }
  n = my_scandir (".");
  chdir (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY]);
  for (i = 0; i < n; i++)
	unlink (name_dlist[i]);
  chdir ("..");
  if (rmdir (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY])) {
	warn_d ("    \n");
	return;
  }
  if (chdir (f_prompt ("../%s/%s", dir_users, pu_cur.user)) != 0)
	error_d (28, " ");
}


/*
   
key     = # 
time_lr = %11d   
sz      = %5d     tof
sz-:
tof     = %s      
}
*/
int
get_db_un (struct U_NEWS **news_s)
{
  GDBM_FILE dbf;
  datum key, content;
  struct U_NEWS *pinf;
  int un, i, u;
  char *s;

  open_db_reader (&dbf, N_DB_NU);
  key = gdbm_firstkey (dbf);
  for (i = 0;; i++) {
	if (key.dptr == NULL)
	  break;
	content = gdbm_fetch (dbf, key);
	if (content.dptr == NULL) {
	errr:
	  error_d (1, "   ");
	}
	content = gdbm_nextkey (dbf, key);
	key = content;
  }
  un = i;
  if (un) {
	pinf = (struct U_NEWS *) mallocc (i * sizeof (struct U_NEWS));
	*news_s = pinf;
	key = gdbm_firstkey (dbf);
	for (i = 0; i < un; i++) {
	  if (key.dptr == NULL)
		goto errr;
	  content = gdbm_fetch (dbf, key);
	  if (content.dptr == NULL)
		goto errr;
	  pinf->num = atoi (key.dptr);
	  s = content.dptr;
	  pinf->time_lr = strtol (s, &s, 10);
	  /* "while" for before 1.8.12 db format compatibile */
	  while(*s == ' ')
	    s++;
	  pinf->sz = strtol (s, &s, 10);
	  if(pinf->sz) {
	    /* "while" for before 1.8.12 db format compatibile */
	    while(*s == ' ')
		s++;
	  }
	  pinf->tof = (char **) mallocc (pinf->sz * sizeof (char *));
	  pinf->hash = (long *) mallocc (pinf->sz * sizeof (long));
	  for (u = 0; u < pinf->sz; u++) {
		pinf->tof[u] = strcpym (s);
		pinf->hash[u] = hash_decode_inc (s);
		s += strlen (s) + 1;
	  }
	  pinf++;
	  content = gdbm_nextkey (dbf, key);
	  key = content;
	}
  }
  gdbm_close (dbf);
  return un;
}

void
put_db_un (struct U_NEWS *pinf, int flg_replace)
{
  char *dc;
  char key_buf[8];
  int i, sz;
  GDBM_FILE dbf;
  datum key, content;

  open_db_writer (&dbf, N_DB_NU);
  sprintf (key_buf, "%d", pinf->num);
  key.dptr = key_buf;
  key.dsize = strlen (key.dptr) + 1;
  sz = 32 + (pinf->sz * (INC_KEY_SIZE + 1));
  content.dsize = sz;
  dc = alloca (sz);
  content.dptr = dc;
  pinf->time_lr = time (0);
  sprintf (dc, "%ld %d ", pinf->time_lr, pinf->sz);
  dc += strlen (dc);
  for (i = 0; i < pinf->sz; i++) {
	strcpy (dc, pinf->tof[i]);
	dc += strlen (dc) + 1;
  }
  sleep_kill_start ();
  gdbm_store (dbf, key, content,
			  flg_replace == 0 ? GDBM_INSERT : GDBM_REPLACE);
  if(flg_replace==2) {
    /*   */
    gdbm_reorganize (dbf);
  }
  gdbm_close (dbf);
  sleep_kill_stop ();
}

void
unlink_db_un (struct U_NEWS *po)
{
  char buf_key[8];
  GDBM_FILE dbf;
  datum key;

  open_db_writer (&dbf, N_DB_NU);
  sprintf (buf_key, "%d", po->num);
  key.dptr = buf_key;
  key.dsize = strlen (buf_key) + 1;
  sleep_kill_start ();
  if (gdbm_exists (dbf, key) <= 0 || gdbm_delete (dbf, key) != 0)
	warn_d ("      \n");
  else
	gdbm_reorganize (dbf);
  sleep_kill_stop ();
  gdbm_close (dbf);
}

/*
  
key - .  %d.%d (    )
                         %s
                         %11d
                     %s
 -              %s
              %4d
               %s
    %s
    %s
        %11d
..............................
*/

int
stat_nn (int ns, int flg_stat)
{
  sprintf (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY], "%d/%s", ns,
							db_names[N_DB_NU]);
  if (flg_stat)
	return stat_db_chg (N_DB_NN);
  return 0;
}

static GDBM_FILE
open_nn (int flg_w, int ns)
{
  GDBM_FILE dbf;
  stat_nn (ns, 0);          /*  patch    */
  if (flg_w)
	open_db_writer (&dbf, N_DB_NN);
  else
	open_db_reader (&dbf, N_DB_NN);
  return dbf;
}

static int
read_db_news_num (struct NUM_NEWS **news_s, GDBM_FILE dbf)
{
  datum key, content;
  struct NUM_NEWS *pinf;
  int un, i, u;
  char *s;

  key = gdbm_firstkey (dbf);
  for (i = 0;; i++) {
	if (key.dptr == NULL)
	  break;
	content = gdbm_fetch (dbf, key);
	if (content.dptr == NULL) {
errr:
	  error_d (1, "   ");
	}
	content = gdbm_nextkey (dbf, key);
	key = content;
  }
  un = i;
  if (un) {
	pinf = (struct NUM_NEWS *) mallocc (i * sizeof (struct NUM_NEWS));
	*news_s = pinf;
	key = gdbm_firstkey (dbf);
	for (i = 0; i < un; i++) {
	  if (key.dptr == NULL)
		goto errr;
	  content = gdbm_fetch (dbf, key);
	  if (content.dptr == NULL)
		goto errr;
	  pinf->tema = atoi (key.dptr);
	  s = key.dptr;
	  while (*s) {
		if (*s == '.')
		  break;
		s++;
	  }
	  if (*s == 0)
		goto errr;
	  pinf->otvet = atoi (s + 1);
	  s = content.dptr;
	  strcpy (pinf->author, s);
	  s += strlen (s) + 1;
	  pinf->ls = atol (s);
	  s += 11;
	  strcpy (pinf->rem, s);
	  s += strlen (s) + 1;
	  strcpy (pinf->bb, s);
	  s += strlen (s) + 1;
	  pinf->nf = atoi (s);
	  s += 4;
	  pinf->namef = (char **) mallocc (pinf->nf * sizeof (char *));
	  pinf->nsf = (char **) mallocc (pinf->nf * sizeof (char *));
	  pinf->nameu = (char **) mallocc (pinf->nf * sizeof (char *));
	  pinf->sizef = (long *) mallocc (pinf->nf * sizeof (long));
	  for (u = 0; u < pinf->nf; u++) {
		pinf->namef[u] = strcpym (s);
		s += strlen (s) + 1;
		pinf->nsf[u] = strcpym (s);
		s += strlen (s) + 1;
		pinf->nameu[u] = strcpym (s);
		s += strlen (s) + 1;
		pinf->sizef[u] = atol (s);
		s += 11;
	  }
	  pinf++;
	  content = gdbm_nextkey (dbf, key);
	  key = content;
	}
  }
  return un;
}

int
get_db_news_num (struct NUM_NEWS **news_s, int ns)
{
  struct NUM_NEWS *pinf;
  GDBM_FILE dbf;
  int u;

  dbf = open_nn (0, ns);
  u = read_db_news_num (&pinf, dbf);
  gdbm_close (dbf);
  if (u)
	*news_s = pinf;
  else
	*news_s = 0;
  return u;
}

static void
free_geted_nn (int n, struct NUM_NEWS *pinf)
{
  int i, o;
  if (n) {
	/*        */
	for (i = 0; i < n; i++) {
	  for (o = 0; o < pinf->nf; o++) {
		free (pinf->namef[o]);
		free (pinf->nsf[o]);
		free (pinf->nameu[o]);
	  }
	  free0 (&pinf->namef);
	  free0 (&pinf->nsf);
	  free0 (&pinf->nameu);
	  free0 (&pinf->sizef);
	  pinf++;
	}
	free (pinf - n);
  }
}


/*    */
static void
st_put_db_news_num_add (struct NUM_NEWS *news_s, int n, struct NUM_NEWS *pinf,
			    datum *pkey, GDBM_FILE *pdbf,
			    int ns, char **b, int *sb)
{
  char *dc;
  int i, o;
  datum content;
  struct NUM_NEWS *pnews_s;
  GDBM_FILE dbf;

  if(news_s==0) {
    /*       */
    dbf = open_nn (1, ns);
    n = read_db_news_num (&news_s, dbf);
  } else {
    dbf = *pdbf;
  }
  for (pnews_s = news_s; pnews_s < (news_s+n); pnews_s++)
	if (pnews_s->tema == pinf->tema && pnews_s->otvet == pinf->otvet)
	  break;
  if (pnews_s == (news_s+n) || strcmp (pnews_s->bb, pinf->bb) != 0) {
	warn_d ("   \n");
	goto ret;
  }

  o = 0;
  for (i = 0; i < pnews_s->nf; i++) {
	o += strlen(pnews_s->namef[i]) + 1;
  }
  if(*sb == 0) {
    /*   */
    for (i = 0; i < pinf->nf; i++)
	o += strlen(pinf->namef[i]) + 1;
  } else {
    /*      */
    char *s = *b;
    int l;

    for (i = 0; i < pinf->nf; i++) {
	l = strlen(s) + 1;      /* namef[i] */
	o += l;
	s += l;
	l = strlen (s) + 1;     /* nsf[i] */
	s += l;
	l = strlen (s) + 1;     /* nameu[i] */
	s += l;
	s += 11;                /* sizef[i] */
    }
    o++;
  }

  dc = mallocc (MAXPU_LEN + 1 + 48 + INC_KEY_SIZE + 1 + 11 + 4 + o +
		    (pnews_s->nf + pinf->nf) *
			(INC_KEY_SIZE + MAXPU_LEN + 3 + 11));
  content.dptr = strcpy (dc, pnews_s->author);
  dc += strlen (dc) + 1;
  sprintf (dc, "%-11ld", pnews_s->ls);
  dc += 11;
  strcpy (dc, pnews_s->rem);
  dc += strlen (dc) + 1;
  strcpy (dc, pnews_s->bb);
  dc += strlen (dc) + 1;
  sprintf (dc, "%-4d", pnews_s->nf + pinf->nf);
  dc += 4;
  for (o = 0; o < pnews_s->nf; o++) {
    strcpy (dc, pnews_s->namef[o]);
    dc += strlen (dc) + 1;
    strcpy (dc, pnews_s->nsf[o]);
    dc += strlen (dc) + 1;
    strcpy (dc, pnews_s->nameu[o]);
    dc += strlen (dc) + 1;
    sprintf (dc, "%-11ld", pnews_s->sizef[o]);
    dc += 11;
  }
  if(*sb == 0) {
    /*   */
    *b = content.dptr;
    for (i = 0; i < pinf->nf; i++) {
	strcpy (dc, pinf->namef[i]);
	dc += strlen (dc) + 1;
	strcpy (dc, pinf->nsf[i]);
	dc += strlen (dc) + 1;
	strcpy (dc, pinf->nameu[i]);
	dc += strlen (dc) + 1;
	sprintf (dc, "%-11ld", pinf->sizef[i]);
	dc += 11;
    }
  } else {
    /*      */
    char *s = *b;

    for (i = 0; i < pinf->nf; i++) {
	strcpy (dc, s);    /* namef[i] */
	o = strlen (dc) + 1;
	dc += o;
	s  += o;
	strcpy (dc, s);    /* nsf[i] */
	o = strlen (dc) + 1;
	dc += o;
	s += o;
	strcpy (dc, s);    /* nameu[i] */
	o = strlen (dc) + 1;
	dc += o;
	s += o;
	for(o=0; o<11; o++) /* sizef[i] */
	    *dc++ = *s++;
    }
  }
  content.dsize = (dc - content.dptr);
  sleep_kill_start ();
  if (gdbm_store (dbf, *pkey, content, GDBM_REPLACE)) {
	warn_d ("%d\n", gdbm_errno);
  }
  if(*sb == 0) {
    char *s = *b;

    /*           */
    for (i = 0; i < pinf->nf; i++) {
	strcpy (s, pinf->namef[i]);
	s += strlen (s) + 1;
	strcpy (s, pinf->nsf[i]);
	s += strlen (s) + 1;
	strcpy (s, pinf->nameu[i]);
	s += strlen (s) + 1;
	sprintf (s, "%-11ld", pinf->sizef[i]);
	s += 11;
    }
    *sb = s - *b;
  }
ret:
  gdbm_close (dbf);
  sleep_kill_stop ();
  free_geted_nn (n, news_s);
}

static void gen_key_bd_num(int tema, int otvet, datum *pkey)
{
    static char key_buf[16];                  /* %d.%d */

    sprintf (key_buf, "%d.%d", tema, otvet);
    pkey->dptr = key_buf;
    pkey->dsize = strlen (key_buf) + 1;
}

/*       */
void
put_db_news_num_add (struct NUM_NEWS *pinf, int ns, char **b, int *sb)
{
    datum key;

    gen_key_bd_num(pinf->tema, pinf->otvet, &key);
    st_put_db_news_num_add (0, 0, pinf, &key, 0, ns, b, sb);
}

/*      */
void
put_db_news_num (struct NUM_NEWS *pinf, int ns, int num_tema, char **b, int *sb)
{
  char *dc;
  int i, n, o;
  GDBM_FILE dbf;
  datum key, content;
  struct NUM_NEWS *news_s;

/*       */
  dbf = open_nn (1, ns);    /*       */
  n = read_db_news_num (&news_s, dbf);
  o = 0;
  if (num_tema == 0) {
	/*   */
	for (i = 0; i < n; i++)
	  if (news_s[i].tema > num_tema)
		num_tema = news_s[i].tema;
	num_tema++; /*   */
  } else {
	for (i = 0; i < n; i++)
	  if (news_s[i].tema == num_tema)
		if (news_s[i].otvet > o)
		  o = news_s[i].otvet;
	o++;        /*   */
  }
  free_geted_nn (n, news_s);
  news_s = mallocc(sizeof(struct NUM_NEWS));
  n=1;    /*         */
  news_s->tema  = pinf->tema = num_tema;
  news_s->otvet = pinf->otvet = o;
  strcpy(news_s->author, pinf->author);
  news_s->ls = pinf->ls;
  strcpy(news_s->rem, pinf->rem);
  strcpy(news_s->bb, pinf->bb);
  news_s->nf = 0; /*   ,    pinf->nf */

  gen_key_bd_num(num_tema, o, &key);

  dc = alloca (MAXPU_LEN + 11 + 48 + (INC_KEY_SIZE + 1) + 4);
  content.dptr = strcpy (dc, pinf->author);
  dc += strlen (dc) + 1;
  sprintf (dc, "%-11ld", pinf->ls);
  dc += 11;
  strcpy (dc, pinf->rem);
  dc += strlen (dc) + 1;
  strcpy (dc, pinf->bb);
  dc += strlen (dc) + 1;
  sprintf (dc, "%-4d", 0);
  dc += 4;
  content.dsize = dc - content.dptr;

  sleep_kill_start ();
  i = 0;
  if (gdbm_store (dbf, key, content, GDBM_INSERT)) {
	warn_d ("%d\n", gdbm_errno);
	i++;
  }
  sleep_kill_stop ();
  if(i==0) {
    /*       */
    st_put_db_news_num_add (news_s, n, pinf, &key, &dbf, ns, b, sb);
  } else {
    /*    gdbm_store (GDBM_INSERT) */
    gdbm_close (dbf);
  }
}

/*    */
void
put_db_news_num_del (struct NUM_NEWS *pinf, int ns, char *nsf)
{
  char *dc;
  int i, n, ok = 0, s;
  GDBM_FILE dbf;
  datum key, content;
  struct NUM_NEWS *news_s;
  struct NUM_NEWS *pnews_s;

	/*       */
  dbf = open_nn (1, ns);
  n = read_db_news_num (&news_s, dbf);
  for (i = 0; i < n; i++)
	if (news_s[i].tema == pinf->tema && news_s[i].otvet == pinf->otvet)
	  break;
  if (n != 0 && i != n && strcmp (news_s[i].bb, pinf->bb) == 0) {
	/*      */
	pnews_s = &news_s[i];
	gen_key_bd_num(pnews_s->tema, pnews_s->otvet, &key);
	for (i = 0; i < pnews_s->nf; i++) {
	  /*    */
	  if (strcmp (pnews_s->nsf[i], nsf) == 0)
		break;
	}
	if (i != pnews_s->nf) {
	    /*   */
	    ok = i;
	    s = 0;
	    for (i = 0; i < pnews_s->nf; i++) {
		if (i == ok)
		    continue;
		s += strlen(pnews_s->namef[i]) + 1;
	    }
	    dc = alloca (MAXPU_LEN + 11 + 48 + (INC_KEY_SIZE + 1) + 4 + s +
		    (pnews_s->nf-1) * (INC_KEY_SIZE + MAXPU_LEN + 2 + 11));
	    content.dptr = strcpy (dc, pnews_s->author);
	    dc += strlen (dc) + 1;
	    sprintf (dc, "%-11ld", pnews_s->ls);
	    dc += 11;
	    strcpy (dc, pnews_s->rem);
	    dc += strlen (dc) + 1;
	    strcpy (dc, pnews_s->bb);
	    dc += strlen (dc) + 1;
	    sprintf (dc, "%-4d", pnews_s->nf - 1);
	    dc += 4;
	    for (i = 0; i < pnews_s->nf; i++) {
		/*    */
		if (i == ok) {
		  sprintf (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY],
			      "%d/%s", ns, nsf);
		  continue;
		}
		strcpy (dc, pnews_s->namef[i]);
		dc += strlen (dc) + 1;
		strcpy (dc, pnews_s->nsf[i]);
		dc += strlen (dc) + 1;
		strcpy (dc, pnews_s->nameu[i]);
		dc += strlen (dc) + 1;
		sprintf (dc, "%-11ld", pnews_s->sizef[i]);
		dc += 11;
	    }
	    content.dsize = dc - content.dptr;
	    ok = content.dsize;
	    sleep_kill_start ();
	    if (gdbm_store (dbf, key, content, GDBM_REPLACE)) {
		warn_d ("%d\n", gdbm_errno);
	    } else
		unlink (buf_news_empty);
	}
  }
  gdbm_close (dbf);
  if (ok)
	sleep_kill_stop ();
  free_geted_nn (n, news_s);
}

void
unlink_db_news_num (struct NUM_NEWS *po, int ns)
{
  GDBM_FILE dbf;
  datum key;

  dbf = open_nn (1, ns);
  gen_key_bd_num(po->tema, po->otvet, &key);
  sleep_kill_start ();
  if (gdbm_exists (dbf, key) <= 0 || gdbm_delete (dbf, key) != 0)
    /* putcs("      \n"); */
	;
  else {
	gdbm_reorganize (dbf);
	if(po->ls) {
	    sprintf (&buf_news_empty[OFFSET_SLASH_BUF_NEWS_EMPTY],
			      "%d/%s", ns, po->bb);
	    unlink (buf_news_empty);
	}
  }
  sleep_kill_stop ();
  gdbm_close (dbf);
}
