#include "keyboard.h"
#include "fun.h"
#include "code.h"
#include "hlps.h"

static void
reput_promt_alarm (char *s1, char *s2, char *s3, int flg_set, int *cursor)
{
  static char *ss1;
  static char *ss2;
  static char *ss3;
  static int *cursor_sav;
  static int sflg_set;

  if (flg_set) {
	ss1 = s1;
	ss2 = s2;
	ss3 = s3;
	cursor_sav = cursor;
	sflg_set = flg_set;
  } else if (ss1 != 0) {
	putcs (ss1);
	if (ss2)
	  putcs (ss2);
	if (ss3) {
	  int l, i;

	  l = strlen (ss3);
	  if (sflg_set > 0)
		putcs (ss3);
	  else            /* passwd -> put *** */
		for (i = 0; i < l; i++)
		  putcc ('*');
	  if (cursor_sav)
		*cursor_sav = l;
	}
	set_alarm_out ();
	fflush (stdout);
	unset_alarm_out ();
  }
}

static int
getkey (void)
{
  int c, l;
  l = flg_inetd ? (pu_cur.l == 'E' ? 'E' : 'K') : pu_cur.l;
  set_alarm ();
  do {
	c = getch (pu_cur.af != 'a');
	if (c > ' ' && l != 'K') {
	  if (l == 'A')
		c = code_k_alt_k8[c];
	  else if (l == 'W')
		c = code_k_win_k8[c];
	}
  }
  while (!code_get_key[c]);
  unset_alarm ();
  return (code_get_key[c]);
}


int
wait_cr (void)
{
  int c, flg_minus;
  static char pwcr[] = "\n   Enter, '-`   => ";
  reput_promt_alarm (pwcr, 0, 0, 1, 0);
  detect_single_mode ();
  putcs (pwcr);
  set_alarm_out ();
  fflush (stdout);
  unset_alarm_out ();
  flg_minus = 0;
  while ((c = getkey ()) != CR)
	if (c == '-') {
	  flg_minus++;
	  break;
	}
  putcc ('\n');
  return flg_minus;
}

void
warn_timeout (int prompt)
{
  detect_single_mode ();
  warn_d ("\n\n !\n");
  if (prompt)
	reput_promt_alarm (0, 0, 0, 0, 0);
  else
	put_log ("Detect input timeout\n");
}


#define LEN_SEDTSTR 1024

static unsigned char sedtstr[LEN_SEDTSTR];

#include "cmdedit.c"
static char *
old_edtstr (char *prompt, int flg_no_echo)
{

  static char prompt0[] = " => ";
  int c, len;
  int maxl;

  detect_single_mode ();
  sedtstr[0] = 0;
  putcs (prompt);
  putcs (prompt0);
  maxl = 76 - strlen (prompt);
  len = 0;
  for (;;) {
	reput_promt_alarm (prompt, prompt0, sedtstr, 1 - flg_no_echo * 2, 0);
	set_alarm_out ();
	fflush (stdout);
	if (cuslog)
	  fflush (cuslog);
	unset_alarm_out ();
	c = getkey ();
	if (c == CR) {
	  putcc ('\n');
	  str_up (sedtstr);
	  while (--len >= 0 && sedtstr[len] == ' ')
		sedtstr[len] = 0;
	  return sedtstr;
	}
	if (c == BS && len > 0) {
	  len--;
	  sedtstr[len] = 0;
	  putcc ('\b');
	  putcc (' ');
	  putcc ('\b');
	  continue;
	}
	if ((c == ' ' || c == BS) && len == 0) {
	  putcc ('\a');
	  continue;
	}
	if (code_k[c] == 0)
	  continue;
	if (len < maxl) {
	  sedtstr[len] = c;
	  len++;
	  sedtstr[len] = 0;
	  if (flg_no_echo)
		putcc ('*');
	  else
		putcc (c);
	}
  }
}

char *
pstringnoecho (char *prompt)
{
  return old_edtstr (prompt, 1);
}

char *
login_editstring (char *prompt)
{
  return old_edtstr (prompt, 0);
}

char *
editstring (char *prompt)
{
  char *s;
  s = edtstr (prompt, 0);
  str_up (s);
  return s;
}

char *
editrem (char *prompt)
{
  return edtstr (prompt, 0);
}

char *
edit_existed (char *prompt)
{
  char *s;
  s = edtstr (prompt, 1);
  str_up (s);
  return s;
}

char *
fixeditstr (char *sets)
{
  return strcpy (sedtstr, sets);
}

static char aliases_file_name[] = "aliases";

struct ALIAS_CMD
{
  char *cmd;
  char *alias;
  int min_notequival_chars;
  struct ALIAS_CMD *next;
};

static struct ALIAS_CMD *s_alias_cmd;

/*         */

static struct ALIAS_CMD alias_cmd_no_carrier = {
  "POKA", "NO CARRIER", 10, 0
};

static struct ALIAS_CMD alias_cmd_ppp = {
  "POKA", "+++", 3, &alias_cmd_no_carrier
};

static void
free_alias_cmd (void)
{
  struct ALIAS_CMD *cur;

  for (cur = s_alias_cmd; cur;) {
	struct ALIAS_CMD *cur_for_free = cur;

	if(cur == &alias_cmd_no_carrier || cur == &alias_cmd_ppp) {
	    cur = cur->next;
	    continue;
	}
	free (cur->cmd);
	free (cur->alias);
	cur = cur->next;
	free (cur_for_free);            /*   cur */
  }
/*    */
  s_alias_cmd = &alias_cmd_ppp;
}

/*    */
void
scan_aliases_change (void)
{
  struct stat st_aliases;
  static time_t mtime_aliases;
  FILE *aliases;
  char buff[80];
  char *s, *sc, *sa;
  struct ALIAS_CMD *cur;

  if (stat (aliases_file_name, &st_aliases) != 0) {
    /*    -     */
	free_alias_cmd ();
	return;
  }
  if (mtime_aliases >= st_aliases.st_mtime)     /*  aliases   */
	return;
  mtime_aliases = st_aliases.st_mtime;  /*    */

  free_alias_cmd ();
  aliases = fopen (aliases_file_name, "r");
  if (aliases == NULL)
	return;
  while (fgets (buff, 80, aliases) != NULL) {
	int flg_bad_format = 0;

	s = buff;
	while (*s == ' ' || *s == '\t')
	  s++;                                          /* skip spaces */
	if (*s == 0 || *s == '\n' || *s == '#')         /* comment */
	  continue;
	sa = s;                                         /*  start alias   */
	sc = 0;                                         /* pointer to cmd */

	while (1) {
	  if (*s == ' ' || *s == '\t') {
		if (sc) {
		  flg_bad_format++;
		  break;
		}
		*s = 0;                                 /* end alias name */
		do
		  s++;
		while (*s == ' ' || *s == '\t');        /* skip spaces */
		sc = s;                                 /* start cmd */
		continue;   /* to check another spaces - bad format */
	  }
	  if (*s == '\n' || !*s) {      /* EOL */
		if (!sc || sc == s)     /* command name not found */
		  flg_bad_format++;
		*s = 0;
		break;                  /* break while(1) - found EOL */
	  } else {
		s++;
	  }
	}

	if (flg_bad_format)
	  continue;

	cur = mallocc (sizeof (struct ALIAS_CMD));
	cur->next = s_alias_cmd;
	s_alias_cmd = cur;
/*         */
	s = sa;
	while (((unsigned char) *s) >= 0340 || *s < 'a')
	  s++;
	cur->min_notequival_chars = (s - sa);
	str_up (sa);
	cur->alias = strcpym (sa);
	str_up (sc);
	cur->cmd = strcpym (sc);
  }
  fclose (aliases);
}

/*         */
int
scan_input (struct IN_DLG *pin)
{
  char **w;
  int *nml;
  unsigned char *s;
  unsigned char *s2;
  int nw, i;

  for (nw = 0; (*(pin->msgs))[nw] != NULL; nw++);
  w = (char **) alloca (nw * sizeof (char *));
  nml = (int *) alloca (nw * sizeof (int));
  for (nw = 0; (*(pin->msgs))[nw] != NULL; nw++) {
	s2 = s = (*(pin->msgs))[nw];
	while (*s >= 0340 || *s < 'a')
	  s++;
	i = (s - s2);
	nml[nw] = i;
	w[nw] = strcpym (s2);
	if (*s2 == '\n') {
	  w[nw][0] = 0;
	  w[nw][1] = 0;
	}
	str_up (w[nw]);
  }
  for (;;) {
	struct ALIAS_CMD *cur;

	s = editstring (pin->prompt);
	if (*s == '?' && (*(s + 1) == ' ' || *(s + 1) == 0)) {
		    /*    */
	  s++;
	  while (*s == ' ')
		s++;
	  if (*s == '*') {
		if (pin->zvezda)
		  putcs_mnogo (pin->zvezda);
		else
		  putcsl (pin);
		continue;
	  }
	  for (i = 0; i < nw; i++)
		if (!strncmp (s, w[i], nml[i]))
		  if (!strncmp (s, w[i], strlen (s)))
			break;
	  if (i == nw || *s == 0)
		putcsl (pin);
	  else
		putcs_mnogo ((*(pin->hlps))[i]);
	  continue;
	}
	if (*s == '?') {                        /*   */
	  putcs_mnogo (pin->alhlp);
	  continue;
	}

	/*     */
	for (cur = s_alias_cmd; cur; cur = cur->next)
	  if (!strncmp (s, cur->alias, cur->min_notequival_chars)) {
		s2 = s;
		while (*s2) {
		  if (*s2 == ' ')
			break;
		  s2++;
		}
		if (!strncmp (s, cur->alias, s2 - s))
		  break;
	  }

	/*   -      */
refind_from_alias:
	for (i = 0; i < nw; i++)
	  if (!strncmp (s, w[i], nml[i])) {
		s2 = s;
		while (*s2) {
		  if (*s2 == ' ')
			break;
		  s2++;
		}
		if (!strncmp (s, w[i], s2 - s))
		  break;
	  }
	if (i == nw) {
	  if (cur) {                            /*   */
		s = cur->cmd;
		cur = 0;
		goto refind_from_alias;
	  }
	  putcs ("  \n");
	  continue;
	}
	while (--nw >= 0)
	  free (w[nw]);
	return i + 1;
  }
}

int
yes_no (char *prompt)
{
  static char *dlg_yn[] = { "", "", "\n", NULL };
  static char *hlps[] = {
	" [ ] -  \n",
	"               -  \n",
	""
  };
  static char prompt_add[] = "? (\r2[]\r7, )";
  static const char alhlp[] = "\n\
   <> -  \n\
                  -  \n\n";

  struct IN_DLG in_dlg;

  char *all_prmt;
  unsigned char sav_sedtstr[LEN_SEDTSTR];
  int ret;

  all_prmt = mallocc (strlen (prompt) + strlen (prompt_add) + 1);
  strcpy (all_prmt, prompt);
  strcat (all_prmt, prompt_add);
  strcpy (sav_sedtstr, sedtstr);

  in_dlg.msgs = &dlg_yn;
  in_dlg.prompt = all_prmt;
  in_dlg.alhlp = alhlp;
  in_dlg.hlps = &hlps;
  in_dlg.zvezda = 0;

  switch (scan_input (&in_dlg)) {
  case 1:
  case 3:
	ret = 1;
	break;
  default:
	ret = 0;
	break;
  }
  strcpy (sedtstr, sav_sedtstr);
  free (all_prmt);
  return ret;
}

int
no_yes (char *prompt)
{
  static char *dlg_yn[] = { "", "", "\n", NULL };
  static char *hlps[] = {
	" [ ] -  \n",
	"                 -  \n",
	""
  };
  static char prompt_add[] = "? (\r2[]\r7, )";
  static const char alhlp[] = "\n\
   <> -  \n\
                    -  \n\n";

  struct IN_DLG in_dlg;

  char *all_prmt;
  unsigned char sav_sedtstr[LEN_SEDTSTR];
  int ret;

  all_prmt = mallocc (strlen (prompt) + strlen (prompt_add) + 1);
  strcpy (all_prmt, prompt);
  strcat (all_prmt, prompt_add);
  strcpy (sav_sedtstr, sedtstr);

  in_dlg.msgs = &dlg_yn;
  in_dlg.prompt = all_prmt;
  in_dlg.alhlp = alhlp;
  in_dlg.hlps = &hlps;
  in_dlg.zvezda = 0;

  switch (scan_input (&in_dlg)) {
  case 1:
  case 3:
	ret = 0;
	break;
  default:
	ret = 1;
	break;
  }
  strcpy (sedtstr, sav_sedtstr);
  free (all_prmt);
  return ret;
}

/*    */
unsigned char *
two_e_word (void)
{
  unsigned char *s;
  unsigned char *s2;
  s = sedtstr;
  while (*s != 0) {                             /*   */
	if (*s == ' ')
	  break;
	s++;
  }
  if (*s == 0)                                  /*     */
	return 0;
  s2 = s;
  while (*s == ' ')
	s++;
  if (*s == 0) {                                /*     */
	*s2 = 0;
	return 0;
  }
  return s;
}

/*        */
int
scan_par (unsigned char *two_word, int err_flg)
{
  unsigned char *s;
  int nml;
  char *w;

  w = strcpym (two_word);
  s = two_word;
  while (*s >= 0340 || *s <= 'A')
	s++;
  nml = (s - two_word);
  str_up (w);
  s = two_e_word ();
rescan:
  if (s != NULL && strncmp (s, w, nml) == 0
	  && strncmp (s, w, strlen (s)) == 0) {
	free (w);
	return 1;
  }
  if (err_flg) {
	if (s == 0) {
	  s = editstring (" ");
	  goto rescan;
	}
	putcs ("  \n");
  }
  free (w);
  return 0;
}

int
g_int (char **s1)
{
  int to;
  char *s;

  s = *s1;
  to = atoi (s);
  while (*s >= '0' && *s <= '9')
	s++;
  *s1 = s;
  return to;
}

static int *flg_from_num;
void
set_flg_from_num (int num, int flg)
{

  if (flg == FLG_START) {
	free0 (&flg_from_num);
	if (num) {
	  flg_from_num = mallocc ((num + 1) * sizeof (int));
	  flg_from_num[0] = num;
	}
  } else
	flg_from_num[num] = flg;
}

/*     

   
   
   -
        

:
   :(||||||)

         

:
   1)                                    - 
   2) :                              -  
   3) 1-3::, 4-6:    -   1,2  3 
  ,  ,  4,5  6  -  
    
   4) 1-3:: 4-6: -  3- 
   */

#define NUM      1
#define NUM_NUM  2

#define Error    3
#define Error_Mo 4

#define Uda      256
#define Sob      257
#define Raz      258
#define Now      259
#define Sta      260
#define Pri      261
#define Nep      262


static char *str_mods[] = {
  "",
  "",
  "",
  "",
  "",
  "",
  "",
};

static int
lex_f_par (int *num1, int *num2)
{
  static char sav_par[LEN_SEDTSTR + 32];
  char *s;
  int i;

  if (num1 == 0) {                  /*      */
	s = two_e_word ();
	if (s == NULL)              /*     */
	  return 0;
	else
	  strcpy (sav_par, s);
	return 1;
  }
  s = sav_par;
rescan:
  if (*s == 0)                      /*   */
	return 0;
  if (*s == ' ' || *s == ',') {     /*   */
	strcpy (s, s + 1);
	goto rescan;
  }
  if (*s >= '0' && *s <= '9') {
	i = g_int (&s);             /*   */
	if (i == 0)                 /*      */
	  return Error;
	*num1 = i;
	while (*s == ' ')
	  s++;
	if (*s != '-') {
	  strcpy (sav_par, s);
	  return NUM;
	}
	s++;
	while (*s == ' ')
	  s++;
	i = g_int (&s);             /*   */
	if (i <= 0)
	  return Error;
	*num2 = i;
	strcpy (sav_par, s);
	return NUM_NUM;
  }
  if (*s == ':') {
	char *s2;
	s++;
	s2 = s;                     /*   */
	while (*s != 0 && *s != ',' && *s != ' ' && *s != ':')
	  s++;
	i = *s;
	*s = 0;
	fixeditstr (f_prompt (": %s", s2));     /*   */
	*s = i;
	strcpy (sav_par, s);        /*     */
	if (scan_par (str_mods[0], 0))
	  return Uda;
	else if (scan_par (str_mods[1], 0))
	  return Sob;
	else if (scan_par (str_mods[2], 0))
	  return Raz;
	else if (scan_par (str_mods[3], 0))
	  return Now;
	else if (scan_par (str_mods[4], 0))
	  return Sta;
	else if (scan_par (str_mods[5], 0))
	  return Pri;
	else if (scan_par (str_mods[6], 0))
	  return Nep;
	else
	  return Error_Mo;
  }
  return Error;
}

int *i_synt_l;

static int
remove_noremoved (int mod_start, int n)
{
  int i, num1;

  if (flg_from_num) {                   /*   */
	for (i = mod_start; i < n; i++)
	  if ((flg_from_num[i_synt_l[i]] & FLG_DELET) != 0)
		i_synt_l[i] = 0;
	num1 = n;
	n = 0;
	for (i = 0; i < num1; i++)      /*   */
	  if (i_synt_l[i])
		i_synt_l[n++] = i_synt_l[i];
  }
  return n;
}

int
chk_f_list (int max)
{
  int num1, num2;
  int i, n, ok;
  int mod_start;                    /*  ,   */
  int retl, pre_retl;
  int remove_flg;

  free0 (&i_synt_l);
/*   :     */
  i_synt_l = (int *) mallocc ((max * 2 + 2) * sizeof (int));
  if (lex_f_par (0, 0) == 0) {      /*     */
	for (i = 0; i < max; i++)
	  i_synt_l[i] = i + 1;
	i = remove_noremoved (0, i);
	i_synt_l[i] = 0;
	return i;
  }
  mod_start = 0;
  n = 0;
  pre_retl = 0;
  remove_flg = 0;                   /*   */
  for (;;) {
	num2 = ok = 0;              /*        */
	retl = lex_f_par (&num1, &num2);
	if (pre_retl >= 256 && retl < 256) {    /*    */
	  if (remove_flg == 0)      /*    */
		n = remove_noremoved (mod_start, n);
	  else
		remove_flg = 0;
	  mod_start = n;            /*    */
	}
	if (pre_retl >= 256 && flg_from_num == 0) {
	  warn_d(
" ':'     ,  \n");
err:
	  warn_d ("\r2    \r7\n");
err2:
	  return 0;
	}
	switch (retl) {
	case Error_Mo:
	  warn_d (" \n");
	  goto err2;
	case Error:
	  goto err;
	case 0:
	  if (remove_flg == 0)          /*    */
		n = remove_noremoved (mod_start, n);
	  i_synt_l[n] = 0;
	  return n;
	case NUM:
	  num2 = num1;
	  /* break; ! */
	case NUM_NUM:
	  if (num2 < num1) {            /*   */
		i = num2;
		num2 = num1;
		num1 = i;
	  }
	  if (num1 > max)
		goto err;
	  while (num1 <= num2) {
		if (num1 > max)
		  break;
		/*       */
		for (i = mod_start; i < n; i++)
		  if (i_synt_l[i] == num1)
			break;
		if (i == n)
		  i_synt_l[n++] = num1;
		num1++;
	  }
	  num2 = 0;
	  break;
	case Uda:
	  remove_flg++;
	  if (n == 0) {                         /*    */
		num2++;
		for (i = 0; i < max;)
		  if ((flg_from_num[++i] & FLG_DELET) != 0) {
			i_synt_l[n++] = i;
			ok++;
		  }
	  } else {                /*    */
		ok++;             /*      -   */
		for (i = mod_start; i < n; i++)
		  if ((flg_from_num[i_synt_l[i]] & FLG_DELET) == 0)
			i_synt_l[i] = 0;
	  }
	  break;
	case Sob:
	  if (n == 0) {                         /*    */
		num2++;
		for (i = 0; i < max;)
		  if ((flg_from_num[++i] & FLG_MY) != 0) {
			i_synt_l[n++] = i;
			ok++;
		  }
	  } else {                              /*    */
		ok++;
		for (i = mod_start; i < n; i++)
		  if ((flg_from_num[i_synt_l[i]] & FLG_MY) == 0)
			i_synt_l[i] = 0;
	  }
	  break;
	case Raz:
	  if (n == 0) {                         /*    */
		num2++;
		for (i = 0; i < max;)
		  if ((flg_from_num[++i] & FLG_MY) == 0) {
			i_synt_l[n++] = i;
			ok++;
		  }
	  } else {                              /*    */
		ok++;
		for (i = mod_start; i < n; i++)
		  if ((flg_from_num[i_synt_l[i]] & FLG_MY) != 0)
			i_synt_l[i] = 0;
	  }
	  break;
	case Now:
	  if (n == 0) {                         /*    */
		num2++;
		for (i = 0; i < max;)
		  if ((flg_from_num[++i] & FLG_NEW) != 0) {
			i_synt_l[n++] = i;
			ok++;
		  }
	  } else {                              /*    */
		ok++;
		for (i = mod_start; i < n; i++)
		  if ((flg_from_num[i_synt_l[i]] & FLG_NEW) == 0)
			i_synt_l[i] = 0;
	  }
	  break;
	case Sta:
	  if (n == 0) {                         /*    */
		num2++;
		for (i = 0; i < max;)
		  if ((flg_from_num[++i] & FLG_NEW) == 0) {
			i_synt_l[n++] = i;
			ok++;
		  }
	  } else {                              /*    */
		ok++;
		for (i = mod_start; i < n; i++)
		  if ((flg_from_num[i_synt_l[i]] & FLG_NEW) != 0)
			i_synt_l[i] = 0;
	  }
	  break;
	case Pri:
	  if (n == 0) {                         /*    */
		num2++;
		for (i = 0; i < max;)
		  if ((flg_from_num[++i] & FLG_READ) != 0) {
			i_synt_l[n++] = i;
			ok++;
		  }
	  } else {                              /*    */
		ok++;
		for (i = mod_start; i < n; i++)
		  if ((flg_from_num[i_synt_l[i]] & FLG_READ) == 0)
			i_synt_l[i] = 0;
	  }
	  break;
	case Nep:
	  if (n == 0) {                         /*    */
		num2++;
		for (i = 0; i < max;)
		  if ((flg_from_num[++i] & FLG_READ) == 0) {
			i_synt_l[n++] = i;
			ok++;
		  }
	  } else {                              /*    */
		ok++;
		for (i = mod_start; i < n; i++)
		  if ((flg_from_num[i_synt_l[i]] & FLG_READ) != 0)
			i_synt_l[i] = 0;
	  }
	  break;
	}
	if (ok) {
	  for (i = 0; i < (n - 1); i++)           /*   */
		for (num1 = i + 1; num1 < n; num1++)
		  if (i_synt_l[i] == i_synt_l[num1])
			i_synt_l[num1] = 0;
	  num1 = n;
	  n = 0;
	  for (i = 0; i < num1; i++)              /*   */
		if (i_synt_l[i])
		  i_synt_l[n++] = i_synt_l[i];
	} else if (num2 != 0) {
	  warn_d ("%s \n", str_mods[retl - Uda]);
	  goto err2;
	}
	pre_retl = retl;
  }
}

/*

     

n                  -     `n'   !      
n.m                -    `m'       !     
		      `n'                    !
n1-n2.m            -  `m'    `n1'  `n2'  !
n.m1-m2            -   `m1'  `m2'  `n' !
n1-n2.m1-m2        -   `m1'  `m2'      !
		     `n1'  `n2'                   !
*                  -                         !
*.m                -  `m'               !
n.*                -    `n'            !
*.*                -                !
*.m1-m2            -   `m1'  `m2'   !
n1-n2.*            -     `n1'  `n2'  !
----------------------------------------------------!---------------------
:       -            !       
:        -             !   
--------------------------------------------------------------------------
+                  -     !
		                         !
+n                 -   `n'   !         
		                         ! `' 
+n1-n2             -   `n1'  `n2'  - !
		             !
--------------------------------------------------------------------------
*/


static int
parser1_konf (char **s1, struct ALL_KONF *to, int n)
{
  int tp, tp2, op, op2;
  int i, j;
  char *s, *s2;
  int max_t, max_o;

  max_t = max_o = 0;
  for (i = 0; i < n; i++) {
	if (to[i].num_news->tema > max_t)
	  max_t = to[i].num_news->tema;
	if (to[i].num_news->otvet > max_o)
	  max_o = to[i].num_news->otvet;
  }

  s = *s1;
  if (*s == 0) {                        /*     */
	for (i = j = 0; i < n; i++)     /*    */
	  if (to[i].nf == 0)
		to[j++] = to[i];
	return j;
  }
  s = *s1;
  switch (*s) {
  case '+':                             /*   */
	for (i = j = 0; i < n; i++)     /*  */
	  if (to[i].nf != 0)
		to[j++] = to[i];
	return j;
  case '0':
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '9':
	tp = g_int (&s);                /*   */
	*s1 = s;
	tp2 = tp;
	break;
  case '*':                             /*   */
	s++;
	*s1 = s;
	tp = 1;
	tp2 = max_t;
	break;
  case ':':
	s++;
	s2 = s;
	while (*s != 0 && *s != '+' && *s != ':')
	  s++;
	op = *s;
	*s = 0;
	fixeditstr (f_prompt (": %s", s2));
	if (scan_par ("", 0))
	  op2 = 1;
	else if (scan_par ("", 0))
	  op2 = 0;
	else {
	  putcs ("    ':'\n");
	  return -1;
	}
	for (i = j = 0; i < n; i++)
	  if ((to[i].flg_r != 0) == op2)
		to[j++] = to[i];
	*s = op;
	*s1 = s;
	return j;
  default:
	goto err;
  }
  *s1 = s;
  switch (*s) {
  case 0:                               /*      */
	for (i = j = 0; i < n; i++)
	  if (to[i].nf == 0 && to[i].num_news->tema >= tp &&
		  to[i].num_news->tema <= tp2 && to[i].num_news->otvet == 0)
		to[j++] = to[i];
	return j;
  case '+':                         /*        */
	for (i = j = 0; i < n; i++)
	  if (to[i].nf != 0 && to[i].num_news->tema >= tp &&
		  to[i].num_news->tema <= tp2 && to[i].num_news->otvet == 0)
		to[j++] = to[i];
	return j;
  case '.':
	break;
  case '-':                             /*   */
	s++;
	if (*s >= '0' && *s <= '9')
	  tp2 = g_int (&s);
	else
	  goto err;
	break;
  default:
	goto err;
  }
  *s1 = s;
  switch (*s) {
  case 0:                               /*   */
	for (i = j = 0; i < n; i++)
	  if (to[i].nf == 0 && to[i].num_news->tema >= tp &&
		  to[i].num_news->tema <= tp2 && to[i].num_news->otvet == 0)
		to[j++] = to[i];
	return j;
  case '+':                             /*     */
	for (i = j = 0; i < n; i++)
	  if (to[i].nf != 0 && to[i].num_news->tema >= tp &&
		  to[i].num_news->tema <= tp2 && to[i].num_news->otvet == 0)
		to[j++] = to[i];
	*s1 = s;
	return j;
  case '.':
	s++;
	break;
  default:
	goto err;
  }
  *s1 = s;
  switch (*s) {
  case '0':
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '9':
	op = g_int (&s);                        /*   */
	op2 = op;
	break;
  case '*':                                     /*   */
	s++;
	op = 1;
	op2 = max_o;
	break;
  default:
	goto err;
  }
  *s1 = s;
  switch (*s) {
  case 0:                               /*     */
	for (i = j = 0; i < n; i++)
	  if (to[i].nf == 0 && to[i].num_news->tema >= tp &&
		  to[i].num_news->tema <= tp2 &&
		  to[i].num_news->otvet >= op && to[i].num_news->otvet <= op2)
		to[j++] = to[i];
	return j;
  case '+':                       /*       */
	for (i = j = 0; i < n; i++)
	  if (to[i].nf != 0 && to[i].num_news->tema >= tp &&
		  to[i].num_news->tema <= tp2 &&
		  to[i].num_news->otvet >= op && to[i].num_news->otvet <= op2)
		to[j++] = to[i];
	return j;
  case '-':                             /*   */
	s++;
	break;
  default:
	goto err;
  }
  if (*s >= '0' && *s <= '9')
	op2 = g_int (&s);
  else
	goto err;
  *s1 = s;
  for (i = j = 0; i < n; i++)
	if ((to[i].nf != 0) == (*s == '+') &&
		to[i].num_news->tema >= tp && to[i].num_news->tema <= tp2 &&
		to[i].num_news->otvet >= op && to[i].num_news->otvet <= op2)
	  to[j++] = to[i];
  return j;
err:
  putcs ("\r2    \r7\n");
  return -1;
}

static int
parser2_konf (char **s1, struct ALL_KONF *to, int n)
{
  int f, f2, i, j;
  char *s;

  f2 = 0;
  for (i = 0; i < n; i++)
	if (to[i].nf > f2)
	  f2 = to[i].nf;
  s = *s1;
  s++;
  switch (*s) {
  case 0:                               /*      */
  case '+':
  case ':':
	f = 1;
	break;
  case '0':
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '9':
	f = g_int (&s);
	if (f == 0)
	  goto err;
	f2 = f;
	break;
  default:
	goto err;
  }
  if (*s == '-') {
	s++;
	if (*s >= '0' && *s <= '9') {
	  f2 = g_int (&s);
	  if (f2 < f)
		goto err;
	} else
	  goto err;
  }
  *s1 = s;
  for (i = j = 0; i < n; i++)
	if (to[i].nf >= f && to[i].nf <= f2)
	  to[j++] = to[i];
  return j;
err:
  putcs ("\r2     \r7\n");
  return -1;
}


static int
not_fond_konf (int nr)
{
  if (nr == 0)
	putcs ("\r2  /   \r7\n");
  return nr <= 0 ? 0 : nr;
}

int
parser_konf (struct NUM_NEWS *to, int n, struct U_NEWS *uns,
			 struct ALL_KONF **pall_conf)
{
  int i, j, nu, sza;
  struct ALL_KONF *all_conf;
  char *s;
  char s0;
  long hash;

  sza = 0;
  for (i = 0; i < n; i++)
	sza += 1 + to[i].nf;
  all_conf = mallocc (sza * sizeof (struct ALL_KONF));
  *pall_conf = all_conf;

  for (i = j = 0; i < n; i++, j++) {
	all_conf[j].num_news = &to[i];
	all_conf[j].nameu = to[i].author;
	all_conf[j].nf = 0;
	all_conf[j].namef = 0;
	all_conf[j].nsf = to[i].bb;
	all_conf[j].flg_r = 0;
	all_conf[j].u_news = -1;
	all_conf[j].sizef = to[i].ls;
	hash = hash_decode_inc (all_conf[j].nsf);
	for (nu = 0; nu < uns->sz; nu++)
	  if (uns->hash[nu] == hash &&
		  strcmp (all_conf[j].nsf, uns->tof[nu]) == 0) {
		all_conf[j].u_news = nu;
		all_conf[j].flg_r = 1;
		break;
	  }
	for (sza = 0; sza < to[i].nf; sza++) {
	  j++;
	  all_conf[j].nameu = to[i].nameu[sza];
	  all_conf[j].num_news = &to[i];
	  all_conf[j].nf = sza + 1;
	  all_conf[j].namef = to[i].namef[sza];
	  all_conf[j].nsf = to[i].nsf[sza];
	  all_conf[j].u_news = -1;
	  all_conf[j].flg_r = 0;
	  all_conf[j].sizef = to[i].sizef[sza];
	  hash = hash_decode_inc (all_conf[j].nsf);
	  for (nu = 0; nu < uns->sz; nu++)
		if (uns->hash[nu] == hash &&
			strcmp (all_conf[j].nsf, uns->tof[nu]) == 0) {
		  all_conf[j].u_news = nu;
		  all_conf[j].flg_r = 1;
		  break;
		}
	}
  }
  s0 = 0;
  s = two_e_word ();
  if (s == NULL)
	s = &s0;
  else {
	char *ss;
	ss = s;
	while (*s) {
	  if (*s == ' ')
		strcpy (s, s + 1);
	  else
		s++;
	}
	s = ss;
  }
  for (;;) {
	if (*s != '+')
	  i = parser1_konf (&s, all_conf, j);
	else
	  i = parser2_konf (&s, all_conf, j);
	if (*s == 0 || i <= 0)
	  return not_fond_konf (i);
	j = i;
  }
}
