#define _GNU_SOURCE
#include "fun.h"
#include "keyboard.h"
#include <stdarg.h>
#include "code.h"
#include "db.h"


void
str_up (unsigned char *s)
{
  while (*s) {
	if ((*s >= 'a' && *s <= 'z') || (*s >= 224 /* && *s <= 255 */))
	  *s ^= 040;
	*s &= 0177;
	s++;
  }
}

static void
err0alloc (void *ptr)
{
  if (ptr == 0)
	error_d (22, " ");
}

void *
mallocc (size_t size)
{
  void *ptr;
  if (size == 0)
	return 0;
  ptr = malloc (size);
  err0alloc (ptr);
  return memset (ptr, 0, size);
}

void *
reallocc (void *ptr, size_t size)
{
  if (ptr) {
	ptr = realloc (ptr, size);
	err0alloc (ptr);
  } else
	ptr = mallocc (size);
  return ptr;
}

void
free0 (void *p)
{
  if (*(void **) p != (void *) 0) {
	free (*(void **) p);
	*(void **) p = (void *) 0;
  }
}

char *
strcpym (const char *s)
{
  char *ptr;
  ptr = mallocc (strlen (s) + 1);
  strcpy (ptr, s);
  return ptr;
}

char *
bb_asprintf(const char *fmt, ...)
{
  char *p;
  va_list ap;

  va_start (ap, fmt);
  p = NULL;
  (void) vasprintf (&p, fmt, ap);
  va_end (ap);
  err0alloc (p);
  return p;
}

char **name_dlist;

void
put_names (char *name, int num)
{
  static int nn;

  if (name == 0) {
	int i;

	for (i = 0; i < nn; i++)
	  free (name_dlist[i]);
	free0 (&name_dlist);
	if (num > 0)
	  name_dlist = (char **) mallocc (num * sizeof (char *));
	nn = 0;
  } else
	name_dlist[nn++] = strcpym (name);
}

int
my_scandir (char *dir)
{
  struct dirent **namelist;
  int n, nc, i;
  struct stat statb;
  int fd_dir;

  n = scandir (dir, &namelist, 0, alphasort);
  if (n <= 0)
	return n;
  nc = 0;
  fd_dir = open (".", O_RDONLY, 0);
  if (fd_dir < 0)
	return -1;
  chdir (dir);
  for (i = 0; i < n; i++) {
	if (stat (namelist[i]->d_name, &statb) < 0)
	  continue;
	if (S_ISREG (statb.st_mode))
	  nc++;
  }
  if (nc <= 0)
    goto ret;
  put_names (0, nc);
  nc = 0;
  for (i = 0; i < n; i++) {
	if (stat (namelist[i]->d_name, &statb) < 0)
	  continue;
	if (S_ISREG (statb.st_mode)) {
	  put_names (namelist[i]->d_name, 0);
	  nc++;
	}
  }
ret:
  fchdir (fd_dir);
  close(fd_dir);
  if (namelist == NULL) {
	nc = -1;
  } else {
	for (i = 0; i < n; i++)
	    free(namelist[i]);
	free(namelist);
  }
  return nc;
}

static char *
translit_out(int c)
{
	static char out[4];
#define U_C(c) ((int)(c) & 0xff)
	switch(c) {
		case U_C(''):
			out[0] = ('v');
			out[1] = 0;
			break;
		case U_C(''):
			out[0] = ('V');
			out[1] = 0;
			break;
		case U_C(''):
			out[0] = ('z');
			out[1] = ('h');
			out[2] = 0;
			break;
		case U_C(''):
			out[0] = ('Z');
			out[1] = ('H');
			out[2] = 0;
			break;
		case U_C(''):
		case U_C(''):
			out[0] = ('4');
			out[1] = 0;
			break;
		case U_C(''):
			out[0] = ('s');
			out[1] = ('h');
			out[2] = 0;
			break;
		case U_C(''):
			out[0] = ('S');
			out[1] = ('H');
			out[2] = 0;
			break;
		case U_C(''):
			out[0] = ('s');
			out[1] = ('c');
			out[2] = ('h');
			break;
		case U_C(''):
			out[0] = ('S');
			out[1] = ('C');
			out[2] = ('H');
			break;
		case U_C(''):
		case U_C(''):
		case U_C(''):
		case U_C(''):
			out[0] = ('`');
			out[1] = 0;
			break;
		case U_C(''):
			out[0] = ('e');
			out[1] = 0;
			break;
		case U_C(''):
			out[0] = ('E');
			out[1] = 0;
			break;
		case U_C(''):
			out[0] = ('y');
			out[1] = ('u');
			out[2] = 0;
			break;
		case U_C(''):
			out[0] = ('Y');
			out[1] = ('U');
			out[2] = 0;
			break;
		case U_C(''):
			out[0] = ('y');
			out[1] = ('a');
			out[2] = 0;
			break;
		case U_C(''):
			out[0] = ('Y');
			out[1] = ('A');
			out[2] = 0;
			break;
		default:
			c &= 0x7f;
			c ^= 040;
			out[0] = (c);
			out[1] = 0;
			break;
	}
	return out;
#undef U_C
}

static char *
encodeput (int c)
{
  static char out[2];

  c &= 0xff;
  switch (pu_cur.l) {
  case 'E':
	if (c > 0200)
	    return translit_out(c);
	out[0] = c;
	break;
  case 'A':
	out[0] = code_k8_2alt[c];
	break;
  case 'W':
	out[0] = code_k8_2win[c];
	break;
  default:
	out[0] = c;
	break;
  }
  return out;
}

long int bytes_out;

void
putcc (int c)
{
  static int no_ansi_log;
  char s2[2];

  if (disable_out == 0) {
	int sav_l = pu_cur.l;

	if (c == '\n')
	  set_alarm_out ();
	if (cuslog) {
	  if (no_ansi_log) {
		if ((c >= '@' && c <= 'Z') || (c >= '`' && c <= 'z'))
		  no_ansi_log--;
	  } else {
		if (c == 033)
		  no_ansi_log = 1;
		else
		  putc (c, cuslog);
	  }
	}
	if (flg_inetd != 0 && pu_cur.l != 'E')
	  pu_cur.l = 'K';
	s2[0] = c;
	s2[1] = 0;
	putc_encode_buff(s2, stdout);
	if (flg_inetd != 0)
	  pu_cur.l = sav_l;
	if (c == '\n')
	  unset_alarm_out ();
	bytes_out++;
  }
}

void
putcs (const unsigned char *s)
{
  while (*s) {
	if (*s == '\r') {                       /*   */
	  s++;
	  if (*s >= '0' && *s <= '7' && pu_cur.flg_nocolor == 0) {
		putcc (033);
		putcc ('[');
		putcc ('3');
		putcc (*s);
		putcc ('m');
	  }
	} else
	  putcc (*s);
	s++;
  }
}

/* Exchange glibc vasprintf - minimize allocate memory version */
static void d_vasprintf(char **string_ptr, const char *format, va_list args)
{
	static char p[1024];
	static char *rets;
	static int   bs;
	char *buff;
	int   done;

	if(bs==0) {
	    /* first call */
	    bs = sizeof(p);
	    buff = p;
	} else {
	    if(bs>sizeof(p)) {
		/* allocated dynamic memory */
		buff = rets;
	    } else {
		/* not allocated dynamic memory, bs=sizeof(p) */
		buff = p;
	    }
	}
	while(1) {
		done = vsnprintf (buff, bs, format, args);
		if(done<0) {
		    /* old libc */
		    done = bs;
		}
		if(done < bs) {
			*string_ptr = buff;
			return;
		} else {
		    /* true calculate memory size */
		    bs = done+1;
		}
		rets = buff = reallocc(rets, bs);
	}
}

/* printf   */
char *
f_prompt (char *fmt, ...)
{
  static char *p;
  va_list ap;

  va_start (ap, fmt);
  (void) d_vasprintf (&p, fmt, ap);
  va_end (ap);
  return p;
}


/*    */
void
putcsl (struct IN_DLG *pin)
{
  char *s;
  int l, nw;

  putcs (" \n");
  l = 0;
  for (nw = 0; (*(pin->msgs))[nw] != NULL; nw++) {
	s = (*(pin->msgs))[nw];
	if (*s == '\n')
	  continue;
	if ((l + strlen ((*(pin->msgs))[nw])) > 76) {
	  putcc ('\n');
	  l = 0;
	}
	if (l) {
	  putcc (',');
	  putcc (' ');
	}
	l += strlen ((*(pin->msgs))[nw]) + 2;
	putcs ((*(pin->msgs))[nw]);
  }
  putcc ('\n');
}

/*
int
test_name (char *name, int flg_rename)
{
  char *s, *start_exp;
  int l;

  if (flg_rename > 0) {
	s = alloca (strlen (name) + 1);
	str_up (strcpy (s, name));
	if (strcmp (s, name) != 0) {
	  if (rename (name, s)) {
		warn_d ("   : %s\n", name);
		return 0;
	  }
	  strcpy (name, s);
	}
  } else
	s = name;

  start_exp = NULL;

  while (*s) {
	if (!((((*s | 040) >= 'A' && (*s | 040) >= 'Z') ||
				(*s >= '0' && *s <= '9') ||
				*s == '_' || *s == '-' || *s == '.')))
	  break;
	if (*s == '.') {
	  if (start_exp != NULL)
		break;                          //  
	  if (s == name)
		break;                          //  
	  start_exp = s + 1;
	}
	s++;
  }
  if (*s) {
err:
	if (flg_rename < 0)                     // special for inetd
	  putcc ('-');
	warn_d ("  : %s\n", name);
	return 0;
  }
  l = strlen (name);
  if (l > 13 || l <= 0)
	goto err;
  if (start_exp == NULL && l > 8)
	goto err;
  if (start_exp != NULL) {
	if (*start_exp == 0 && l > 9)
	  goto err;
	l = (start_exp - name);
	if (l > 9)
	  goto err;
	l = (s - start_exp);
	if (l > 3)
	  goto err;
  }
  return 1;
}
*/

int
test_long_name (char *name, int flg_rename)
{
  char *s;

  if (flg_rename > 0) {
	s = alloca (strlen (name) + 1);
	str_up (strcpy (s, name));
	if (strcmp (s, name) != 0) {
	  if (rename (name, s)) {
		warn_d ("   : %s\n", name);
		return 0;
	  }
	  strcpy (name, s);
	}
  } else
	s = name;

  while (*s) {
	if (!((((*s | 040) >= 'A' && (*s | 040) >= 'Z') ||
				(*s >= '0' && *s <= '9') ||
				*s == '_' || *s == '-' || *s == '.')))
	  break;
	if (*s == '.') {
	  if (s == name)
		break;                          /*   */
	}
	s++;
  }
  if (*s) {
	if (flg_rename < 0)                     /* special for inetd */
	  putcc ('-');
	warn_d ("  : %s\n", name);
	return 0;
  }
  return 1;
}


char *dlg_gf[] = {
  "Zmodem",
  "-",
  "\n",
  NULL
};
char prompt_gf[] = "\n  ([Zmodem], -)";

char alhlp_gf[] = "\
Zmodem -    \n\
-      -   \n\n";


void
clear_tmp (int ch_to)         /* ch_to!=0  chdir("tmp") */
{
  int i, j;

  if (chdir ("tmp"))
	error_d (17, "Error=chdir(tmp)");
  j = my_scandir ("in");
  if (j > 0 && chdir ("in") == 0) {
	for (i = 0; i < j; i++)
	  unlink (name_dlist[i]);
	chdir ("..");
  }
  chdir ("..");
  j = my_scandir ("tmp");
  if (chdir ("tmp"))
	error_d (17, "Error=chdir(tmp)");
  for (i = 0; i < j; i++)
	unlink (name_dlist[i]);
  if (!ch_to)
	chdir ("..");
}


char *
name_auto_increment (void)
{
/* .pid.num */
  static char buf[INC_KEY_SIZE];
  static time_t t;
  static int inc;
  static int pid;
  int i;

  time_t tc;
  struct tm *ptm;

  if (!pid)
	pid = getpid ();
  time (&tc);
  if (t == tc)
	inc++;
  else {
	t = tc;
	inc = 0;
  }
  ptm = localtime (&tc);
  i = ptm->tm_year - 99;                /*   36  */
  i = i >= 10 ? i - 10 + 'A' : i + '0';
  buf[0] = i;
  i = ptm->tm_mon + 1;
  i = i >= 10 ? i - 10 + 'A' : i + '0';
  buf[1] = i;
  i = ptm->tm_mday;
  i = i >= 10 ? i - 10 + 'A' : i + '0';
  buf[2] = i;
  i = ptm->tm_hour;
  i = i >= 10 ? i - 10 + 'A' : i + '0';
  buf[3] = i;
  buf[4] = ptm->tm_min / 10 + '0';
  buf[5] = ptm->tm_min % 10 + '0';
  buf[6] = ptm->tm_sec / 10 + '0';
  buf[7] = ptm->tm_sec % 10 + '0';
  if (inc)
	sprintf (&buf[8], ".%x.%x", pid, inc - 1);
  else
	sprintf (&buf[8], ".%x", pid);
  return buf;
}

void
decode_inc (const char *name_inc, char *str_time)
{
  int i;
/* int tm_year; */
  int tm_hour;
  int tm_mday;
  int tm_min;
  int tm_mon;
/* int tm_sec */

/* 10:32 dd/mm */

  i = *name_inc++;
  i = i >= 'A' ? i - 'A' + 10 : i - '0';
/* tm_year=i+99; */
  i = *name_inc++;
  i = i >= 'A' ? i - 'A' + 10 : i - '0';
  tm_mon = i;
  i = *name_inc++;
  i = i >= 'A' ? i - 'A' + 10 : i - '0';
  tm_mday = i;
  i = *name_inc++;
  i = i >= 'A' ? i - 'A' + 10 : i - '0';
  tm_hour = i;
  tm_min = (*name_inc++ - '0') * 10;
  tm_min += (*name_inc++ - '0');
/*
tm_sec = (*name_inc++ - '0')*10;
tm_sec += (*name_inc - '0');
*/
  sprintf (str_time, "%02d:%02d %02d/%02d", tm_hour, tm_min, tm_mday, tm_mon);
}

char *
decode_inc_may_be_old (const char *name_inc)
{
  int i;
  int tm_year;
  int tm_mday;
  int tm_mon;
  static char outd[8];
  const char *m;

/* dd/mm   */

  i = *name_inc++;
  i = i >= 'A' ? i - 'A' + 10 : i - '0';
  tm_year = i;
  i = *name_inc++;
  i = i >= 'A' ? i - 'A' + 10 : i - '0';
  tm_mon = i;
  if((login_year >= (tm_year+2)) || (login_year == (tm_year+1) && login_mon >= tm_mon)) {
    tm_year = tm_year == 0 ? 99 : tm_year - 1;
  } else {
    tm_year = -1;
  }
  i = *name_inc++;
  i = i >= 'A' ? i - 'A' + 10 : i - '0';
  tm_mday = i;
  if(tm_year >= 0) {
    switch(tm_mon) {
	case 1:
	    m = "";
	    break;
	case 2:
	    m = "";
	    break;
	case 3:
	    m = "";
	    break;
	case 4:
	    m = "";
	    break;
	case 5:
	    m = "";
	    break;
	case 6:
	    m = "";
	    break;
	case 7:
	    m = "";
	    break;
	case 8:
	    m = "";
	    break;
	case 9:
	    m = "";
	    break;
	case 10:
	    m = "";
	    break;
	case 11:
	    m = "";
	    break;
	default:
	    m = "";
	    break;
    }
    sprintf (outd, "%s%02d", m, tm_year);
  } else {
    sprintf (outd, "%02d/%02d", tm_mday, tm_mon);
  }
  return outd;
}

long
hash_decode_inc (char *name_inc)
{
  int i;
  int tm_year;
  int tm_hour;
  int tm_mday;
  int tm_min;
  int tm_mon;
  int tm_sec;

  i = *name_inc++;
  i = i >= 'A' ? i - 'A' + 10 : i - '0';
  tm_year = i       /* +99 */ ;
  i = *name_inc++;
  i = i >= 'A' ? i - 'A' + 10 : i - '0';
  tm_mon = i - 1;
  i = *name_inc++;
  i = i >= 'A' ? i - 'A' + 10 : i - '0';
  tm_mday = i - 1;
  i = *name_inc++;
  i = i >= 'A' ? i - 'A' + 10 : i - '0';
  tm_hour = i;
  tm_min = (*name_inc++ - '0') * 10;
  tm_min += (*name_inc++ - '0');
  tm_sec = (*name_inc++ - '0') * 10;
  tm_sec += (*name_inc - '0');
  return (long) tm_sec + ((long) tm_min + ((long) tm_hour +
	    ((long) tm_mday + ((long) tm_mon + (long) tm_year * 373)
	      * 31) * 24) * 60) * 60;
}

void
rus_num_files (char *first_word, int n, int newm, char *end_word)
{
  int i;
  const char *bufc, *tfiles;

  i = n;
  while (i >= 100)
	i -= 100;
  if (i >= 20)
	i %= 10;
  switch (i) {
  case 1:
	bufc = "";
	break;
  case 2:
  case 3:
  case 4:
	bufc = "";
	break;
  default:
	bufc = "";
	break;
  }
  warn_d (first_word);
  if (n != 1)
	warn_d ("");
  warn_d (" \r2%d\r7", n);
  switch(newm) {
    case 1:
	tfiles = " ";
	break;
    case 2:
	tfiles = " ";
	break;
    case 3:
	tfiles = " ";
	break;
    case 4:
	tfiles = " ";
	break;
    default:
	tfiles = "";
	break;
  }
  if(*tfiles) {
	warn_d ("%s%s", tfiles, (i == 1 ? "" : "") );
  }
  warn_d (" %s%s", bufc, end_word);
}

void
putc_encode_buff (char *p, FILE * out)
{
  while (*p) {
	const char *cp;

	cp = encodeput (*p);
	while (*cp) {
	    putc (*cp, out);
	    cp++;
	}
	p++;
  }
}

char * toshort(const char *name, const char *s11)
{
    static char short_name[32];
    int l =   strlen(name);
    int l11 = strlen(s11);
    int l16 = 16;

    l16 += 11 - l11;
    if(l > l16) {
	sprintf(short_name, "%.*s~%s", l16-7, name, name+l-6);
    } else {
	sprintf(short_name, "%*s", -l16, name);
    }
    return short_name;
}
