
#include "fun.h"
#include "keyboard.h"
#include <signal.h>
#include <sys/wait.h>

#define max(a,b) ((a)>(b)?(a):(b))

#define IDLE_RUN 180

static time_t set_alarm_com (void);
static volatile int flg_alarmed;

static void
handler_alarm_com (int ns)
{
  flg_alarmed++;
}

static time_t
set_alarm_com (void)
{
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  struct sigaction sa;
#endif
  signal (SIGALRM, handler_alarm_com);
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  sigaction (SIGALRM, (struct sigaction *) 0, &sa);
#ifdef SA_RESTART
  sa.sa_flags &= ~SA_RESTART;
#endif
#ifdef SA_INTERRUPT
  sa.sa_flags |= SA_INTERRUPT;
#endif
  sigaction (SIGALRM, &sa, (struct sigaction *) 0);
#endif
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  sigaction (SIGTERM, (struct sigaction *) 0, &sa);
#ifdef SA_RESTART
  sa.sa_flags &= ~SA_RESTART;
#endif
#ifdef SA_INTERRUPT
  sa.sa_flags |= SA_INTERRUPT;
#endif
  sigaction (SIGTERM, &sa, (struct sigaction *) 0);
#endif
#if defined(SA_RESTART) || defined(SA_INTERRUPT)
  sigaction (SIGHUP, (struct sigaction *) 0, &sa);
#ifdef SA_RESTART
  sa.sa_flags &= ~SA_RESTART;
#endif
#ifdef SA_INTERRUPT
  sa.sa_flags |= SA_INTERRUPT;
#endif
  sigaction (SIGHUP, &sa, (struct sigaction *) 0);
#endif
  flg_alarmed = 0;
  alarm (IDLE_RUN);				/* 180 sec - retrain */
  return time (0) + IDLE_RUN;
}

static void
unset_alarm_com (void)
{
  alarm (0);
  signal (SIGALRM, SIG_IGN);
}


void
run_and_check_idle (char *command)
{
  struct stat statb;
  time_t idle, start;
  int pid, status;
  char *argv[4];

  start = time (0);
  switch (pid = fork ()) {
  case -1:
	error_d (1, "    .");
  case 0:
	signal (SIGTERM, SIG_DFL);
	signal (SIGHUP, SIG_DFL);
	signal (SIGALRM, SIG_IGN);
	scanclose ();
	argv[0] = "sh";
	argv[1] = "-c";
	argv[2] = command;
	argv[3] = 0;
	execv ("/bin/sh", argv);
	error_d (127, "exec (%s) failed", command);
  default:
	start = set_alarm_com ();
	for (;;) {
	  if (waitpid (pid, &status, 0) == -1) {
		if (flg_alarmed != 0 || if_killed () != 0) {
		  fstat (0, &statb);
		  idle = start - max (statb.st_atime, statb.st_mtime);
		  if (idle >= IDLE_RUN || if_killed () != 0) {
			kill (pid, SIGTERM);
			put_log ("exec (%s) killed\n", command);
			pid = 0;
			break;
		  }
		}
		start = set_alarm_com ();
	  } else
		break;
	}
	unset_alarm_com ();
	scaninit ();
	if (pid == 0)
	  putcs ("  \n");
  }
}
