#define DVERSION " 1.9.15"

#include "keyboard.h"
#include "fun.h"
#include "db.h"
#include "ex.h"
#include "hlps.h"

#include <pwd.h>

#ifdef linux
#include "setproctitle.h"
#endif

static void
test_and_create_dirs (char *dir, int flg_create)
{
  int fd_dir = open(".", O_RDONLY, 0);

  if (fd_dir < 0)
	error_d (2, "Error=open(.)");
  if (chdir (dir) < 0) {
	if (flg_create == 0)
	  error_d (2, "Error=chdir(%s)", dir);
	if (mkdir (dir, 0770))
	  error_d (2, "Error: Can`t create %s directory", dir);
  } else {
	fchdir (fd_dir);
	close(fd_dir);
  }
}

static char *dlg_com[] = {
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "-",
  "\n",
  NULL
};

static char prompt[] =
  "\n, , , , , ..., , ?";
static const char alhlp[] = "\
     -      \n\
  -   \n\
       -        \n\
  -   \n\
        -   \n\
         -   telnet  Point-to-Point \n\
         -   \n";

static int
usage (char *prgname)
{
  printf ("\
Usage %s [-c] or [-m] [-h number]\n\
\t-c create new %s directory tree and user `%s',\n\
\t   or login to user `%s'\n\
\t-i run from inetd as service\n\
\t-m run as mailfilter for mailgate\n\
\t-d dump users database to stdout\n\
\t-h host direction number\n\
\t-l load predumped and edited users file from stdin\n",
			      prgname, dir_users, Adm, Adm);
  return 1;
}

int
main (int argc, char **argv)
{
  struct IN_DLG in_dlg;
  char *s;
  char *name;

  int flg_create = 0;
  int flg_mail = 0;
  int flg_dump_load = 0;

  int flg_ext = 0;
  int flg_ext_srv = 0;
  int opt;

  int sg = geteuid ();
  struct passwd *pwd = getpwuid (sg);

  for (s = name = argv[0]; *s != '\0';)
	if (*s++ == '/')
	  name = s;
  if (*name == '-' && *(name + 1) != 0) /* -shell */
	name++;
  if(strcmp(name, "rz") == 0 || strcmp(name, "sz") == 0) {
    return(lsz_main(argc, argv));
  }
  prgname = strcpym (name);
  pu_cur.l = 'E'; /*   -      */
  s = prgname+strlen(prgname);
  if(s==prgname)
	error_d (4, "Error: empty programm name");
  s--;
  opt = 0;
  /*         */
  while( *s >= 0 && *s <= '9' && s >= prgname) {
    opt++;
    s--;
  }
  if(opt>0) {
    s += opt;
    host_direction = g_int(&s);
    if(host_direction < 0 || host_direction > MAX_DIRECTIONS)
	host_direction = 0;
  }

  /* Parse any options */
  while ((opt=getopt(argc, argv, "cimdlseSh:")) != -1) {
	switch(opt) {
		case 'c':
			flg_create++;
			break;
		case 'i':
			flg_inetd++;
			break;
		case 'm':
			flg_mail++;
			break;
		case 'd':
		case 'l':
			if(flg_dump_load)
			    return usage (prgname);
			flg_dump_load = opt;
			break;
		case 's':
			flg_test++;
			break;
		case 'e':
			flg_ext++;
			break;
		case 'S':
			flg_ext_srv++;
			break;
		case 'h':
			host_direction = g_int(&optarg);
			if(*optarg != 0 || host_direction < 0 ||
					host_direction > MAX_DIRECTIONS) {
			    printf("Bad host direction number\n");
			    return usage (prgname);
			}
			break;
		default:
			return usage (prgname);
	}
  }

  if (sg == 0)
	error_d (2, "Error: Can`t running with uid = 0!");
  if (pwd == NULL)
	error_d (2, "Error: Can`t get user information for uid=%d", sg);

  for (s = name = pwd->pw_shell; *s != '\0';)
	if (*s++ == '/')
	  name = s;

  if (strcmp (name, prgname) != 0 && flg_test==0)
	error_d (2, "Error: shell for uid=%d is not current programm", sg);

  if (flg_test==0) {
	test_and_create_dirs (pwd->pw_dir, 0);
	chdir (pwd->pw_dir);
  }
  setlinebuf (stdout);

  test_and_create_dirs (dir_users, flg_create);
  test_and_create_dirs (dir_news, flg_create);
  test_and_create_dirs (dir_arch, flg_create);
  test_and_create_dirs (f_prompt ("%s/%s", dir_arch, dir_arch_no_arch),
						flg_create);
  test_and_create_dirs (dir_auto, flg_create);
  if (chdir (dir_users) < 0)
	error_d (2, "Error=chdir(%s)", dir_users);
  if (flg_mail)
	return main_mailgate ();
  if (flg_dump_load || flg_ext || flg_ext_srv) {
/*  /       koi8 */
	pu_cur.l = 'K';
	strcpy(pu_cur.user, Adm);
  }
  if (chdir (Adm) < 0) {
	if (flg_create == 0)
	  error_d (11, "Error=chdir(%s/%s)", dir_users, Adm);
	mkdir (Adm, 0770);
	chdir (Adm);
	if (create_empty (0))
	  return 3;
	chdir (Adm);
	if (add_user (Adm) == 0)
	  error_d (3, "    `'");
  }
  test_and_create_dirs (dir_ext, flg_create);
  if (flg_dump_load == 'd')
	return dump_abon ();
  if (flg_dump_load == 'l')
	return load_abon_from_stdin ();
  if(flg_ext)
	return d_cle();
  if(flg_ext_srv)
	return d_sle();

  scaninit ();
  putcs_mnogo_in_file ("../../issue.top");
  sg = login (flg_create);
  warn_d ("\n  . " DVERSION "\n");
  warn_d ("  . mailto: dzo@simtreas.ru\n\n");
  log_login (sg);

#ifdef linux
  initproctitle (argc, argv);
  setproctitle (hostname, pu_cur.user);
#endif

  putcs_mnogo_in_file ("../../issue");
  putcs_mnogo_in_file ("issue");
  putcs_mnogo_in_file ("issue.1");
  unlink ("issue.1");
  if (sg != 0)
	start_gate (sg);
  if(putcs_mnogo_in_file ("lock") > 0)
    return 0;
  in_dlg.msgs = &dlg_com;
  in_dlg.prompt = prompt;
  in_dlg.alhlp = alhlp;
  in_dlg.hlps = &main_dlg_hlps;
  in_dlg.zvezda = 0;

  get_ouf ();
  sg = 0;
  for (;;) {
	scan_aliases_change ();
	scan_new_reg ();
	scan_new_files ();
	scan_new_konf ();
	if (sg == 0) {
	  flg_auto_cr = 1;
	  sg = scan_input (&in_dlg);
	  flg_auto_cr = 0;
	}
	switch (sg) {
	case 1:
	  sg = dlg_abon ();
	  switch (sg) {
	  case -1:
		chdir ("../../");
		scanclose ();
		system (pwd->pw_shell);
		scaninit ();
		chdir (f_prompt ("%s/%s", dir_users, pu_cur.user));
		sg = 1;
		break;
	  case 1:
		goto end;
	  default:
		break;
	  }
	  break;
	case 2:
	  sg = dlg_konf ();
	  switch (sg) {
	  case 100:
		goto end;
	  default:
		break;
	  }
	  break;
	case 3:
	  if(mode_nolist) {
	    warn_d ("   `'  \n");
	    sg = 5;
	    break;
	  }
	  sg = dlg_list ();
	  switch (sg) {
	  case 100:
		goto end;
	  default:
		break;
	  }
	  break;
	case 4:
	  sg = dlg_reg ();
	  switch (sg) {
	  case 100:
		goto end;
	  default:
		break;
	  }
	  break;
	case 5:
	  if (test_big_reg ()) {
		putcs ("    .\n");
		sg = 0;
	  } else {
		sg = dlg_files ();
		switch (sg) {
		case 100:
		  goto end;
		default:
		  break;
		}
	  }
	  break;
	case 6:
	  sg = dlg_gates ();
	  switch (sg) {
	  case 100:
		goto end;
	  default:
		break;
	  }
	  break;
	case 7:
	  goto end;
	case 8:
	case 9:
	  sg = 0;
	  break;
	}
  }
end:
  delete_reg ();
  delete_in ();
  delete_out ();
  return 0;
}
