#include "fun.h"
#include "keyboard.h"
#include "code.h"
#include <stdarg.h>

static void ErrorMsgAndDie (int exit_code, const char *s, va_list p)
  __attribute__ ((noreturn));

static void
ErrorMsgAndDie (int exit_code, const char *s, va_list p)
{

  char buff[128];

  putcs ("\r1");
  vsnprintf (buff, sizeof (buff), s, p);
  putcs (buff);
  putcs ("\r7");
  putcc ('\n');
  va_end (p);
  fflush(stdout);
  sleep(1);
  exit (exit_code);
}

void
error_d (int exit_code, const char *s, ...)
{
  va_list p;

  va_start (p, s);
  ErrorMsgAndDie (exit_code, s, p);
}

void
error_prot (int exit_code, const char *s, ...)
{
  va_list p;

  va_start (p, s);
  putchar (*s++);
  ErrorMsgAndDie (exit_code, s, p);
}

void
warn_d (const char *s, ...)
{
  va_list p;
  char buff[128];

  va_start (p, s);
  vsnprintf (buff, sizeof (buff), s, p);
  putcs (buff);
  va_end (p);
}

#include <netinet/in.h>
#include <netdb.h>
#include <sys/socket.h>
#include <arpa/inet.h>

static char *myClientHost = 0;

/* TCP/IP return name from socket */

static char *
tcp_name (struct sockaddr_in *sin)
{
  char tmp[512];
  struct hostent *he;
  if (!(he = gethostbyaddr ((char *) &sin->sin_addr,
		      sizeof (struct in_addr), sin->sin_family)))
	  sprintf (tmp, "[%s]", inet_ntoa (sin->sin_addr));
  else
	sprintf (tmp, "[%s]", he->h_name);
  return strdup (tmp);
}


char *
tcp_clienthost (void)
{
  if (!myClientHost) {
	struct sockaddr_in sin;
	int sinlen = sizeof (struct sockaddr_in);
	/* get stdin's peer name */
	if (getpeername (0, (struct sockaddr *) &sin, (void *) &sinlen) == 0)
	  myClientHost = tcp_name (&sin);
	else {
	  myClientHost = getenv ("FROMHOST");
	  if (myClientHost == 0) {
		myClientHost = ttyname(0);
		if(myClientHost) {
		    if(strncmp(myClientHost, "/dev/", 5)==0)
			myClientHost += 5;
		} else {
		    myClientHost = "[UNKNOWN]";
		}
	  } else {
		myClientHost = strcpym (myClientHost);
	  }
	}
  }
  return myClientHost;
}
