#include "fun.h"
#include "keyboard.h"
#include "db.h"
#include "ex.h"
#include <gdbm.h>

static char default_user_rem[] = " ";
static char default_ADM_rem[] = "";


/*
  
key = USER%MAXPU_LENs
 L:
	- K koi8
	- A alt
	- W win1251
	- E translit
	- 7- -  ansi- 
 GATE  -   (telnet  ppp) %4d
 GRANTS:
	- GRANTS_ADM administrator
	- GRANTS_NO_GRANTS default
	- bits see fun.h
 KEEPING -     %5d
 DIRECT  -   %4d
 A(access flag):
       - A access
       - D deny
       - a access, no detect idle
       - d deny, no detect idle for change to access
       - e external host
 ARCH(     -)
	- '1'  '0'
       (  -)
	- ARCH|2 -       
	- ARCH|4 -         
	- ARCH|8 -        
 GROUP=%MAXPU_LENs
 PASS=%MAXPU_LENs
	- password=* - any
 REM=%MAXPU_REM_LENs
 naliases= %d (s) (  )
 aliases = N%s
*/


void
form_r_pu (const char *recu, char *rec, int size_rec, struct PU *pu)
{
  int msk_flg;

  strcpy (pu->user, recu);
  pu->l = *rec;
  if (*rec < 0) {
	pu->l &= 0177;
	pu->flg_nocolor = 1;
  } else
	pu->flg_nocolor = 0;
  rec++;
  pu->gate = atoi (rec);
  rec += 4;
  pu->priv = *rec++;
  /* old database compatibility */
  if ((pu->priv & GRANTS_NO_GRANTS))
	pu->priv = GRANTS_NO_GRANTS;
  if (strcmp (pu->user, Adm) == 0)
	pu->priv = GRANTS_ADM;          /* end old database compatibility */
  else                                  /* to (unsigned char) */
	pu->priv &= GRANTS_ADM;
  pu->time_keeping_files = atoi (rec);
  rec += 5;
  pu->ex_direction = atoi (rec);
  rec += 4;
  pu->af = *rec++;
  msk_flg = *rec++;
  pu->flg_ar = msk_flg & 1;
  pu->flg_gmf = (msk_flg - '0') & ~1;
  strcpy (pu->grp, rec);
  rec += strlen (rec) + 1;
  strcpy (pu->pass, rec);
  rec += strlen (rec) + 1;
  strcpy (pu->rem, rec);
  rec += strlen (rec) + 1;
  if (size_rec > UBD_REC_LEN) {
	pu->naliases = atoi (rec);
	rec += strlen (rec) + 1;
	pu->aliases = (char **) mallocc (pu->naliases * sizeof (char *));
	for (msk_flg = 0; msk_flg < pu->naliases; msk_flg++) {
	  pu->aliases[msk_flg] = strcpym (rec);
	  rec += strlen (rec) + 1;
	}
  } else {
	pu->naliases = 0;
	pu->aliases  = 0;
  }
}


char *
form_w_pu (struct PU *pu, int *size_rec)
{
  char *rec;
  char *s;
  int i, sz;
  char nas[32];

  sz = UBD_REC_LEN;
  if (pu->naliases > 0) {
	sprintf (nas, "%d", pu->naliases);
	sz += strlen (nas) + 1;
	for (i = 0; i < pu->naliases; i++)
	  sz += strlen (pu->aliases[i]) + 1;
  }
  rec = mallocc (sz);
  s = rec;
  *s = pu->l;
  if (pu->flg_nocolor)
	*s |= 0200;
  s++;
  sprintf (s, "%-4d", pu->gate);
  s += 4;
  *s++ = pu->priv;
  sprintf (s, "%-5d", pu->time_keeping_files);
  s += 5;
  sprintf (s, "%-4d", pu->ex_direction);
  s += 4;
  *s++ = pu->af;
  *s = pu->flg_ar ? '1' : '0';
  *s |= pu->flg_gmf;
  s++;
  strcpy (s, pu->grp);
  s += strlen (s) + 1;
  strcpy (s, pu->pass);
  s += strlen (s) + 1;
  strcpy (s, pu->rem);
  s += strlen (s) + 1;
  if (pu->naliases > 0) {
	strcpy (s, nas);
	s += strlen (s) + 1;
	for (i = 0; i < pu->naliases; i++) {
	  strcpy (s, pu->aliases[i]);
	  s += strlen (s) + 1;
	}
  }
  *size_rec = sz;
  return rec;
}

/*      
        .  :
     <= ' '          '`'  (     +'@') (0->'@' - ' '->'`')
   '`'                  '`'  '~'
   127                  '`'  '?'
   0x9b                 '`'  'e'
   0xff                 '`'  'z'
*/
char *
form_code_ex(unsigned char *s0, int size_rec)
{
    int i, bytes;
    unsigned char *sret;

    for(bytes=i=0; i<size_rec; i++,bytes++)
	if(s0[i]<=' ' ||  s0[i]==127 || s0[i]==0x9b || s0[i]==0xff)
	    bytes++;
    sret = mallocc (bytes+1);
    for(bytes=i=0; i<size_rec; i++) {
	if(s0[i]<=' ' ||  s0[i]==127 || s0[i]==0x9b || s0[i]==0xff)
	    sret[bytes++] = '`';
	else
	    sret[bytes++] = s0[i];
	if(s0[i]<=' ')
	    sret[bytes++] = s0[i] + '@';
	else if(s0[i] == '`')
	    sret[bytes++] = '~';
	else if(s0[i] == 127)
	    sret[bytes++] = '?';
	else if(s0[i] ==  0x9b)
	    sret[bytes++] = 'e';
	else if(s0[i] ==  0xff)
	    sret[bytes++] = 'z';
    }
    sret[bytes] = 0;
    return sret;
}

/*        */
char *
form_w_pu_ex(struct PU *pu)
{
    unsigned char *s0;
    int size_rec;

    s0 = (unsigned char *)form_w_pu(pu, &size_rec);
    return form_code_ex(s0, size_rec);
}


/*  -    form_code_ex() */
char *
form_decode_ex(const char *scode, int *sb)
{
    int bytes;
    char *s0, *sret;

    s0 = (char *)scode;
    bytes = 0;
    while(*scode) {
	if(*scode == '`')
	    scode++;
	if(*scode == 0) {
	    /*      */
	    return 0;
	}
	scode++;
	bytes++;
    }
    sret = mallocc (bytes);
    scode = s0;
    s0 = sret;
    while(*scode) {
	if(*scode == '`') {
	    if(*++scode == '~')
		*s0 = '`';
	    else if(*scode == '?')
		*s0 = 127;
	    else if(*scode == 'e')
		*s0 = 0x9b;
	    else if(*scode == 'z')
		*s0 = 0xff;
	    else
		*s0 =  *scode - '@';
	    s0++;
	} else {
	    *s0++ =  *scode;
	}
	scode++;
    }
    *sb = bytes;
    return sret;
}

char *
form_w_u (char *recu)
{
  static char rec[LEN_UDB_KEY];
  memset (rec, 0, LEN_UDB_KEY);
  return strcpy (rec, recu);
}


struct PU pu_cur;
struct PU *all_pu;
int n_all_pu;


int
get_pu (struct PU *pu, char *u)
{                                         /*      */
  int i;
  char usr_and_host[MAXPU_LEN + MAXPU_LEN + 2];
  char *su;

  su = strncpy (usr_and_host, u, sizeof (usr_and_host));
  su[sizeof (usr_and_host) - 1] = 0;
  su = strchr(usr_and_host, '@');
  if(su)
    *su = 0;
  if (stat_db_chg (N_DB_USR)) {
	free0 (&all_pu);
	n_all_pu = all_abonents (&all_pu);
  }
  for (i = 0; i < n_all_pu; i++)
	if (strcmp (usr_and_host, all_pu[i].user) == 0)
	  break;
  if (i == n_all_pu) {
	int n;
	for (i = 0; i < n_all_pu; i++) {
	  for (n = 0; n < all_pu[i].naliases; n++)
		if (strcmp (usr_and_host, all_pu[i].aliases[n]) == 0)
		  break;
	  if (n < all_pu[i].naliases)
		break;
	}
  }
  if (i != n_all_pu) {
	*pu = all_pu[i];
	return 1;
  }
  return 0;
}


/*     */
void
set_pu (struct PU *pu)
{
  int i;

  for (i = 0; i < n_all_pu; i++)
	if (strcmp (pu->user, all_pu[i].user) == 0)
	  break;
  if (i != n_all_pu)
	all_pu[i] = *pu;
}



int
add_user (char *user)
{
  struct PU pu;
  int a;

  memset (&pu, 0, sizeof (struct PU));
  strcpy (pu.user, user);
  strcpy (pu.pass, "*");                /*   */
  pu.l = 'K';                           /* 8 */
/*     */
  a = strcmp (user, Adm);
  pu.priv = a ? GRANTS_NO_GRANTS : GRANTS_ADM;
  pu.af = 'A';            /*  ,   */
  strcpy (pu.grp, "ALL");
  strcpy (pu.rem, a ? default_user_rem : default_ADM_rem);
/*   , ,  ,  
     ,  =0 */
  n_all_pu++;
  all_pu = reallocc (all_pu, n_all_pu * sizeof (struct PU));
  all_pu[n_all_pu - 1] = pu;
  return db_add_user (&pu, 0);
}

static char name_user_db[] = "../users.db";
static char name_news_db[] = "../news.db";

static char name_reg_db[] = "reg.db";
static char name_inf_db[] = "in.db";
static char name_ouf_db[] = "out.db";
static char name_nu_db[] = "news.db";

char *db_names[N_DB_NUM] = {
  name_user_db,
  name_news_db,
  name_reg_db, name_inf_db, name_ouf_db, name_nu_db, buf_news_empty
};

/*       */

int
create_empty (int start_bd)
{
  int i, ret;
  GDBM_FILE dbf;
  datum key_num, content_num;

  ret = 0;
  sleep_kill_start ();
  for (i = start_bd; i < N_DB_NAMES; i++) {
	dbf = gdbm_open (db_names[i], 512, GDBM_WRCREAT, 0660, 0);
	if (dbf == NULL) {
	  warn_d ("     %s  \n",
			  db_names[i]);
	  ret = 1;
	} else {
	  if (i == N_DB_USR) {
		key_num.dptr = "*";
		key_num.dsize = 2;
		content_num.dptr = "0";
		content_num.dsize = 2;
		gdbm_store (dbf, key_num, content_num, GDBM_INSERT);
	  }
	  gdbm_close (dbf);
	}
  }
  if (mkdir ("out", 0770) || mkdir ("list", 0770) || mkdir ("tmp", 0770)) {
	putcs ("      \n");
	ret = 1;
  }
  if(start_bd==0) {
    create_new_direction(0);
  }
  sleep_kill_stop ();
  chdir ("..");
  return ret;
}


/*         */
void
unlink_all_db (void)
{
  int i, k;

  static char *dirs[] = { "out", "list", "tmp", 0 };

  for (i = START_USERS_DB; i < N_DB_NAMES; i++)
	if (unlink (db_names[i]))
	  putcs ("     \n");
  for (k = 0; dirs[k] != 0; k++) {
	i = my_scandir (dirs[k]);
	while (i) {
	  i--;
	  unlink (f_prompt ("%s/%s", dirs[k], name_dlist[i]));
	}
	rmdir (dirs[k]);
  }
}

static int
qsort_compar_users (const void *p1, const void *p2)
{
  struct PU *inf_s1;
  struct PU *inf_s2;
  inf_s1 = (struct PU *) p1;
  inf_s2 = (struct PU *) p2;
  return strcmp (inf_s1->user, inf_s2->user);
}

char *
sort_db_abonents (void)
{
  char *flg_no_cur_grp;
  int n = n_all_pu;
  int i;
  struct PU pu;

  if (n == 0)
	return 0;
/*      */
  if (get_pu (&pu, pu_cur.user) == 0 || n_all_pu == 0)
	error_d (71, "    ");
  n = n_all_pu;
  qsort (all_pu, n, sizeof (struct PU), qsort_compar_users);
  flg_no_cur_grp = mallocc (n);
  if (strcmp (pu_cur.grp, "ALL"))
	for (i = 0; i < n; i++)
	  if (strcmp (all_pu[i].grp, "ALL") != 0
		  && strcmp (all_pu[i].grp, pu_cur.grp) != 0)
		flg_no_cur_grp[i] = 1;
  return flg_no_cur_grp;
}

struct GROUP_LIST
{
  char *gname;
  struct GROUP_LIST *next;
};

int
get_all_group (char ***groupsl)
{
  struct GROUP_LIST *top = 0;
  struct GROUP_LIST *cur;
  int n = n_all_pu;
  int i;
  struct GROUP_LIST *prev = NULL;
  char **pg;

  for (i = 0; i < n; i++) {
	for (cur = top; cur; cur = cur->next)
	  if (!strcmp (all_pu[i].grp, cur->gname))
		break;
	if (cur == 0) {
	  cur = (struct GROUP_LIST *) mallocc (sizeof (struct GROUP_LIST));
	  cur->gname = strcpym (all_pu[i].grp);
	  cur->next = 0;
	  if (top)
		prev->next = cur;
	  else
		top = cur;
	  prev = cur;
	}
  }
  n = 0;
  for (cur = top; cur; cur = cur->next)
	n++;
  *groupsl = mallocc (n * sizeof (char *));
  pg = *groupsl;
  n = 0;
  for (cur = top; cur != 0;) {
	pg[n++] = cur->gname;
	prev = cur;
	cur = cur->next;
	free (prev);
  }
  return n;
}

int have_ext_users(void)
{
    int i;

    if (stat_db_chg (N_DB_USR)) {
	free0 (&all_pu);
	n_all_pu = all_abonents (&all_pu);
      }
    for (i = 0; i < n_all_pu; i++)
	if (all_pu[i].ex_direction!=host_direction)
	    return 1;
    return 0;
}

int have_ex_direction(int direction)
{
    int i;

    if(direction==host_direction)    /*   */
	return 1;
    if(have_ext_users()) {
	for (i = 0; i < n_all_pu; i++)
	    if (all_pu[i].ex_direction==direction)
		return 1;
    }
    return 0;
}
