/*     core */
//#define DEBUG

#define CORE_SIZE 1024000


#include "keyboard.h"

#include <signal.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>

#define STDIN 0         /* file descriptor for standard input */

#include <termios.h>
#ifdef __FreeBSD__
#define termio termios
#endif  /* __FreeBSD__ */

			/* super marazmatic non statdart. DZO */
#ifdef TCSETA
#define TSETA TCSETA
#else
#ifdef TIOCSETA
#define TSETA TIOCSETA
#endif /* TIOCSETA */
#endif /* TCSETA */

#ifdef TCGETA
#define TGETA TCGETA
#else
#ifdef TIOCGETA
#define TGETA TIOCGETA
#endif /* TIOCGETA */
#endif /* TCGETA */


static struct termio tnew, told;
static int keybrd_inited;

void
scanclose (void)
{
  if (keybrd_inited) {
	if (flg_inetd == 0) {
	  ioctl (0, TSETA, &told);
	  signal (SIGILL, SIG_DFL);
	  signal (SIGSEGV, SIG_DFL);
	  signal (SIGBUS, SIG_DFL);
	}
	keybrd_inited = 0;
  }
}

static void
sig_catcher (int sig)
{
  if (keybrd_inited) {
	scanclose ();
	put_log ("Detect sig_catcher\n");
	kill (getpid (), 9);
  }
}

#ifdef DEBUG
#include <sys/resource.h>
#endif

void
scaninit (void)
{
#ifdef DEBUG
  struct rlimit rlim;

  getrlimit (RLIMIT_CORE, &rlim);
  if (rlim.rlim_max < CORE_SIZE)
	rlim.rlim_cur = rlim.rlim_max;
  else
	rlim.rlim_cur = CORE_SIZE;
#endif
  if (!keybrd_inited) {
	atexit (scanclose);
	set_hup_term ();
	debug1 (setrlimit (RLIMIT_CORE, &rlim));
	if (flg_inetd == 0) {           /* no set terminals mode to socket */
	  debug2 (signal (SIGILL, sig_catcher));
	  debug2 (signal (SIGSEGV, sig_catcher));
	  debug2 (signal (SIGBUS, sig_catcher));
	  ioctl (0, TGETA, &told);
	  tnew = told;
	  tnew.c_cc[VMIN] = 1;
	  tnew.c_cc[VTIME] = 0;
	  tnew.c_lflag &= ~(ISIG | ECHO | ECHOE | ECHOK | ICANON);
	  tnew.c_iflag &=
		~(IXON | ICRNL | INLCR | PARMRK | INPCK | ISTRIP | IGNCR |
		  IXANY | IXOFF);
	}
	keybrd_inited = 1;
  }
  if (flg_inetd == 0)
	ioctl (0, TSETA, &tnew);        /* if(keybrd_inited) -> reset tty */
}

long int bytes_in;
int flg_auto_cr;

int
getch (int detect_idle)
{
  unsigned char c;
  struct timeval tv;
  fd_set readfds;

  if(flg_auto_cr < 0) {
      flg_auto_cr = -flg_auto_cr;
      bytes_in++;
      return '\n';
  }

  for (;;) {
	tv.tv_sec = ALARM_KILL;
	if (detect_idle)
	  tv.tv_sec *= detect_idle == 0 ? 1 : detect_idle;
	tv.tv_usec = 0;

	FD_ZERO (&readfds);
	FD_SET (STDIN, &readfds);
	select (STDIN + 1, &readfds, NULL, NULL, &tv);
	if (FD_ISSET (STDIN, &readfds)) {
	  if (read (0, &c, 1) <= 0) {   /* signal */
		/* fd 0 closed */
		if (errno == ENOENT || errno == EINVAL || errno == ECONNRESET
			|| errno == EPIPE)
		  kill (getpid (), SIGHUP);
		return 0;
	  }
	  bytes_in++;
	  return c;
	} else {
	  if (detect_idle) {
		handler_alarm ();
		return 0;
	  } else
		detect_single_mode ();
		if(flg_auto_cr) {
		    flg_auto_cr = -flg_auto_cr;
		    bytes_in++;
		    return '-';
		}
	}
  }
}

/*
int rdchk(int fd) {
int rd;
ioctl(fd,FIONREAD,&rd);
return rd;
}

int kbhit(void) { return rdchk(0); }
*/
