/*
	Copyright (c) 1997 Magosnyi rpd

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

	Modified for modern gcc by Vladimir Oleynik (C) 2002:
	 Remove warnings, all static declared
*/

#define _GNU_SOURCE

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <fcntl.h>


static const char *applet_name;


static void perror_msg_and_die(int ec, const char *s, ...)
		__attribute__ ((noreturn, format (printf, 2, 3)));

static void perror_msg_and_die(int ec, const char *s, ...)
{
	va_list p;
	int e = errno;

	va_start(p, s);
	fprintf( stderr, "%s: ", applet_name );
	vfprintf(stderr, s, p);
	errno = e;
	fprintf( stderr, ": %m\n");
	va_end(p);
	exit(ec);
}


int main(int argc,char *argv[])
{
	int fd;
	char **args;
	int i, tty_fd, err_fd;
	const char *s;

	applet_name = argv[0];

	for (s = applet_name; *s != '\0';) {
		if (*s++ == '/')
			applet_name = s;
	}

	if(argc <= 1) {
		fprintf(stderr, "Usage: %s cmd [arg]\n", applet_name);
		return 1;
	}
	if(NULL==(args=malloc((argc)*sizeof(char *))))
		perror_msg_and_die(3, "Can't allocate memory to write args");
	for(i = 1; i < argc; i++)
		args[i-1] = argv[i];
	args[argc-1] = NULL;    /* End of params */

	fclose(stdin);
	fd = open("/dev/ptmx", 2);
	if (fd >= 0) {
		grantpt(fd);
		unlockpt(fd);
		fprintf(stdout, "%s\n", ptsname(fd));
	} else {
		perror_msg_and_die(4, "Can't find free ptty");
	}

	err_fd = dup(fileno(stderr));
	fcntl(err_fd, F_SETFD, FD_CLOEXEC);

	fclose(stdout);
	fclose(stderr);

	if( (dup( fd )) != 1 )
		perror_msg_and_die(2, "Can't dup to stdout");
	if( (dup( fd )) != 2 )
		perror_msg_and_die(2, "Can't dup to stderr");

	if( (tty_fd = open("/dev/tty", O_RDWR)) > 0 ) {
		ioctl(tty_fd, TIOCNOTTY);
		close(tty_fd);
	}
	stderr = fdopen(err_fd, "w");

	i = vfork();
	if(i < 0)
		perror_msg_and_die(6, "Can't vfork");
	if(i) {
		return 0;
	} else {
		execvp(argv[1], args);
		perror_msg_and_die(5, "Can't exec redirected process");
	}
}
